//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct CNContainerType(pub NSInteger);
impl CNContainerType {
    #[doc(alias = "CNContainerTypeUnassigned")]
    pub const Unassigned: Self = Self(0);
    #[doc(alias = "CNContainerTypeLocal")]
    pub const Local: Self = Self(1);
    #[doc(alias = "CNContainerTypeExchange")]
    pub const Exchange: Self = Self(2);
    #[doc(alias = "CNContainerTypeCardDAV")]
    pub const CardDAV: Self = Self(3);
}

unsafe impl Encode for CNContainerType {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for CNContainerType {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct CNContainer;

    unsafe impl ClassType for CNContainer {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSCoding for CNContainer {}

unsafe impl NSCopying for CNContainer {}

unsafe impl NSObjectProtocol for CNContainer {}

unsafe impl NSSecureCoding for CNContainer {}

extern_methods!(
    unsafe impl CNContainer {
        #[method_id(@__retain_semantics Other identifier)]
        pub unsafe fn identifier(&self) -> Retained<NSString>;

        #[method_id(@__retain_semantics Other name)]
        pub unsafe fn name(&self) -> Retained<NSString>;

        #[method(type)]
        pub unsafe fn r#type(&self) -> CNContainerType;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl CNContainer {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);

extern "C" {
    pub static CNContainerIdentifierKey: &'static NSString;
}

extern "C" {
    pub static CNContainerNameKey: &'static NSString;
}

extern "C" {
    pub static CNContainerTypeKey: &'static NSString;
}
