//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ptr::NonNull;
use objc2_core_foundation::*;

use crate::*;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgexrtonemappinggammadefog?language=objc)
    pub static kCGEXRToneMappingGammaDefog: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgexrtonemappinggammaexposure?language=objc)
    pub static kCGEXRToneMappingGammaExposure: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgexrtonemappinggammakneelow?language=objc)
    pub static kCGEXRToneMappingGammaKneeLow: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgexrtonemappinggammakneehigh?language=objc)
    pub static kCGEXRToneMappingGammaKneeHigh: &'static CFString;
}

#[inline]
pub extern "C-unwind" fn CGEXRToneMappingGammaGetDefaultOptions() -> CFRetained<CFDictionary> {
    extern "C-unwind" {
        fn CGEXRToneMappingGammaGetDefaultOptions() -> Option<NonNull<CFDictionary>>;
    }
    let ret = unsafe { CGEXRToneMappingGammaGetDefaultOptions() };
    let ret = ret.expect("function was marked as returning non-null, but actually returned NULL");
    unsafe { CFRetained::retain(ret) }
}
