// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::GStr;

#[doc(alias = "GTK_LEVEL_BAR_OFFSET_FULL")]
pub static LEVEL_BAR_OFFSET_FULL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_LEVEL_BAR_OFFSET_FULL) };
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_HIGH")]
pub static LEVEL_BAR_OFFSET_HIGH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_LEVEL_BAR_OFFSET_HIGH) };
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_LOW")]
pub static LEVEL_BAR_OFFSET_LOW: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_LEVEL_BAR_OFFSET_LOW) };
#[doc(alias = "GTK_PAPER_NAME_A3")]
pub static PAPER_NAME_A3: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_A3) };
#[doc(alias = "GTK_PAPER_NAME_A4")]
pub static PAPER_NAME_A4: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_A4) };
#[doc(alias = "GTK_PAPER_NAME_A5")]
pub static PAPER_NAME_A5: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_A5) };
#[doc(alias = "GTK_PAPER_NAME_B5")]
pub static PAPER_NAME_B5: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_B5) };
#[doc(alias = "GTK_PAPER_NAME_EXECUTIVE")]
pub static PAPER_NAME_EXECUTIVE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_EXECUTIVE) };
#[doc(alias = "GTK_PAPER_NAME_LEGAL")]
pub static PAPER_NAME_LEGAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_LEGAL) };
#[doc(alias = "GTK_PAPER_NAME_LETTER")]
pub static PAPER_NAME_LETTER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_LETTER) };
#[doc(alias = "GTK_PRINT_SETTINGS_COLLATE")]
pub static PRINT_SETTINGS_COLLATE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_COLLATE) };
#[doc(alias = "GTK_PRINT_SETTINGS_DEFAULT_SOURCE")]
pub static PRINT_SETTINGS_DEFAULT_SOURCE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_DEFAULT_SOURCE) };
#[doc(alias = "GTK_PRINT_SETTINGS_DITHER")]
pub static PRINT_SETTINGS_DITHER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_DITHER) };
#[doc(alias = "GTK_PRINT_SETTINGS_DUPLEX")]
pub static PRINT_SETTINGS_DUPLEX: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_DUPLEX) };
#[doc(alias = "GTK_PRINT_SETTINGS_FINISHINGS")]
pub static PRINT_SETTINGS_FINISHINGS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_FINISHINGS) };
#[doc(alias = "GTK_PRINT_SETTINGS_MEDIA_TYPE")]
pub static PRINT_SETTINGS_MEDIA_TYPE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_MEDIA_TYPE) };
#[doc(alias = "GTK_PRINT_SETTINGS_NUMBER_UP")]
pub static PRINT_SETTINGS_NUMBER_UP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_NUMBER_UP) };
#[doc(alias = "GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT")]
pub static PRINT_SETTINGS_NUMBER_UP_LAYOUT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT) };
#[doc(alias = "GTK_PRINT_SETTINGS_N_COPIES")]
pub static PRINT_SETTINGS_N_COPIES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_N_COPIES) };
#[doc(alias = "GTK_PRINT_SETTINGS_ORIENTATION")]
pub static PRINT_SETTINGS_ORIENTATION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_ORIENTATION) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_BASENAME")]
pub static PRINT_SETTINGS_OUTPUT_BASENAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_BASENAME) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_BIN")]
pub static PRINT_SETTINGS_OUTPUT_BIN: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_BIN) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_DIR")]
pub static PRINT_SETTINGS_OUTPUT_DIR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_DIR) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT")]
pub static PRINT_SETTINGS_OUTPUT_FILE_FORMAT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_URI")]
pub static PRINT_SETTINGS_OUTPUT_URI: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_URI) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAGE_RANGES")]
pub static PRINT_SETTINGS_PAGE_RANGES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAGE_RANGES) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAGE_SET")]
pub static PRINT_SETTINGS_PAGE_SET: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAGE_SET) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_FORMAT")]
pub static PRINT_SETTINGS_PAPER_FORMAT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAPER_FORMAT) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_HEIGHT")]
pub static PRINT_SETTINGS_PAPER_HEIGHT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAPER_HEIGHT) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_WIDTH")]
pub static PRINT_SETTINGS_PAPER_WIDTH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAPER_WIDTH) };
#[doc(alias = "GTK_PRINT_SETTINGS_PRINTER")]
pub static PRINT_SETTINGS_PRINTER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PRINTER) };
#[doc(alias = "GTK_PRINT_SETTINGS_PRINTER_LPI")]
pub static PRINT_SETTINGS_PRINTER_LPI: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PRINTER_LPI) };
#[doc(alias = "GTK_PRINT_SETTINGS_PRINT_PAGES")]
pub static PRINT_SETTINGS_PRINT_PAGES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PRINT_PAGES) };
#[doc(alias = "GTK_PRINT_SETTINGS_QUALITY")]
pub static PRINT_SETTINGS_QUALITY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_QUALITY) };
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION")]
pub static PRINT_SETTINGS_RESOLUTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_RESOLUTION) };
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION_X")]
pub static PRINT_SETTINGS_RESOLUTION_X: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_RESOLUTION_X) };
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION_Y")]
pub static PRINT_SETTINGS_RESOLUTION_Y: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_RESOLUTION_Y) };
#[doc(alias = "GTK_PRINT_SETTINGS_REVERSE")]
pub static PRINT_SETTINGS_REVERSE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_REVERSE) };
#[doc(alias = "GTK_PRINT_SETTINGS_SCALE")]
pub static PRINT_SETTINGS_SCALE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_SCALE) };
#[doc(alias = "GTK_PRINT_SETTINGS_USE_COLOR")]
pub static PRINT_SETTINGS_USE_COLOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_USE_COLOR) };
#[doc(alias = "GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA")]
pub static PRINT_SETTINGS_WIN32_DRIVER_EXTRA: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA) };
#[doc(alias = "GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION")]
pub static PRINT_SETTINGS_WIN32_DRIVER_VERSION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION) };
#[doc(alias = "GTK_STYLE_CLASS_ACCELERATOR")]
pub static STYLE_CLASS_ACCELERATOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_ACCELERATOR) };
#[doc(alias = "GTK_STYLE_CLASS_ARROW")]
pub static STYLE_CLASS_ARROW: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_ARROW) };
#[doc(alias = "GTK_STYLE_CLASS_BACKGROUND")]
pub static STYLE_CLASS_BACKGROUND: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_BACKGROUND) };
#[doc(alias = "GTK_STYLE_CLASS_BOTTOM")]
pub static STYLE_CLASS_BOTTOM: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_BOTTOM) };
#[doc(alias = "GTK_STYLE_CLASS_BUTTON")]
pub static STYLE_CLASS_BUTTON: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_BUTTON) };
#[doc(alias = "GTK_STYLE_CLASS_CALENDAR")]
pub static STYLE_CLASS_CALENDAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_CALENDAR) };
#[doc(alias = "GTK_STYLE_CLASS_CELL")]
pub static STYLE_CLASS_CELL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_CELL) };
#[doc(alias = "GTK_STYLE_CLASS_CHECK")]
pub static STYLE_CLASS_CHECK: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_CHECK) };
#[doc(alias = "GTK_STYLE_CLASS_COMBOBOX_ENTRY")]
pub static STYLE_CLASS_COMBOBOX_ENTRY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_COMBOBOX_ENTRY) };
#[doc(alias = "GTK_STYLE_CLASS_CONTEXT_MENU")]
pub static STYLE_CLASS_CONTEXT_MENU: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_CONTEXT_MENU) };
#[doc(alias = "GTK_STYLE_CLASS_CSD")]
pub static STYLE_CLASS_CSD: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_CSD) };
#[doc(alias = "GTK_STYLE_CLASS_CURSOR_HANDLE")]
pub static STYLE_CLASS_CURSOR_HANDLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_CURSOR_HANDLE) };
#[doc(alias = "GTK_STYLE_CLASS_DEFAULT")]
pub static STYLE_CLASS_DEFAULT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_DEFAULT) };
#[doc(alias = "GTK_STYLE_CLASS_DESTRUCTIVE_ACTION")]
pub static STYLE_CLASS_DESTRUCTIVE_ACTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_DESTRUCTIVE_ACTION) };
#[doc(alias = "GTK_STYLE_CLASS_DIM_LABEL")]
pub static STYLE_CLASS_DIM_LABEL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_DIM_LABEL) };
#[doc(alias = "GTK_STYLE_CLASS_DND")]
pub static STYLE_CLASS_DND: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_DND) };
#[doc(alias = "GTK_STYLE_CLASS_DOCK")]
pub static STYLE_CLASS_DOCK: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_DOCK) };
#[doc(alias = "GTK_STYLE_CLASS_ENTRY")]
pub static STYLE_CLASS_ENTRY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_ENTRY) };
#[doc(alias = "GTK_STYLE_CLASS_ERROR")]
pub static STYLE_CLASS_ERROR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_ERROR) };
#[doc(alias = "GTK_STYLE_CLASS_EXPANDER")]
pub static STYLE_CLASS_EXPANDER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_EXPANDER) };
#[doc(alias = "GTK_STYLE_CLASS_FLAT")]
pub static STYLE_CLASS_FLAT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_FLAT) };
#[doc(alias = "GTK_STYLE_CLASS_FRAME")]
pub static STYLE_CLASS_FRAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_FRAME) };
#[doc(alias = "GTK_STYLE_CLASS_GRIP")]
pub static STYLE_CLASS_GRIP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_GRIP) };
#[doc(alias = "GTK_STYLE_CLASS_HEADER")]
pub static STYLE_CLASS_HEADER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_HEADER) };
#[doc(alias = "GTK_STYLE_CLASS_HIGHLIGHT")]
pub static STYLE_CLASS_HIGHLIGHT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_HIGHLIGHT) };
#[doc(alias = "GTK_STYLE_CLASS_HORIZONTAL")]
pub static STYLE_CLASS_HORIZONTAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_HORIZONTAL) };
#[doc(alias = "GTK_STYLE_CLASS_IMAGE")]
pub static STYLE_CLASS_IMAGE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_IMAGE) };
#[doc(alias = "GTK_STYLE_CLASS_INFO")]
pub static STYLE_CLASS_INFO: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_INFO) };
#[doc(alias = "GTK_STYLE_CLASS_INLINE_TOOLBAR")]
pub static STYLE_CLASS_INLINE_TOOLBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_INLINE_TOOLBAR) };
#[doc(alias = "GTK_STYLE_CLASS_INSERTION_CURSOR")]
pub static STYLE_CLASS_INSERTION_CURSOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_INSERTION_CURSOR) };
#[doc(alias = "GTK_STYLE_CLASS_LABEL")]
pub static STYLE_CLASS_LABEL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_LABEL) };
#[doc(alias = "GTK_STYLE_CLASS_LEFT")]
pub static STYLE_CLASS_LEFT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_LEFT) };
#[doc(alias = "GTK_STYLE_CLASS_LEVEL_BAR")]
pub static STYLE_CLASS_LEVEL_BAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_LEVEL_BAR) };
#[doc(alias = "GTK_STYLE_CLASS_LINKED")]
pub static STYLE_CLASS_LINKED: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_LINKED) };
#[doc(alias = "GTK_STYLE_CLASS_LIST")]
pub static STYLE_CLASS_LIST: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_LIST) };
#[doc(alias = "GTK_STYLE_CLASS_LIST_ROW")]
pub static STYLE_CLASS_LIST_ROW: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_LIST_ROW) };
#[doc(alias = "GTK_STYLE_CLASS_MARK")]
pub static STYLE_CLASS_MARK: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_MARK) };
#[doc(alias = "GTK_STYLE_CLASS_MENU")]
pub static STYLE_CLASS_MENU: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_MENU) };
#[doc(alias = "GTK_STYLE_CLASS_MENUBAR")]
pub static STYLE_CLASS_MENUBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_MENUBAR) };
#[doc(alias = "GTK_STYLE_CLASS_MENUITEM")]
pub static STYLE_CLASS_MENUITEM: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_MENUITEM) };
#[doc(alias = "GTK_STYLE_CLASS_MESSAGE_DIALOG")]
pub static STYLE_CLASS_MESSAGE_DIALOG: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_MESSAGE_DIALOG) };
#[doc(alias = "GTK_STYLE_CLASS_MONOSPACE")]
pub static STYLE_CLASS_MONOSPACE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_MONOSPACE) };
#[doc(alias = "GTK_STYLE_CLASS_NEEDS_ATTENTION")]
pub static STYLE_CLASS_NEEDS_ATTENTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_NEEDS_ATTENTION) };
#[doc(alias = "GTK_STYLE_CLASS_NOTEBOOK")]
pub static STYLE_CLASS_NOTEBOOK: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_NOTEBOOK) };
#[doc(alias = "GTK_STYLE_CLASS_OSD")]
pub static STYLE_CLASS_OSD: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_OSD) };
#[doc(alias = "GTK_STYLE_CLASS_OVERSHOOT")]
pub static STYLE_CLASS_OVERSHOOT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_OVERSHOOT) };
#[doc(alias = "GTK_STYLE_CLASS_PANE_SEPARATOR")]
pub static STYLE_CLASS_PANE_SEPARATOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_PANE_SEPARATOR) };
#[doc(alias = "GTK_STYLE_CLASS_PAPER")]
pub static STYLE_CLASS_PAPER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_PAPER) };
#[doc(alias = "GTK_STYLE_CLASS_POPOVER")]
pub static STYLE_CLASS_POPOVER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_POPOVER) };
#[doc(alias = "GTK_STYLE_CLASS_POPUP")]
pub static STYLE_CLASS_POPUP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_POPUP) };
#[doc(alias = "GTK_STYLE_CLASS_PRIMARY_TOOLBAR")]
pub static STYLE_CLASS_PRIMARY_TOOLBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_PRIMARY_TOOLBAR) };
#[doc(alias = "GTK_STYLE_CLASS_PROGRESSBAR")]
pub static STYLE_CLASS_PROGRESSBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_PROGRESSBAR) };
#[doc(alias = "GTK_STYLE_CLASS_PULSE")]
pub static STYLE_CLASS_PULSE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_PULSE) };
#[doc(alias = "GTK_STYLE_CLASS_QUESTION")]
pub static STYLE_CLASS_QUESTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_QUESTION) };
#[doc(alias = "GTK_STYLE_CLASS_RADIO")]
pub static STYLE_CLASS_RADIO: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_RADIO) };
#[doc(alias = "GTK_STYLE_CLASS_RAISED")]
pub static STYLE_CLASS_RAISED: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_RAISED) };
#[doc(alias = "GTK_STYLE_CLASS_READ_ONLY")]
pub static STYLE_CLASS_READ_ONLY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_READ_ONLY) };
#[doc(alias = "GTK_STYLE_CLASS_RIGHT")]
pub static STYLE_CLASS_RIGHT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_RIGHT) };
#[doc(alias = "GTK_STYLE_CLASS_RUBBERBAND")]
pub static STYLE_CLASS_RUBBERBAND: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_RUBBERBAND) };
#[doc(alias = "GTK_STYLE_CLASS_SCALE")]
pub static STYLE_CLASS_SCALE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SCALE) };
#[doc(alias = "GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE")]
pub static STYLE_CLASS_SCALE_HAS_MARKS_ABOVE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE) };
#[doc(alias = "GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW")]
pub static STYLE_CLASS_SCALE_HAS_MARKS_BELOW: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW) };
#[doc(alias = "GTK_STYLE_CLASS_SCROLLBAR")]
pub static STYLE_CLASS_SCROLLBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SCROLLBAR) };
#[doc(alias = "GTK_STYLE_CLASS_SCROLLBARS_JUNCTION")]
pub static STYLE_CLASS_SCROLLBARS_JUNCTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SCROLLBARS_JUNCTION) };
#[doc(alias = "GTK_STYLE_CLASS_SEPARATOR")]
pub static STYLE_CLASS_SEPARATOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SEPARATOR) };
#[doc(alias = "GTK_STYLE_CLASS_SIDEBAR")]
pub static STYLE_CLASS_SIDEBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SIDEBAR) };
#[doc(alias = "GTK_STYLE_CLASS_SLIDER")]
pub static STYLE_CLASS_SLIDER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SLIDER) };
#[doc(alias = "GTK_STYLE_CLASS_SPINBUTTON")]
pub static STYLE_CLASS_SPINBUTTON: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SPINBUTTON) };
#[doc(alias = "GTK_STYLE_CLASS_SPINNER")]
pub static STYLE_CLASS_SPINNER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SPINNER) };
#[doc(alias = "GTK_STYLE_CLASS_STATUSBAR")]
pub static STYLE_CLASS_STATUSBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_STATUSBAR) };
#[doc(alias = "GTK_STYLE_CLASS_SUBTITLE")]
pub static STYLE_CLASS_SUBTITLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SUBTITLE) };
#[doc(alias = "GTK_STYLE_CLASS_SUGGESTED_ACTION")]
pub static STYLE_CLASS_SUGGESTED_ACTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_SUGGESTED_ACTION) };
#[doc(alias = "GTK_STYLE_CLASS_TITLE")]
pub static STYLE_CLASS_TITLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TITLE) };
#[doc(alias = "GTK_STYLE_CLASS_TITLEBAR")]
pub static STYLE_CLASS_TITLEBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TITLEBAR) };
#[doc(alias = "GTK_STYLE_CLASS_TOOLBAR")]
pub static STYLE_CLASS_TOOLBAR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TOOLBAR) };
#[doc(alias = "GTK_STYLE_CLASS_TOOLTIP")]
pub static STYLE_CLASS_TOOLTIP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TOOLTIP) };
#[doc(alias = "GTK_STYLE_CLASS_TOP")]
pub static STYLE_CLASS_TOP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TOP) };
#[doc(alias = "GTK_STYLE_CLASS_TOUCH_SELECTION")]
pub static STYLE_CLASS_TOUCH_SELECTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TOUCH_SELECTION) };
#[doc(alias = "GTK_STYLE_CLASS_TROUGH")]
pub static STYLE_CLASS_TROUGH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_TROUGH) };
#[doc(alias = "GTK_STYLE_CLASS_UNDERSHOOT")]
pub static STYLE_CLASS_UNDERSHOOT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_UNDERSHOOT) };
#[doc(alias = "GTK_STYLE_CLASS_VERTICAL")]
pub static STYLE_CLASS_VERTICAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_VERTICAL) };
#[doc(alias = "GTK_STYLE_CLASS_VIEW")]
pub static STYLE_CLASS_VIEW: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_VIEW) };
#[doc(alias = "GTK_STYLE_CLASS_WARNING")]
pub static STYLE_CLASS_WARNING: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_WARNING) };
#[doc(alias = "GTK_STYLE_CLASS_WIDE")]
pub static STYLE_CLASS_WIDE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_CLASS_WIDE) };
#[doc(alias = "GTK_STYLE_PROPERTY_BACKGROUND_COLOR")]
pub static STYLE_PROPERTY_BACKGROUND_COLOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_BACKGROUND_COLOR) };
#[doc(alias = "GTK_STYLE_PROPERTY_BACKGROUND_IMAGE")]
pub static STYLE_PROPERTY_BACKGROUND_IMAGE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_BACKGROUND_IMAGE) };
#[doc(alias = "GTK_STYLE_PROPERTY_BORDER_COLOR")]
pub static STYLE_PROPERTY_BORDER_COLOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_BORDER_COLOR) };
#[doc(alias = "GTK_STYLE_PROPERTY_BORDER_RADIUS")]
pub static STYLE_PROPERTY_BORDER_RADIUS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_BORDER_RADIUS) };
#[doc(alias = "GTK_STYLE_PROPERTY_BORDER_STYLE")]
pub static STYLE_PROPERTY_BORDER_STYLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_BORDER_STYLE) };
#[doc(alias = "GTK_STYLE_PROPERTY_BORDER_WIDTH")]
pub static STYLE_PROPERTY_BORDER_WIDTH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_BORDER_WIDTH) };
#[doc(alias = "GTK_STYLE_PROPERTY_COLOR")]
pub static STYLE_PROPERTY_COLOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_COLOR) };
#[doc(alias = "GTK_STYLE_PROPERTY_FONT")]
pub static STYLE_PROPERTY_FONT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_FONT) };
#[doc(alias = "GTK_STYLE_PROPERTY_MARGIN")]
pub static STYLE_PROPERTY_MARGIN: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_MARGIN) };
#[doc(alias = "GTK_STYLE_PROPERTY_PADDING")]
pub static STYLE_PROPERTY_PADDING: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_STYLE_PROPERTY_PADDING) };
