//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;
use objc2::__framework_prelude::*;

use crate::*;

extern_class!(
    /// [Apple's documentation](https://developer.apple.com/documentation/uikit/uitextinputcontext?language=objc)
    #[unsafe(super(NSObject))]
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UITextInputContext;
);

extern_conformance!(
    unsafe impl NSObjectProtocol for UITextInputContext {}
);

impl UITextInputContext {
    extern_methods!(
        #[unsafe(method(isPencilInputExpected))]
        #[unsafe(method_family = none)]
        pub fn isPencilInputExpected(&self) -> bool;

        /// Setter for [`isPencilInputExpected`][Self::isPencilInputExpected].
        #[unsafe(method(setPencilInputExpected:))]
        #[unsafe(method_family = none)]
        pub fn setPencilInputExpected(&self, pencil_input_expected: bool);

        #[unsafe(method(isDictationInputExpected))]
        #[unsafe(method_family = none)]
        pub fn isDictationInputExpected(&self) -> bool;

        /// Setter for [`isDictationInputExpected`][Self::isDictationInputExpected].
        #[unsafe(method(setDictationInputExpected:))]
        #[unsafe(method_family = none)]
        pub fn setDictationInputExpected(&self, dictation_input_expected: bool);

        #[unsafe(method(isHardwareKeyboardInputExpected))]
        #[unsafe(method_family = none)]
        pub fn isHardwareKeyboardInputExpected(&self) -> bool;

        /// Setter for [`isHardwareKeyboardInputExpected`][Self::isHardwareKeyboardInputExpected].
        #[unsafe(method(setHardwareKeyboardInputExpected:))]
        #[unsafe(method_family = none)]
        pub fn setHardwareKeyboardInputExpected(&self, hardware_keyboard_input_expected: bool);

        #[unsafe(method(init))]
        #[unsafe(method_family = init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[unsafe(method(new))]
        #[unsafe(method_family = new)]
        pub unsafe fn new() -> Retained<Self>;

        #[unsafe(method(current))]
        #[unsafe(method_family = none)]
        pub fn current() -> Option<Retained<UITextInputContext>>;
    );
}
