//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSBackgroundActivityResult(pub NSInteger);
impl NSBackgroundActivityResult {
    #[doc(alias = "NSBackgroundActivityResultFinished")]
    pub const Finished: Self = Self(1);
    #[doc(alias = "NSBackgroundActivityResultDeferred")]
    pub const Deferred: Self = Self(2);
}

unsafe impl Encode for NSBackgroundActivityResult {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSBackgroundActivityResult {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

#[cfg(feature = "block2")]
pub type NSBackgroundActivityCompletionHandler =
    *mut block2::Block<dyn Fn(NSBackgroundActivityResult)>;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct NSBackgroundActivityScheduler;

    unsafe impl ClassType for NSBackgroundActivityScheduler {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSObjectProtocol for NSBackgroundActivityScheduler {}

extern_methods!(
    unsafe impl NSBackgroundActivityScheduler {
        #[cfg(feature = "NSString")]
        #[method_id(@__retain_semantics Init initWithIdentifier:)]
        pub unsafe fn initWithIdentifier(
            this: Allocated<Self>,
            identifier: &NSString,
        ) -> Retained<Self>;

        #[cfg(feature = "NSString")]
        #[method_id(@__retain_semantics Other identifier)]
        pub unsafe fn identifier(&self) -> Retained<NSString>;

        #[cfg(feature = "NSObjCRuntime")]
        #[method(qualityOfService)]
        pub unsafe fn qualityOfService(&self) -> NSQualityOfService;

        #[cfg(feature = "NSObjCRuntime")]
        #[method(setQualityOfService:)]
        pub unsafe fn setQualityOfService(&self, quality_of_service: NSQualityOfService);

        #[method(repeats)]
        pub unsafe fn repeats(&self) -> bool;

        #[method(setRepeats:)]
        pub unsafe fn setRepeats(&self, repeats: bool);

        #[cfg(feature = "NSDate")]
        #[method(interval)]
        pub unsafe fn interval(&self) -> NSTimeInterval;

        #[cfg(feature = "NSDate")]
        #[method(setInterval:)]
        pub unsafe fn setInterval(&self, interval: NSTimeInterval);

        #[cfg(feature = "NSDate")]
        #[method(tolerance)]
        pub unsafe fn tolerance(&self) -> NSTimeInterval;

        #[cfg(feature = "NSDate")]
        #[method(setTolerance:)]
        pub unsafe fn setTolerance(&self, tolerance: NSTimeInterval);

        #[cfg(feature = "block2")]
        #[method(scheduleWithBlock:)]
        pub unsafe fn scheduleWithBlock(
            &self,
            block: &block2::Block<dyn Fn(NSBackgroundActivityCompletionHandler)>,
        );

        #[method(invalidate)]
        pub unsafe fn invalidate(&self);

        #[method(shouldDefer)]
        pub unsafe fn shouldDefer(&self) -> bool;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl NSBackgroundActivityScheduler {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
