//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

mod private_NSFileWrapperNSExtensions {
    pub trait Sealed {}
}

/// Category "NSExtensions" on [`NSFileWrapper`].
#[doc(alias = "NSExtensions")]
pub unsafe trait NSFileWrapperNSExtensions:
    ClassType + Sized + private_NSFileWrapperNSExtensions::Sealed
{
    extern_methods!(
        #[cfg(feature = "NSImage")]
        #[unsafe(method(icon))]
        #[unsafe(method_family = none)]
        fn icon(&self) -> Option<Retained<NSImage>>;

        #[cfg(feature = "NSImage")]
        /// Setter for [`icon`][Self::icon].
        #[unsafe(method(setIcon:))]
        #[unsafe(method_family = none)]
        fn setIcon(&self, icon: Option<&NSImage>);
    );
}

impl private_NSFileWrapperNSExtensions::Sealed for NSFileWrapper {}
unsafe impl NSFileWrapperNSExtensions for NSFileWrapper {}
