// This file has been automatically generated by `objc2`'s `header-translator`.
// DO NOT EDIT

#![allow(unused_imports)]
#![allow(deprecated)]
#![allow(non_snake_case)]
#![allow(non_camel_case_types)]
#![allow(non_upper_case_globals)]
#![allow(missing_docs)]
#![allow(clippy::too_many_arguments)]
#![allow(clippy::type_complexity)]
#![allow(clippy::upper_case_acronyms)]
#![allow(clippy::identity_op)]
#![allow(clippy::missing_safety_doc)]

#[link(name = "Contacts", kind = "framework")]
extern "C" {}

#[cfg(feature = "CNChangeHistoryEvent")]
#[path = "CNChangeHistoryEvent.rs"]
mod __CNChangeHistoryEvent;
#[cfg(feature = "CNChangeHistoryFetchRequest")]
#[path = "CNChangeHistoryFetchRequest.rs"]
mod __CNChangeHistoryFetchRequest;
#[cfg(feature = "CNContact")]
#[path = "CNContact.rs"]
mod __CNContact;
#[cfg(feature = "CNContactFetchRequest")]
#[path = "CNContactFetchRequest.rs"]
mod __CNContactFetchRequest;
#[cfg(feature = "CNContactFormatter")]
#[path = "CNContactFormatter.rs"]
mod __CNContactFormatter;
#[cfg(feature = "CNContactProperty")]
#[path = "CNContactProperty.rs"]
mod __CNContactProperty;
#[cfg(feature = "CNContactRelation")]
#[path = "CNContactRelation.rs"]
mod __CNContactRelation;
#[cfg(feature = "CNContactStore")]
#[path = "CNContactStore.rs"]
mod __CNContactStore;
#[cfg(feature = "CNContactVCardSerialization")]
#[path = "CNContactVCardSerialization.rs"]
mod __CNContactVCardSerialization;
#[cfg(feature = "CNContact_NSItemProvider")]
#[path = "CNContact_NSItemProvider.rs"]
mod __CNContact_NSItemProvider;
#[cfg(feature = "CNContact_Predicates")]
#[path = "CNContact_Predicates.rs"]
mod __CNContact_Predicates;
#[cfg(feature = "CNContactsUserDefaults")]
#[path = "CNContactsUserDefaults.rs"]
mod __CNContactsUserDefaults;
#[cfg(feature = "CNContainer")]
#[path = "CNContainer.rs"]
mod __CNContainer;
#[cfg(feature = "CNContainer_Predicates")]
#[path = "CNContainer_Predicates.rs"]
mod __CNContainer_Predicates;
#[cfg(feature = "CNError")]
#[path = "CNError.rs"]
mod __CNError;
#[cfg(feature = "CNFetchRequest")]
#[path = "CNFetchRequest.rs"]
mod __CNFetchRequest;
#[cfg(feature = "CNFetchResult")]
#[path = "CNFetchResult.rs"]
mod __CNFetchResult;
#[cfg(feature = "CNGroup")]
#[path = "CNGroup.rs"]
mod __CNGroup;
#[cfg(feature = "CNGroup_Predicates")]
#[path = "CNGroup_Predicates.rs"]
mod __CNGroup_Predicates;
#[cfg(feature = "CNInstantMessageAddress")]
#[path = "CNInstantMessageAddress.rs"]
mod __CNInstantMessageAddress;
#[cfg(feature = "CNLabeledValue")]
#[path = "CNLabeledValue.rs"]
mod __CNLabeledValue;
#[cfg(feature = "CNMutableContact")]
#[path = "CNMutableContact.rs"]
mod __CNMutableContact;
#[cfg(feature = "CNMutableGroup")]
#[path = "CNMutableGroup.rs"]
mod __CNMutableGroup;
#[cfg(feature = "CNMutablePostalAddress")]
#[path = "CNMutablePostalAddress.rs"]
mod __CNMutablePostalAddress;
#[cfg(feature = "CNPhoneNumber")]
#[path = "CNPhoneNumber.rs"]
mod __CNPhoneNumber;
#[cfg(feature = "CNPostalAddress")]
#[path = "CNPostalAddress.rs"]
mod __CNPostalAddress;
#[cfg(feature = "CNPostalAddressFormatter")]
#[path = "CNPostalAddressFormatter.rs"]
mod __CNPostalAddressFormatter;
#[cfg(feature = "CNSaveRequest")]
#[path = "CNSaveRequest.rs"]
mod __CNSaveRequest;
#[cfg(feature = "CNSocialProfile")]
#[path = "CNSocialProfile.rs"]
mod __CNSocialProfile;
#[cfg(feature = "ContactsDefines")]
#[path = "ContactsDefines.rs"]
mod __ContactsDefines;

#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryAddContactEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryAddGroupEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryAddMemberToGroupEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryAddSubgroupToGroupEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryDeleteContactEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryDeleteGroupEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryDropEverythingEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryEventVisitor;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryRemoveMemberFromGroupEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryRemoveSubgroupFromGroupEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryUpdateContactEvent;
#[cfg(feature = "CNChangeHistoryEvent")]
pub use self::__CNChangeHistoryEvent::CNChangeHistoryUpdateGroupEvent;
#[cfg(all(feature = "CNChangeHistoryFetchRequest", feature = "CNFetchRequest"))]
pub use self::__CNChangeHistoryFetchRequest::CNChangeHistoryFetchRequest;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContact;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactBirthdayKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactDatesKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactDepartmentNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactEmailAddressesKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactFamilyNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactGivenNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactIdentifierKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactImageDataAvailableKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactImageDataKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactInstantMessageAddressesKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactJobTitleKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactMiddleNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactNamePrefixKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactNameSuffixKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactNicknameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactNonGregorianBirthdayKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactNoteKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactOrganizationNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPhoneNumbersKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPhoneticFamilyNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPhoneticGivenNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPhoneticMiddleNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPhoneticOrganizationNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPostalAddressesKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPreviousFamilyNameKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactPropertyNotFetchedExceptionName;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactRelationsKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactSocialProfilesKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactSortOrder;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactThumbnailImageDataKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactType;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactTypeKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNContactUrlAddressesKey;
#[cfg(feature = "CNContact")]
pub use self::__CNContact::CNKeyDescriptor;
#[cfg(all(feature = "CNContactFetchRequest", feature = "CNFetchRequest"))]
pub use self::__CNContactFetchRequest::CNContactFetchRequest;
#[cfg(feature = "CNContactFormatter")]
pub use self::__CNContactFormatter::CNContactDisplayNameOrder;
#[cfg(feature = "CNContactFormatter")]
pub use self::__CNContactFormatter::CNContactFormatter;
#[cfg(feature = "CNContactFormatter")]
pub use self::__CNContactFormatter::CNContactFormatterStyle;
#[cfg(feature = "CNContactFormatter")]
pub use self::__CNContactFormatter::CNContactPropertyAttribute;
#[cfg(feature = "CNContactProperty")]
pub use self::__CNContactProperty::CNContactProperty;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNContactRelation;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAssistant;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAunt;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntFathersBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntFathersElderBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntFathersElderSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntFathersSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntFathersYoungerBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntFathersYoungerSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntMothersBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntMothersElderSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntMothersSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntMothersYoungerSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntParentsElderSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntParentsSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationAuntParentsYoungerSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBoyfriend;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawElderSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawHusbandsBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawHusbandsSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawSpousesBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawWifesBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawWifesSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationBrotherInLawYoungerSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationChild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationChildInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCoBrotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCoFatherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCoMotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCoParentInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCoSiblingInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCoSisterInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationColleague;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinFathersBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinFathersBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinFathersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinFathersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinGrandparentsSiblingsChild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinGrandparentsSiblingsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinGrandparentsSiblingsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinMothersBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinMothersBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinMothersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinMothersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinOrSiblingsChild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinParentsSiblingsChild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinParentsSiblingsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationCousinParentsSiblingsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationDaughterInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationDaughterInLawOrSisterInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationDaughterInLawOrStepdaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderBrotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinFathersBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinFathersBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinFathersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinFathersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinMothersBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinMothersBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinMothersSiblingsDaughterOrFathersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinMothersSiblingsSonOrFathersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinMothersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinMothersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinParentsSiblingsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderCousinParentsSiblingsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderSiblingInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationElderSisterInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationEldestBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationEldestSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFatherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFatherInLawHusbandsFather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFatherInLawOrStepfather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFatherInLawWifesFather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFemaleCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFemaleFriend;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFemalePartner;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationFriend;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGirlfriend;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGirlfriendOrBoyfriend;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandaunt;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandchild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandchildOrSiblingsChild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGranddaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGranddaughterDaughtersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGranddaughterOrNiece;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGranddaughterSonsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandfather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandfatherFathersFather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandfatherMothersFather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandmother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandmotherFathersMother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandmotherMothersMother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandnephew;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandnephewBrothersGrandson;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandnephewSistersGrandson;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandniece;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandnieceBrothersGranddaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandnieceSistersGranddaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandparent;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandson;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandsonDaughtersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandsonOrNephew;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGrandsonSonsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGranduncle;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGrandchild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGrandchildOrSiblingsGrandchild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGranddaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGrandfather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGrandmother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGrandparent;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationGreatGrandson;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMaleCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMaleFriend;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMalePartner;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationManager;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMotherInLawHusbandsMother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMotherInLawOrStepmother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationMotherInLawWifesMother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNephew;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNephewBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNephewBrothersSonOrHusbandsSiblingsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNephewOrCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNephewSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNephewSistersSonOrWifesSiblingsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNiece;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNieceBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNieceBrothersDaughterOrHusbandsSiblingsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNieceOrCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNieceSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationNieceSistersDaughterOrWifesSiblingsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParent;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsElderSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSiblingFathersElderSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSiblingFathersSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSiblingFathersYoungerSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSiblingMothersElderSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSiblingMothersSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsSiblingMothersYoungerSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationParentsYoungerSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationPartner;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSiblingInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSiblingsChild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawElderBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawHusbandsBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawHusbandsSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawSpousesSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawWifesBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawWifesSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSisterInLawYoungerBrothersWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSonInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSonInLawOrBrotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSonInLawOrStepson;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationSpouse;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepbrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepchild;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepdaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepfather;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepmother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepparent;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepsister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationStepson;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationTeacher;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncle;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleFathersBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleFathersElderBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleFathersElderSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleFathersSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleFathersYoungerBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleFathersYoungerSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleMothersBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleMothersElderBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleMothersSistersHusband;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleMothersYoungerBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleParentsBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleParentsElderBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationUncleParentsYoungerBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationWife;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerBrotherInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousin;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinFathersBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinFathersBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinFathersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinFathersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinMothersBrothersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinMothersBrothersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinMothersSiblingsDaughterOrFathersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinMothersSiblingsSonOrFathersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinMothersSistersDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinMothersSistersSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinParentsSiblingsDaughter;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerCousinParentsSiblingsSon;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerSibling;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerSiblingInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerSister;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungerSisterInLaw;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungestBrother;
#[cfg(feature = "CNContactRelation")]
pub use self::__CNContactRelation::CNLabelContactRelationYoungestSister;
#[cfg(feature = "CNContactStore")]
pub use self::__CNContactStore::CNAuthorizationStatus;
#[cfg(feature = "CNContactStore")]
pub use self::__CNContactStore::CNContactStore;
#[cfg(feature = "CNContactStore")]
pub use self::__CNContactStore::CNContactStoreDidChangeNotification;
#[cfg(feature = "CNContactStore")]
pub use self::__CNContactStore::CNEntityType;
#[cfg(feature = "CNContactVCardSerialization")]
pub use self::__CNContactVCardSerialization::CNContactVCardSerialization;
#[cfg(feature = "CNContactsUserDefaults")]
pub use self::__CNContactsUserDefaults::CNContactsUserDefaults;
#[cfg(feature = "CNContainer")]
pub use self::__CNContainer::CNContainer;
#[cfg(feature = "CNContainer")]
pub use self::__CNContainer::CNContainerIdentifierKey;
#[cfg(feature = "CNContainer")]
pub use self::__CNContainer::CNContainerNameKey;
#[cfg(feature = "CNContainer")]
pub use self::__CNContainer::CNContainerType;
#[cfg(feature = "CNContainer")]
pub use self::__CNContainer::CNContainerTypeKey;
#[cfg(feature = "CNError")]
pub use self::__CNError::CNErrorCode;
#[cfg(feature = "CNError")]
pub use self::__CNError::CNErrorDomain;
#[cfg(feature = "CNError")]
pub use self::__CNError::CNErrorUserInfoAffectedRecordIdentifiersKey;
#[cfg(feature = "CNError")]
pub use self::__CNError::CNErrorUserInfoAffectedRecordsKey;
#[cfg(feature = "CNError")]
pub use self::__CNError::CNErrorUserInfoKeyPathsKey;
#[cfg(feature = "CNError")]
pub use self::__CNError::CNErrorUserInfoValidationErrorsKey;
#[cfg(feature = "CNFetchRequest")]
pub use self::__CNFetchRequest::CNFetchRequest;
#[cfg(feature = "CNFetchResult")]
pub use self::__CNFetchResult::CNFetchResult;
#[cfg(feature = "CNGroup")]
pub use self::__CNGroup::CNGroup;
#[cfg(feature = "CNGroup")]
pub use self::__CNGroup::CNGroupIdentifierKey;
#[cfg(feature = "CNGroup")]
pub use self::__CNGroup::CNGroupNameKey;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageAddress;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageAddressServiceKey;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageAddressUsernameKey;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceAIM;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceFacebook;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceGaduGadu;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceGoogleTalk;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceICQ;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceJabber;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceMSN;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceQQ;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceSkype;
#[cfg(feature = "CNInstantMessageAddress")]
pub use self::__CNInstantMessageAddress::CNInstantMessageServiceYahoo;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelDateAnniversary;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelEmailiCloud;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelHome;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelOther;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelSchool;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelURLAddressHomePage;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabelWork;
#[cfg(feature = "CNLabeledValue")]
pub use self::__CNLabeledValue::CNLabeledValue;
#[cfg(all(feature = "CNContact", feature = "CNMutableContact"))]
pub use self::__CNMutableContact::CNMutableContact;
#[cfg(all(feature = "CNGroup", feature = "CNMutableGroup"))]
pub use self::__CNMutableGroup::CNMutableGroup;
#[cfg(all(feature = "CNMutablePostalAddress", feature = "CNPostalAddress"))]
pub use self::__CNMutablePostalAddress::CNMutablePostalAddress;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberAppleWatch;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberHomeFax;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberMain;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberMobile;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberOtherFax;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberPager;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberWorkFax;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNLabelPhoneNumberiPhone;
#[cfg(feature = "CNPhoneNumber")]
pub use self::__CNPhoneNumber::CNPhoneNumber;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddress;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressCityKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressCountryKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressISOCountryCodeKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressPostalCodeKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressStateKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressStreetKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressSubAdministrativeAreaKey;
#[cfg(feature = "CNPostalAddress")]
pub use self::__CNPostalAddress::CNPostalAddressSubLocalityKey;
#[cfg(feature = "CNPostalAddressFormatter")]
pub use self::__CNPostalAddressFormatter::CNPostalAddressFormatter;
#[cfg(feature = "CNPostalAddressFormatter")]
pub use self::__CNPostalAddressFormatter::CNPostalAddressFormatterStyle;
#[cfg(feature = "CNPostalAddressFormatter")]
pub use self::__CNPostalAddressFormatter::CNPostalAddressLocalizedPropertyNameAttribute;
#[cfg(feature = "CNPostalAddressFormatter")]
pub use self::__CNPostalAddressFormatter::CNPostalAddressPropertyAttribute;
#[cfg(feature = "CNSaveRequest")]
pub use self::__CNSaveRequest::CNSaveRequest;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfile;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceFacebook;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceFlickr;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceGameCenter;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceKey;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceLinkedIn;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceMySpace;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceSinaWeibo;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceTencentWeibo;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceTwitter;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileServiceYelp;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileURLStringKey;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileUserIdentifierKey;
#[cfg(feature = "CNSocialProfile")]
pub use self::__CNSocialProfile::CNSocialProfileUsernameKey;
