//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSEnergyFormatterUnit(pub NSInteger);
impl NSEnergyFormatterUnit {
    #[doc(alias = "NSEnergyFormatterUnitJoule")]
    pub const Joule: Self = Self(11);
    #[doc(alias = "NSEnergyFormatterUnitKilojoule")]
    pub const Kilojoule: Self = Self(14);
    #[doc(alias = "NSEnergyFormatterUnitCalorie")]
    pub const Calorie: Self = Self((7 << 8) + 1);
    #[doc(alias = "NSEnergyFormatterUnitKilocalorie")]
    pub const Kilocalorie: Self = Self((7 << 8) + 2);
}

unsafe impl Encode for NSEnergyFormatterUnit {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSEnergyFormatterUnit {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    #[cfg(feature = "NSFormatter")]
    pub struct NSEnergyFormatter;

    #[cfg(feature = "NSFormatter")]
    unsafe impl ClassType for NSEnergyFormatter {
        #[inherits(NSObject)]
        type Super = NSFormatter;
        type Mutability = InteriorMutable;
    }
);

#[cfg(all(feature = "NSFormatter", feature = "NSObject"))]
unsafe impl NSCoding for NSEnergyFormatter {}

#[cfg(all(feature = "NSFormatter", feature = "NSObject"))]
unsafe impl NSCopying for NSEnergyFormatter {}

#[cfg(feature = "NSFormatter")]
unsafe impl NSObjectProtocol for NSEnergyFormatter {}

extern_methods!(
    #[cfg(feature = "NSFormatter")]
    unsafe impl NSEnergyFormatter {
        #[cfg(feature = "NSNumberFormatter")]
        #[method_id(@__retain_semantics Other numberFormatter)]
        pub unsafe fn numberFormatter(&self) -> Retained<NSNumberFormatter>;

        #[cfg(feature = "NSNumberFormatter")]
        #[method(setNumberFormatter:)]
        pub unsafe fn setNumberFormatter(&self, number_formatter: Option<&NSNumberFormatter>);

        #[method(unitStyle)]
        pub unsafe fn unitStyle(&self) -> NSFormattingUnitStyle;

        #[method(setUnitStyle:)]
        pub unsafe fn setUnitStyle(&self, unit_style: NSFormattingUnitStyle);

        #[method(isForFoodEnergyUse)]
        pub unsafe fn isForFoodEnergyUse(&self) -> bool;

        #[method(setForFoodEnergyUse:)]
        pub unsafe fn setForFoodEnergyUse(&self, for_food_energy_use: bool);

        #[cfg(feature = "NSString")]
        #[method_id(@__retain_semantics Other stringFromValue:unit:)]
        pub unsafe fn stringFromValue_unit(
            &self,
            value: c_double,
            unit: NSEnergyFormatterUnit,
        ) -> Retained<NSString>;

        #[cfg(feature = "NSString")]
        #[method_id(@__retain_semantics Other stringFromJoules:)]
        pub unsafe fn stringFromJoules(&self, number_in_joules: c_double) -> Retained<NSString>;

        #[cfg(feature = "NSString")]
        #[method_id(@__retain_semantics Other unitStringFromValue:unit:)]
        pub unsafe fn unitStringFromValue_unit(
            &self,
            value: c_double,
            unit: NSEnergyFormatterUnit,
        ) -> Retained<NSString>;

        #[cfg(feature = "NSString")]
        #[method_id(@__retain_semantics Other unitStringFromJoules:usedUnit:)]
        pub unsafe fn unitStringFromJoules_usedUnit(
            &self,
            number_in_joules: c_double,
            unitp: *mut NSEnergyFormatterUnit,
        ) -> Retained<NSString>;

        #[cfg(feature = "NSString")]
        #[method(getObjectValue:forString:errorDescription:)]
        pub unsafe fn getObjectValue_forString_errorDescription(
            &self,
            obj: Option<&mut Option<Retained<AnyObject>>>,
            string: &NSString,
            error: Option<&mut Option<Retained<NSString>>>,
        ) -> bool;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    #[cfg(feature = "NSFormatter")]
    unsafe impl NSEnergyFormatter {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
