//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ptr::NonNull;
use objc2_foundation::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/uikit/uiniboptionskey?language=objc)
// NS_TYPED_ENUM
pub type UINibOptionsKey = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/uikit/uinibexternalobjects?language=objc)
    pub static UINibExternalObjects: &'static UINibOptionsKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/uikit/uinibproxiedobjectskey?language=objc)
    #[deprecated]
    pub static UINibProxiedObjectsKey: &'static NSString;
}
