//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_methods!(
    /// NSItemProvider
    #[cfg(feature = "CNContact")]
    unsafe impl CNContact {}
);

#[cfg(feature = "CNContact")]
unsafe impl NSItemProviderReading for CNContact {}

#[cfg(feature = "CNContact")]
unsafe impl NSItemProviderWriting for CNContact {}
