//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct CAFrameRateRange {
    pub minimum: c_float,
    pub maximum: c_float,
    pub preferred: c_float,
}

unsafe impl Encode for CAFrameRateRange {
    const ENCODING: Encoding = Encoding::Struct(
        "CAFrameRateRange",
        &[
            <c_float>::ENCODING,
            <c_float>::ENCODING,
            <c_float>::ENCODING,
        ],
    );
}

unsafe impl RefEncode for CAFrameRateRange {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern "C" {
    pub static CAFrameRateRangeDefault: CAFrameRateRange;
}

extern "C" {
    pub fn CAFrameRateRangeMake(
        minimum: c_float,
        maximum: c_float,
        preferred: c_float,
    ) -> CAFrameRateRange;
}

extern "C" {
    pub fn CAFrameRateRangeIsEqualToRange(range: CAFrameRateRange, other: CAFrameRateRange)
        -> bool;
}
