//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UISwipeActionsConfiguration;

    unsafe impl ClassType for UISwipeActionsConfiguration {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UISwipeActionsConfiguration {}

extern_methods!(
    unsafe impl UISwipeActionsConfiguration {
        #[cfg(feature = "UIContextualAction")]
        #[method_id(@__retain_semantics Other configurationWithActions:)]
        pub unsafe fn configurationWithActions(
            actions: &NSArray<UIContextualAction>,
            mtm: MainThreadMarker,
        ) -> Retained<Self>;

        #[cfg(feature = "UIContextualAction")]
        #[method_id(@__retain_semantics Other actions)]
        pub unsafe fn actions(&self) -> Retained<NSArray<UIContextualAction>>;

        #[method(performsFirstActionWithFullSwipe)]
        pub unsafe fn performsFirstActionWithFullSwipe(&self) -> bool;

        #[method(setPerformsFirstActionWithFullSwipe:)]
        pub unsafe fn setPerformsFirstActionWithFullSwipe(
            &self,
            performs_first_action_with_full_swipe: bool,
        );
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UISwipeActionsConfiguration {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
