//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIImageConfiguration;

    unsafe impl ClassType for UIImageConfiguration {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl Send for UIImageConfiguration {}

unsafe impl Sync for UIImageConfiguration {}

unsafe impl NSCoding for UIImageConfiguration {}

unsafe impl NSCopying for UIImageConfiguration {}

unsafe impl NSObjectProtocol for UIImageConfiguration {}

unsafe impl NSSecureCoding for UIImageConfiguration {}

extern_methods!(
    unsafe impl UIImageConfiguration {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[cfg(feature = "UITraitCollection")]
        #[method_id(@__retain_semantics Other traitCollection)]
        pub unsafe fn traitCollection(&self) -> Option<Retained<UITraitCollection>>;

        #[cfg(feature = "UITraitCollection")]
        #[method_id(@__retain_semantics Other configurationWithTraitCollection:)]
        pub unsafe fn configurationWithTraitCollection(
            &self,
            trait_collection: Option<&UITraitCollection>,
        ) -> Retained<Self>;

        #[cfg(feature = "UITraitCollection")]
        #[method_id(@__retain_semantics Other configurationWithTraitCollection:)]
        pub unsafe fn configurationWithTraitCollection_class(
            trait_collection: Option<&UITraitCollection>,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Other locale)]
        pub unsafe fn locale(&self) -> Option<Retained<NSLocale>>;

        #[method_id(@__retain_semantics Other configurationWithLocale:)]
        pub unsafe fn configurationWithLocale(&self, locale: Option<&NSLocale>) -> Retained<Self>;

        #[method_id(@__retain_semantics Other configurationWithLocale:)]
        pub unsafe fn configurationWithLocale_class(locale: Option<&NSLocale>) -> Retained<Self>;

        #[method_id(@__retain_semantics Other configurationByApplyingConfiguration:)]
        pub unsafe fn configurationByApplyingConfiguration(
            &self,
            other_configuration: Option<&UIImageConfiguration>,
        ) -> Retained<Self>;
    }
);
