//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_methods!(
    /// URLPreviews
    #[cfg(feature = "UIDragPreview")]
    unsafe impl UIDragPreview {
        #[method_id(@__retain_semantics Other previewForURL:)]
        pub unsafe fn previewForURL(url: &NSURL, mtm: MainThreadMarker) -> Retained<Self>;

        #[method_id(@__retain_semantics Other previewForURL:title:)]
        pub unsafe fn previewForURL_title(
            url: &NSURL,
            title: Option<&NSString>,
            mtm: MainThreadMarker,
        ) -> Retained<Self>;
    }
);

extern_methods!(
    /// URLPreviews
    #[cfg(all(feature = "UITargetedDragPreview", feature = "UITargetedPreview"))]
    unsafe impl UITargetedDragPreview {
        #[method_id(@__retain_semantics Other previewForURL:target:)]
        pub unsafe fn previewForURL_target(
            url: &NSURL,
            target: &UIDragPreviewTarget,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Other previewForURL:title:target:)]
        pub unsafe fn previewForURL_title_target(
            url: &NSURL,
            title: Option<&NSString>,
            target: &UIDragPreviewTarget,
        ) -> Retained<Self>;
    }
);
