//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use objc2::__framework_prelude::*;

use crate::*;

extern_protocol!(
    /// [Apple's documentation](https://developer.apple.com/documentation/uikit/uispringloadedinteractionsupporting?language=objc)
    pub unsafe trait UISpringLoadedInteractionSupporting:
        NSObjectProtocol + MainThreadOnly
    {
        #[unsafe(method(isSpringLoaded))]
        #[unsafe(method_family = none)]
        fn isSpringLoaded(&self) -> bool;

        /// Setter for [`isSpringLoaded`][Self::isSpringLoaded].
        #[unsafe(method(setSpringLoaded:))]
        #[unsafe(method_family = none)]
        fn setSpringLoaded(&self, spring_loaded: bool);
    }
);
