//! Directory: `linux/`
//!
//! <https://github.com/torvalds/linux/tree/master/include/uapi/linux>

pub(crate) mod can;
pub(crate) mod keyctl;
pub(crate) mod membarrier;
pub(crate) mod netlink;
pub(crate) mod pidfd;
