//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ptr::NonNull;
use objc2::__framework_prelude::*;

use crate::*;

extern_class!(
    /// A base class you use to handle suggestions from the keyboard or system.
    ///
    /// See also [Apple's documentation](https://developer.apple.com/documentation/uikit/uiinputsuggestion?language=objc)
    #[unsafe(super(NSObject))]
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIInputSuggestion;
);

extern_conformance!(
    unsafe impl NSObjectProtocol for UIInputSuggestion {}
);

impl UIInputSuggestion {
    extern_methods!();
}

/// Methods declared on superclass `NSObject`.
impl UIInputSuggestion {
    extern_methods!(
        #[unsafe(method(init))]
        #[unsafe(method_family = init)]
        pub fn init(this: Allocated<Self>) -> Retained<Self>;

        #[unsafe(method(new))]
        #[unsafe(method_family = new)]
        pub fn new() -> Retained<Self>;
    );
}

impl DefaultRetained for UIInputSuggestion {
    #[inline]
    fn default_retained() -> Retained<Self> {
        Self::new()
    }
}
