//! Bindings for the NDK media classes.
//!
//! See also [the NDK docs](https://developer.android.com/ndk/reference/group/media)
#![cfg(feature = "media")]

pub mod image_reader;
pub mod media_codec;
pub mod media_format;
