#![doc = include_str!("README.md")]

#[doc = include_str!("deref.md")]
pub mod deref {}

#[doc = include_str!("list.md")]
#[doc = include_str!("list_data.md")]
pub mod list {}

#[doc = include_str!("CHANGELOG.md")]
pub mod changelog {}

#[doc = include_str!("cargo_features.md")]
pub mod cargo_features {}
