//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_methods!(
    /// Predicates
    #[cfg(feature = "CNContact")]
    unsafe impl CNContact {
        #[method_id(@__retain_semantics Other predicateForContactsMatchingName:)]
        pub unsafe fn predicateForContactsMatchingName(name: &NSString) -> Retained<NSPredicate>;

        #[method_id(@__retain_semantics Other predicateForContactsMatchingEmailAddress:)]
        pub unsafe fn predicateForContactsMatchingEmailAddress(
            email_address: &NSString,
        ) -> Retained<NSPredicate>;

        #[cfg(feature = "CNPhoneNumber")]
        #[method_id(@__retain_semantics Other predicateForContactsMatchingPhoneNumber:)]
        pub unsafe fn predicateForContactsMatchingPhoneNumber(
            phone_number: &CNPhoneNumber,
        ) -> Retained<NSPredicate>;

        #[method_id(@__retain_semantics Other predicateForContactsWithIdentifiers:)]
        pub unsafe fn predicateForContactsWithIdentifiers(
            identifiers: &NSArray<NSString>,
        ) -> Retained<NSPredicate>;

        #[method_id(@__retain_semantics Other predicateForContactsInGroupWithIdentifier:)]
        pub unsafe fn predicateForContactsInGroupWithIdentifier(
            group_identifier: &NSString,
        ) -> Retained<NSPredicate>;

        #[method_id(@__retain_semantics Other predicateForContactsInContainerWithIdentifier:)]
        pub unsafe fn predicateForContactsInContainerWithIdentifier(
            container_identifier: &NSString,
        ) -> Retained<NSPredicate>;
    }
);
