/* automatically generated by rust-bindgen 0.71.1 */

pub const __BIONIC__: u32 = 1;
pub const __WORDSIZE: u32 = 32;
pub const __bos_level: u32 = 0;
pub const __ANDROID_API_FUTURE__: u32 = 10000;
pub const __ANDROID_API__: u32 = 10000;
pub const __ANDROID_API_G__: u32 = 9;
pub const __ANDROID_API_I__: u32 = 14;
pub const __ANDROID_API_J__: u32 = 16;
pub const __ANDROID_API_J_MR1__: u32 = 17;
pub const __ANDROID_API_J_MR2__: u32 = 18;
pub const __ANDROID_API_K__: u32 = 19;
pub const __ANDROID_API_L__: u32 = 21;
pub const __ANDROID_API_L_MR1__: u32 = 22;
pub const __ANDROID_API_M__: u32 = 23;
pub const __ANDROID_API_N__: u32 = 24;
pub const __ANDROID_API_N_MR1__: u32 = 25;
pub const __ANDROID_API_O__: u32 = 26;
pub const __ANDROID_API_O_MR1__: u32 = 27;
pub const __ANDROID_API_P__: u32 = 28;
pub const __ANDROID_API_Q__: u32 = 29;
pub const __ANDROID_API_R__: u32 = 30;
pub const __ANDROID_API_S__: u32 = 31;
pub const __ANDROID_API_T__: u32 = 33;
pub const __ANDROID_API_U__: u32 = 34;
pub const __ANDROID_API_V__: u32 = 35;
pub const __ANDROID_NDK__: u32 = 1;
pub const __NDK_MAJOR__: u32 = 29;
pub const __NDK_MINOR__: u32 = 0;
pub const __NDK_BETA__: u32 = 0;
pub const __NDK_BUILD__: u32 = 14206865;
pub const __NDK_CANARY__: u32 = 0;
pub const INT8_MIN: i32 = -128;
pub const INT8_MAX: u32 = 127;
pub const INT_LEAST8_MIN: i32 = -128;
pub const INT_LEAST8_MAX: u32 = 127;
pub const INT_FAST8_MIN: i32 = -128;
pub const INT_FAST8_MAX: u32 = 127;
pub const UINT8_MAX: u32 = 255;
pub const UINT_LEAST8_MAX: u32 = 255;
pub const UINT_FAST8_MAX: u32 = 255;
pub const INT16_MIN: i32 = -32768;
pub const INT16_MAX: u32 = 32767;
pub const INT_LEAST16_MIN: i32 = -32768;
pub const INT_LEAST16_MAX: u32 = 32767;
pub const UINT16_MAX: u32 = 65535;
pub const UINT_LEAST16_MAX: u32 = 65535;
pub const INT32_MIN: i32 = -2147483648;
pub const INT32_MAX: u32 = 2147483647;
pub const INT_LEAST32_MIN: i32 = -2147483648;
pub const INT_LEAST32_MAX: u32 = 2147483647;
pub const INT_FAST32_MIN: i32 = -2147483648;
pub const INT_FAST32_MAX: u32 = 2147483647;
pub const UINT32_MAX: u32 = 4294967295;
pub const UINT_LEAST32_MAX: u32 = 4294967295;
pub const UINT_FAST32_MAX: u32 = 4294967295;
pub const SIG_ATOMIC_MAX: u32 = 2147483647;
pub const SIG_ATOMIC_MIN: i32 = -2147483648;
pub const WINT_MAX: u32 = 4294967295;
pub const WINT_MIN: u32 = 0;
pub const INTPTR_MIN: i32 = -2147483648;
pub const INTPTR_MAX: u32 = 2147483647;
pub const UINTPTR_MAX: u32 = 4294967295;
pub const PTRDIFF_MIN: i32 = -2147483648;
pub const PTRDIFF_MAX: u32 = 2147483647;
pub const SIZE_MAX: u32 = 4294967295;
pub const __BITS_PER_LONG: u32 = 32;
pub const __BITS_PER_LONG_LONG: u32 = 64;
pub const __FD_SETSIZE: u32 = 1024;
pub const AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT: u32 = 8;
pub const __bool_true_false_are_defined: u32 = 1;
pub const true_: u32 = 1;
pub const false_: u32 = 0;
pub const __PRI_64_prefix: &[u8; 3] = b"ll\0";
pub const PRId8: &[u8; 2] = b"d\0";
pub const PRId16: &[u8; 2] = b"d\0";
pub const PRId32: &[u8; 2] = b"d\0";
pub const PRId64: &[u8; 4] = b"lld\0";
pub const PRIdLEAST8: &[u8; 2] = b"d\0";
pub const PRIdLEAST16: &[u8; 2] = b"d\0";
pub const PRIdLEAST32: &[u8; 2] = b"d\0";
pub const PRIdLEAST64: &[u8; 4] = b"lld\0";
pub const PRIdFAST8: &[u8; 2] = b"d\0";
pub const PRIdFAST64: &[u8; 4] = b"lld\0";
pub const PRIdMAX: &[u8; 3] = b"jd\0";
pub const PRIi8: &[u8; 2] = b"i\0";
pub const PRIi16: &[u8; 2] = b"i\0";
pub const PRIi32: &[u8; 2] = b"i\0";
pub const PRIi64: &[u8; 4] = b"lli\0";
pub const PRIiLEAST8: &[u8; 2] = b"i\0";
pub const PRIiLEAST16: &[u8; 2] = b"i\0";
pub const PRIiLEAST32: &[u8; 2] = b"i\0";
pub const PRIiLEAST64: &[u8; 4] = b"lli\0";
pub const PRIiFAST8: &[u8; 2] = b"i\0";
pub const PRIiFAST64: &[u8; 4] = b"lli\0";
pub const PRIiMAX: &[u8; 3] = b"ji\0";
pub const PRIb8: &[u8; 2] = b"b\0";
pub const PRIb16: &[u8; 2] = b"b\0";
pub const PRIb32: &[u8; 2] = b"b\0";
pub const PRIb64: &[u8; 4] = b"llb\0";
pub const PRIbLEAST8: &[u8; 2] = b"b\0";
pub const PRIbLEAST16: &[u8; 2] = b"b\0";
pub const PRIbLEAST32: &[u8; 2] = b"b\0";
pub const PRIbLEAST64: &[u8; 4] = b"llb\0";
pub const PRIbFAST8: &[u8; 2] = b"b\0";
pub const PRIbFAST64: &[u8; 4] = b"llb\0";
pub const PRIbMAX: &[u8; 3] = b"jb\0";
pub const PRIB8: &[u8; 2] = b"B\0";
pub const PRIB16: &[u8; 2] = b"B\0";
pub const PRIB32: &[u8; 2] = b"B\0";
pub const PRIB64: &[u8; 4] = b"llB\0";
pub const PRIBLEAST8: &[u8; 2] = b"B\0";
pub const PRIBLEAST16: &[u8; 2] = b"B\0";
pub const PRIBLEAST32: &[u8; 2] = b"B\0";
pub const PRIBLEAST64: &[u8; 4] = b"llB\0";
pub const PRIBFAST8: &[u8; 2] = b"B\0";
pub const PRIBFAST64: &[u8; 4] = b"llB\0";
pub const PRIBMAX: &[u8; 3] = b"jB\0";
pub const PRIo8: &[u8; 2] = b"o\0";
pub const PRIo16: &[u8; 2] = b"o\0";
pub const PRIo32: &[u8; 2] = b"o\0";
pub const PRIo64: &[u8; 4] = b"llo\0";
pub const PRIoLEAST8: &[u8; 2] = b"o\0";
pub const PRIoLEAST16: &[u8; 2] = b"o\0";
pub const PRIoLEAST32: &[u8; 2] = b"o\0";
pub const PRIoLEAST64: &[u8; 4] = b"llo\0";
pub const PRIoFAST8: &[u8; 2] = b"o\0";
pub const PRIoFAST64: &[u8; 4] = b"llo\0";
pub const PRIoMAX: &[u8; 3] = b"jo\0";
pub const PRIu8: &[u8; 2] = b"u\0";
pub const PRIu16: &[u8; 2] = b"u\0";
pub const PRIu32: &[u8; 2] = b"u\0";
pub const PRIu64: &[u8; 4] = b"llu\0";
pub const PRIuLEAST8: &[u8; 2] = b"u\0";
pub const PRIuLEAST16: &[u8; 2] = b"u\0";
pub const PRIuLEAST32: &[u8; 2] = b"u\0";
pub const PRIuLEAST64: &[u8; 4] = b"llu\0";
pub const PRIuFAST8: &[u8; 2] = b"u\0";
pub const PRIuFAST64: &[u8; 4] = b"llu\0";
pub const PRIuMAX: &[u8; 3] = b"ju\0";
pub const PRIx8: &[u8; 2] = b"x\0";
pub const PRIx16: &[u8; 2] = b"x\0";
pub const PRIx32: &[u8; 2] = b"x\0";
pub const PRIx64: &[u8; 4] = b"llx\0";
pub const PRIxLEAST8: &[u8; 2] = b"x\0";
pub const PRIxLEAST16: &[u8; 2] = b"x\0";
pub const PRIxLEAST32: &[u8; 2] = b"x\0";
pub const PRIxLEAST64: &[u8; 4] = b"llx\0";
pub const PRIxFAST8: &[u8; 2] = b"x\0";
pub const PRIxFAST64: &[u8; 4] = b"llx\0";
pub const PRIxMAX: &[u8; 3] = b"jx\0";
pub const PRIX8: &[u8; 2] = b"X\0";
pub const PRIX16: &[u8; 2] = b"X\0";
pub const PRIX32: &[u8; 2] = b"X\0";
pub const PRIX64: &[u8; 4] = b"llX\0";
pub const PRIXLEAST8: &[u8; 2] = b"X\0";
pub const PRIXLEAST16: &[u8; 2] = b"X\0";
pub const PRIXLEAST32: &[u8; 2] = b"X\0";
pub const PRIXLEAST64: &[u8; 4] = b"llX\0";
pub const PRIXFAST8: &[u8; 2] = b"X\0";
pub const PRIXFAST64: &[u8; 4] = b"llX\0";
pub const PRIXMAX: &[u8; 3] = b"jX\0";
pub const SCNd8: &[u8; 4] = b"hhd\0";
pub const SCNd16: &[u8; 3] = b"hd\0";
pub const SCNd32: &[u8; 2] = b"d\0";
pub const SCNd64: &[u8; 4] = b"lld\0";
pub const SCNdLEAST8: &[u8; 4] = b"hhd\0";
pub const SCNdLEAST16: &[u8; 3] = b"hd\0";
pub const SCNdLEAST32: &[u8; 2] = b"d\0";
pub const SCNdLEAST64: &[u8; 4] = b"lld\0";
pub const SCNdFAST8: &[u8; 4] = b"hhd\0";
pub const SCNdFAST64: &[u8; 4] = b"lld\0";
pub const SCNdMAX: &[u8; 3] = b"jd\0";
pub const SCNi8: &[u8; 4] = b"hhi\0";
pub const SCNi16: &[u8; 3] = b"hi\0";
pub const SCNi32: &[u8; 2] = b"i\0";
pub const SCNi64: &[u8; 4] = b"lli\0";
pub const SCNiLEAST8: &[u8; 4] = b"hhi\0";
pub const SCNiLEAST16: &[u8; 3] = b"hi\0";
pub const SCNiLEAST32: &[u8; 2] = b"i\0";
pub const SCNiLEAST64: &[u8; 4] = b"lli\0";
pub const SCNiFAST8: &[u8; 4] = b"hhi\0";
pub const SCNiFAST64: &[u8; 4] = b"lli\0";
pub const SCNiMAX: &[u8; 3] = b"ji\0";
pub const SCNb8: &[u8; 4] = b"hhb\0";
pub const SCNb16: &[u8; 3] = b"hb\0";
pub const SCNb32: &[u8; 2] = b"b\0";
pub const SCNb64: &[u8; 4] = b"llb\0";
pub const SCNbLEAST8: &[u8; 4] = b"hhb\0";
pub const SCNbLEAST16: &[u8; 3] = b"hb\0";
pub const SCNbLEAST32: &[u8; 2] = b"b\0";
pub const SCNbLEAST64: &[u8; 4] = b"llb\0";
pub const SCNbFAST8: &[u8; 4] = b"hhb\0";
pub const SCNbFAST64: &[u8; 4] = b"llb\0";
pub const SCNbMAX: &[u8; 3] = b"jb\0";
pub const SCNB8: &[u8; 4] = b"hhB\0";
pub const SCNB16: &[u8; 3] = b"hB\0";
pub const SCNB32: &[u8; 2] = b"B\0";
pub const SCNB64: &[u8; 4] = b"llB\0";
pub const SCNBLEAST8: &[u8; 4] = b"hhB\0";
pub const SCNBLEAST16: &[u8; 3] = b"hB\0";
pub const SCNBLEAST32: &[u8; 2] = b"B\0";
pub const SCNBLEAST64: &[u8; 4] = b"llB\0";
pub const SCNBFAST8: &[u8; 4] = b"hhB\0";
pub const SCNBFAST64: &[u8; 4] = b"llB\0";
pub const SCNBMAX: &[u8; 3] = b"jB\0";
pub const SCNo8: &[u8; 4] = b"hho\0";
pub const SCNo16: &[u8; 3] = b"ho\0";
pub const SCNo32: &[u8; 2] = b"o\0";
pub const SCNo64: &[u8; 4] = b"llo\0";
pub const SCNoLEAST8: &[u8; 4] = b"hho\0";
pub const SCNoLEAST16: &[u8; 3] = b"ho\0";
pub const SCNoLEAST32: &[u8; 2] = b"o\0";
pub const SCNoLEAST64: &[u8; 4] = b"llo\0";
pub const SCNoFAST8: &[u8; 4] = b"hho\0";
pub const SCNoFAST64: &[u8; 4] = b"llo\0";
pub const SCNoMAX: &[u8; 3] = b"jo\0";
pub const SCNu8: &[u8; 4] = b"hhu\0";
pub const SCNu16: &[u8; 3] = b"hu\0";
pub const SCNu32: &[u8; 2] = b"u\0";
pub const SCNu64: &[u8; 4] = b"llu\0";
pub const SCNuLEAST8: &[u8; 4] = b"hhu\0";
pub const SCNuLEAST16: &[u8; 3] = b"hu\0";
pub const SCNuLEAST32: &[u8; 2] = b"u\0";
pub const SCNuLEAST64: &[u8; 4] = b"llu\0";
pub const SCNuFAST8: &[u8; 4] = b"hhu\0";
pub const SCNuFAST64: &[u8; 4] = b"llu\0";
pub const SCNuMAX: &[u8; 3] = b"ju\0";
pub const SCNx8: &[u8; 4] = b"hhx\0";
pub const SCNx16: &[u8; 3] = b"hx\0";
pub const SCNx32: &[u8; 2] = b"x\0";
pub const SCNx64: &[u8; 4] = b"llx\0";
pub const SCNxLEAST8: &[u8; 4] = b"hhx\0";
pub const SCNxLEAST16: &[u8; 3] = b"hx\0";
pub const SCNxLEAST32: &[u8; 2] = b"x\0";
pub const SCNxLEAST64: &[u8; 4] = b"llx\0";
pub const SCNxFAST8: &[u8; 4] = b"hhx\0";
pub const SCNxFAST64: &[u8; 4] = b"llx\0";
pub const SCNxMAX: &[u8; 3] = b"jx\0";
pub const GAME_ACTIVITY_POINTER_INFO_AXIS_COUNT: u32 = 48;
pub const GAMEACTIVITY_MAX_NUM_POINTERS_IN_MOTION_EVENT: u32 = 8;
pub const GAMETEXTINPUT_MAJOR_VERSION: u32 = 4;
pub const GAMETEXTINPUT_MINOR_VERSION: u32 = 3;
pub const GAMETEXTINPUT_BUGFIX_VERSION: u32 = 0;
pub const GAMEACTIVITY_MAJOR_VERSION: u32 = 4;
pub const GAMEACTIVITY_MINOR_VERSION: u32 = 4;
pub const GAMEACTIVITY_BUGFIX_VERSION: u32 = 0;
pub const POLLIN: u32 = 1;
pub const POLLPRI: u32 = 2;
pub const POLLOUT: u32 = 4;
pub const POLLERR: u32 = 8;
pub const POLLHUP: u32 = 16;
pub const POLLNVAL: u32 = 32;
pub const POLLRDNORM: u32 = 64;
pub const POLLRDBAND: u32 = 128;
pub const POLLWRNORM: u32 = 256;
pub const POLLWRBAND: u32 = 512;
pub const POLLMSG: u32 = 1024;
pub const POLLREMOVE: u32 = 4096;
pub const POLLRDHUP: u32 = 8192;
pub const FP_XSTATE_MAGIC1: u32 = 1179670611;
pub const FP_XSTATE_MAGIC2: u32 = 1179670597;
pub const X86_FXSR_MAGIC: u32 = 0;
pub const NR_OPEN: u32 = 1024;
pub const NGROUPS_MAX: u32 = 65536;
pub const ARG_MAX: u32 = 131072;
pub const LINK_MAX: u32 = 127;
pub const MAX_CANON: u32 = 255;
pub const MAX_INPUT: u32 = 255;
pub const NAME_MAX: u32 = 255;
pub const PATH_MAX: u32 = 4096;
pub const PIPE_BUF: u32 = 4096;
pub const XATTR_NAME_MAX: u32 = 255;
pub const XATTR_SIZE_MAX: u32 = 65536;
pub const XATTR_LIST_MAX: u32 = 65536;
pub const RTSIG_MAX: u32 = 32;
pub const NL_ARGMAX: u32 = 9;
pub const NL_LANGMAX: u32 = 14;
pub const NL_MSGMAX: u32 = 32767;
pub const NL_NMAX: u32 = 1;
pub const NL_SETMAX: u32 = 255;
pub const NL_TEXTMAX: u32 = 255;
pub const PASS_MAX: u32 = 128;
pub const TMP_MAX: u32 = 308915776;
pub const LONG_BIT: u32 = 32;
pub const WORD_BIT: u32 = 32;
pub const NSIG_MAX: u32 = 65;
pub const MB_LEN_MAX: u32 = 4;
pub const NZERO: u32 = 20;
pub const IOV_MAX: u32 = 1024;
pub const SEM_VALUE_MAX: u32 = 1073741823;
pub const _POSIX_VERSION: u32 = 200809;
pub const _POSIX2_VERSION: u32 = 200809;
pub const _XOPEN_VERSION: u32 = 700;
pub const __BIONIC_POSIX_FEATURE_MISSING: i32 = -1;
pub const _POSIX_ASYNCHRONOUS_IO: i32 = -1;
pub const _POSIX_CHOWN_RESTRICTED: u32 = 1;
pub const _POSIX_CPUTIME: u32 = 200809;
pub const _POSIX_FSYNC: u32 = 200809;
pub const _POSIX_IPV6: u32 = 200809;
pub const _POSIX_MAPPED_FILES: u32 = 200809;
pub const _POSIX_MEMLOCK_RANGE: u32 = 200809;
pub const _POSIX_MEMORY_PROTECTION: u32 = 200809;
pub const _POSIX_MESSAGE_PASSING: i32 = -1;
pub const _POSIX_MONOTONIC_CLOCK: u32 = 200809;
pub const _POSIX_NO_TRUNC: u32 = 1;
pub const _POSIX_PRIORITIZED_IO: i32 = -1;
pub const _POSIX_PRIORITY_SCHEDULING: u32 = 200809;
pub const _POSIX_RAW_SOCKETS: u32 = 200809;
pub const _POSIX_READER_WRITER_LOCKS: u32 = 200809;
pub const _POSIX_REGEXP: u32 = 1;
pub const _POSIX_SAVED_IDS: u32 = 1;
pub const _POSIX_SEMAPHORES: u32 = 200809;
pub const _POSIX_SHARED_MEMORY_OBJECTS: i32 = -1;
pub const _POSIX_SHELL: u32 = 1;
pub const _POSIX_SPORADIC_SERVER: i32 = -1;
pub const _POSIX_SYNCHRONIZED_IO: u32 = 200809;
pub const _POSIX_THREAD_ATTR_STACKADDR: u32 = 200809;
pub const _POSIX_THREAD_ATTR_STACKSIZE: u32 = 200809;
pub const _POSIX_THREAD_CPUTIME: u32 = 200809;
pub const _POSIX_THREAD_PRIO_INHERIT: i32 = -1;
pub const _POSIX_THREAD_PRIO_PROTECT: i32 = -1;
pub const _POSIX_THREAD_PRIORITY_SCHEDULING: u32 = 200809;
pub const _POSIX_THREAD_PROCESS_SHARED: u32 = 200809;
pub const _POSIX_THREAD_ROBUST_PRIO_INHERIT: i32 = -1;
pub const _POSIX_THREAD_ROBUST_PRIO_PROTECT: i32 = -1;
pub const _POSIX_THREAD_SAFE_FUNCTIONS: u32 = 200809;
pub const _POSIX_THREAD_SPORADIC_SERVER: i32 = -1;
pub const _POSIX_THREADS: u32 = 200809;
pub const _POSIX_TIMERS: u32 = 200809;
pub const _POSIX_TRACE: i32 = -1;
pub const _POSIX_TRACE_EVENT_FILTER: i32 = -1;
pub const _POSIX_TRACE_INHERIT: i32 = -1;
pub const _POSIX_TRACE_LOG: i32 = -1;
pub const _POSIX_TYPED_MEMORY_OBJECTS: i32 = -1;
pub const _POSIX_VDISABLE: u8 = 0u8;
pub const _POSIX2_C_BIND: u32 = 200809;
pub const _POSIX2_C_DEV: i32 = -1;
pub const _POSIX2_CHAR_TERM: u32 = 200809;
pub const _POSIX2_FORT_DEV: i32 = -1;
pub const _POSIX2_FORT_RUN: i32 = -1;
pub const _POSIX2_LOCALEDEF: i32 = -1;
pub const _POSIX2_SW_DEV: i32 = -1;
pub const _POSIX2_UPE: i32 = -1;
pub const _POSIX_V7_ILP32_OFF32: u32 = 1;
pub const _POSIX_V7_ILP32_OFFBIG: i32 = -1;
pub const _POSIX_V7_LP64_OFF64: i32 = -1;
pub const _POSIX_V7_LPBIG_OFFBIG: i32 = -1;
pub const _XOPEN_CRYPT: i32 = -1;
pub const _XOPEN_ENH_I18N: u32 = 1;
pub const _XOPEN_LEGACY: i32 = -1;
pub const _XOPEN_REALTIME: u32 = 1;
pub const _XOPEN_REALTIME_THREADS: u32 = 1;
pub const _XOPEN_SHM: u32 = 1;
pub const _XOPEN_STREAMS: i32 = -1;
pub const _XOPEN_UNIX: u32 = 1;
pub const _POSIX_AIO_LISTIO_MAX: u32 = 2;
pub const _POSIX_AIO_MAX: u32 = 1;
pub const _POSIX_ARG_MAX: u32 = 4096;
pub const _POSIX_CHILD_MAX: u32 = 25;
pub const _POSIX_CLOCKRES_MIN: u32 = 20000000;
pub const _POSIX_DELAYTIMER_MAX: u32 = 32;
pub const _POSIX_HOST_NAME_MAX: u32 = 255;
pub const _POSIX_LINK_MAX: u32 = 8;
pub const _POSIX_LOGIN_NAME_MAX: u32 = 9;
pub const _POSIX_MAX_CANON: u32 = 255;
pub const _POSIX_MAX_INPUT: u32 = 255;
pub const _POSIX_MQ_OPEN_MAX: u32 = 8;
pub const _POSIX_MQ_PRIO_MAX: u32 = 32;
pub const _POSIX_NAME_MAX: u32 = 14;
pub const _POSIX_NGROUPS_MAX: u32 = 8;
pub const _POSIX_OPEN_MAX: u32 = 20;
pub const _POSIX_PATH_MAX: u32 = 256;
pub const _POSIX_PIPE_BUF: u32 = 512;
pub const _POSIX_RE_DUP_MAX: u32 = 255;
pub const _POSIX_RTSIG_MAX: u32 = 8;
pub const _POSIX_SEM_NSEMS_MAX: u32 = 256;
pub const _POSIX_SEM_VALUE_MAX: u32 = 32767;
pub const _POSIX_SIGQUEUE_MAX: u32 = 32;
pub const _POSIX_SSIZE_MAX: u32 = 32767;
pub const _POSIX_STREAM_MAX: u32 = 8;
pub const _POSIX_SS_REPL_MAX: u32 = 4;
pub const _POSIX_SYMLINK_MAX: u32 = 255;
pub const _POSIX_SYMLOOP_MAX: u32 = 8;
pub const _POSIX_THREAD_DESTRUCTOR_ITERATIONS: u32 = 4;
pub const _POSIX_THREAD_KEYS_MAX: u32 = 128;
pub const _POSIX_THREAD_THREADS_MAX: u32 = 64;
pub const _POSIX_TIMER_MAX: u32 = 32;
pub const _POSIX_TRACE_EVENT_NAME_MAX: u32 = 30;
pub const _POSIX_TRACE_NAME_MAX: u32 = 8;
pub const _POSIX_TRACE_SYS_MAX: u32 = 8;
pub const _POSIX_TRACE_USER_EVENT_MAX: u32 = 32;
pub const _POSIX_TTY_NAME_MAX: u32 = 9;
pub const _POSIX_TZNAME_MAX: u32 = 6;
pub const _POSIX2_BC_BASE_MAX: u32 = 99;
pub const _POSIX2_BC_DIM_MAX: u32 = 2048;
pub const _POSIX2_BC_SCALE_MAX: u32 = 99;
pub const _POSIX2_BC_STRING_MAX: u32 = 1000;
pub const _POSIX2_CHARCLASS_NAME_MAX: u32 = 14;
pub const _POSIX2_COLL_WEIGHTS_MAX: u32 = 2;
pub const _POSIX2_EXPR_NEST_MAX: u32 = 32;
pub const _POSIX2_LINE_MAX: u32 = 2048;
pub const _POSIX2_RE_DUP_MAX: u32 = 255;
pub const _XOPEN_IOV_MAX: u32 = 16;
pub const _XOPEN_NAME_MAX: u32 = 255;
pub const _XOPEN_PATH_MAX: u32 = 1024;
pub const HOST_NAME_MAX: u32 = 255;
pub const LOGIN_NAME_MAX: u32 = 256;
pub const TTY_NAME_MAX: u32 = 32;
pub const PTHREAD_DESTRUCTOR_ITERATIONS: u32 = 4;
pub const PTHREAD_KEYS_MAX: u32 = 128;
pub const _KERNEL_NSIG: u32 = 32;
pub const SIGHUP: u32 = 1;
pub const SIGINT: u32 = 2;
pub const SIGQUIT: u32 = 3;
pub const SIGILL: u32 = 4;
pub const SIGTRAP: u32 = 5;
pub const SIGABRT: u32 = 6;
pub const SIGIOT: u32 = 6;
pub const SIGBUS: u32 = 7;
pub const SIGFPE: u32 = 8;
pub const SIGKILL: u32 = 9;
pub const SIGUSR1: u32 = 10;
pub const SIGSEGV: u32 = 11;
pub const SIGUSR2: u32 = 12;
pub const SIGPIPE: u32 = 13;
pub const SIGALRM: u32 = 14;
pub const SIGTERM: u32 = 15;
pub const SIGSTKFLT: u32 = 16;
pub const SIGCHLD: u32 = 17;
pub const SIGCONT: u32 = 18;
pub const SIGSTOP: u32 = 19;
pub const SIGTSTP: u32 = 20;
pub const SIGTTIN: u32 = 21;
pub const SIGTTOU: u32 = 22;
pub const SIGURG: u32 = 23;
pub const SIGXCPU: u32 = 24;
pub const SIGXFSZ: u32 = 25;
pub const SIGVTALRM: u32 = 26;
pub const SIGPROF: u32 = 27;
pub const SIGWINCH: u32 = 28;
pub const SIGIO: u32 = 29;
pub const SIGPOLL: u32 = 29;
pub const SIGPWR: u32 = 30;
pub const SIGSYS: u32 = 31;
pub const SIGUNUSED: u32 = 31;
pub const __SIGRTMIN: u32 = 32;
pub const SA_RESTORER: u32 = 67108864;
pub const MINSIGSTKSZ: u32 = 2048;
pub const SIGSTKSZ: u32 = 8192;
pub const SA_NOCLDSTOP: u32 = 1;
pub const SA_NOCLDWAIT: u32 = 2;
pub const SA_SIGINFO: u32 = 4;
pub const SA_UNSUPPORTED: u32 = 1024;
pub const SA_EXPOSE_TAGBITS: u32 = 2048;
pub const SA_ONSTACK: u32 = 134217728;
pub const SA_RESTART: u32 = 268435456;
pub const SA_NODEFER: u32 = 1073741824;
pub const SA_RESETHAND: u32 = 2147483648;
pub const SA_NOMASK: u32 = 1073741824;
pub const SA_ONESHOT: u32 = 2147483648;
pub const SIG_BLOCK: u32 = 0;
pub const SIG_UNBLOCK: u32 = 1;
pub const SIG_SETMASK: u32 = 2;
pub const SI_MAX_SIZE: u32 = 128;
pub const SI_USER: u32 = 0;
pub const SI_KERNEL: u32 = 128;
pub const SI_QUEUE: i32 = -1;
pub const SI_TIMER: i32 = -2;
pub const SI_MESGQ: i32 = -3;
pub const SI_ASYNCIO: i32 = -4;
pub const SI_SIGIO: i32 = -5;
pub const SI_TKILL: i32 = -6;
pub const SI_DETHREAD: i32 = -7;
pub const SI_ASYNCNL: i32 = -60;
pub const ILL_ILLOPC: u32 = 1;
pub const ILL_ILLOPN: u32 = 2;
pub const ILL_ILLADR: u32 = 3;
pub const ILL_ILLTRP: u32 = 4;
pub const ILL_PRVOPC: u32 = 5;
pub const ILL_PRVREG: u32 = 6;
pub const ILL_COPROC: u32 = 7;
pub const ILL_BADSTK: u32 = 8;
pub const ILL_BADIADDR: u32 = 9;
pub const __ILL_BREAK: u32 = 10;
pub const __ILL_BNDMOD: u32 = 11;
pub const NSIGILL: u32 = 11;
pub const FPE_INTDIV: u32 = 1;
pub const FPE_INTOVF: u32 = 2;
pub const FPE_FLTDIV: u32 = 3;
pub const FPE_FLTOVF: u32 = 4;
pub const FPE_FLTUND: u32 = 5;
pub const FPE_FLTRES: u32 = 6;
pub const FPE_FLTINV: u32 = 7;
pub const FPE_FLTSUB: u32 = 8;
pub const __FPE_DECOVF: u32 = 9;
pub const __FPE_DECDIV: u32 = 10;
pub const __FPE_DECERR: u32 = 11;
pub const __FPE_INVASC: u32 = 12;
pub const __FPE_INVDEC: u32 = 13;
pub const FPE_FLTUNK: u32 = 14;
pub const FPE_CONDTRAP: u32 = 15;
pub const NSIGFPE: u32 = 15;
pub const SEGV_MAPERR: u32 = 1;
pub const SEGV_ACCERR: u32 = 2;
pub const SEGV_BNDERR: u32 = 3;
pub const SEGV_PKUERR: u32 = 4;
pub const SEGV_ACCADI: u32 = 5;
pub const SEGV_ADIDERR: u32 = 6;
pub const SEGV_ADIPERR: u32 = 7;
pub const SEGV_MTEAERR: u32 = 8;
pub const SEGV_MTESERR: u32 = 9;
pub const SEGV_CPERR: u32 = 10;
pub const NSIGSEGV: u32 = 10;
pub const BUS_ADRALN: u32 = 1;
pub const BUS_ADRERR: u32 = 2;
pub const BUS_OBJERR: u32 = 3;
pub const BUS_MCEERR_AR: u32 = 4;
pub const BUS_MCEERR_AO: u32 = 5;
pub const NSIGBUS: u32 = 5;
pub const TRAP_BRKPT: u32 = 1;
pub const TRAP_TRACE: u32 = 2;
pub const TRAP_BRANCH: u32 = 3;
pub const TRAP_HWBKPT: u32 = 4;
pub const TRAP_UNK: u32 = 5;
pub const TRAP_PERF: u32 = 6;
pub const NSIGTRAP: u32 = 6;
pub const TRAP_PERF_FLAG_ASYNC: u32 = 1;
pub const CLD_EXITED: u32 = 1;
pub const CLD_KILLED: u32 = 2;
pub const CLD_DUMPED: u32 = 3;
pub const CLD_TRAPPED: u32 = 4;
pub const CLD_STOPPED: u32 = 5;
pub const CLD_CONTINUED: u32 = 6;
pub const NSIGCHLD: u32 = 6;
pub const POLL_IN: u32 = 1;
pub const POLL_OUT: u32 = 2;
pub const POLL_MSG: u32 = 3;
pub const POLL_ERR: u32 = 4;
pub const POLL_PRI: u32 = 5;
pub const POLL_HUP: u32 = 6;
pub const NSIGPOLL: u32 = 6;
pub const SYS_SECCOMP: u32 = 1;
pub const SYS_USER_DISPATCH: u32 = 2;
pub const NSIGSYS: u32 = 2;
pub const EMT_TAGOVF: u32 = 1;
pub const NSIGEMT: u32 = 1;
pub const SIGEV_SIGNAL: u32 = 0;
pub const SIGEV_NONE: u32 = 1;
pub const SIGEV_THREAD: u32 = 2;
pub const SIGEV_THREAD_ID: u32 = 4;
pub const SIGEV_MAX_SIZE: u32 = 64;
pub const SS_ONSTACK: u32 = 1;
pub const SS_DISABLE: u32 = 2;
pub const SS_AUTODISARM: u32 = 2147483648;
pub const SS_FLAG_BITS: u32 = 2147483648;
pub const NSIG: u32 = 65;
pub const _NSIG: u32 = 65;
pub const _KERNEL__NSIG: u32 = 64;
pub const PAGE_SIZE: u32 = 4096;
pub const PAGE_MASK: i32 = -4096;
pub const UPAGES: u32 = 1;
pub const SIG2STR_MAX: u32 = 32;
pub const ITIMER_REAL: u32 = 0;
pub const ITIMER_VIRTUAL: u32 = 1;
pub const ITIMER_PROF: u32 = 2;
pub const CLOCK_REALTIME: u32 = 0;
pub const CLOCK_MONOTONIC: u32 = 1;
pub const CLOCK_PROCESS_CPUTIME_ID: u32 = 2;
pub const CLOCK_THREAD_CPUTIME_ID: u32 = 3;
pub const CLOCK_MONOTONIC_RAW: u32 = 4;
pub const CLOCK_REALTIME_COARSE: u32 = 5;
pub const CLOCK_MONOTONIC_COARSE: u32 = 6;
pub const CLOCK_BOOTTIME: u32 = 7;
pub const CLOCK_REALTIME_ALARM: u32 = 8;
pub const CLOCK_BOOTTIME_ALARM: u32 = 9;
pub const CLOCK_SGI_CYCLE: u32 = 10;
pub const CLOCK_TAI: u32 = 11;
pub const MAX_CLOCKS: u32 = 16;
pub const CLOCKS_MASK: u32 = 1;
pub const CLOCKS_MONO: u32 = 1;
pub const TIMER_ABSTIME: u32 = 1;
pub const FD_SETSIZE: u32 = 1024;
pub const CLOCKS_PER_SEC: u32 = 1000000;
pub const TIME_UTC: u32 = 1;
pub const TIME_MONOTONIC: u32 = 2;
pub const TIME_ACTIVE: u32 = 3;
pub const TIME_THREAD_ACTIVE: u32 = 4;
pub const CSIGNAL: u32 = 255;
pub const CLONE_VM: u32 = 256;
pub const CLONE_FS: u32 = 512;
pub const CLONE_FILES: u32 = 1024;
pub const CLONE_SIGHAND: u32 = 2048;
pub const CLONE_PIDFD: u32 = 4096;
pub const CLONE_PTRACE: u32 = 8192;
pub const CLONE_VFORK: u32 = 16384;
pub const CLONE_PARENT: u32 = 32768;
pub const CLONE_THREAD: u32 = 65536;
pub const CLONE_NEWNS: u32 = 131072;
pub const CLONE_SYSVSEM: u32 = 262144;
pub const CLONE_SETTLS: u32 = 524288;
pub const CLONE_PARENT_SETTID: u32 = 1048576;
pub const CLONE_CHILD_CLEARTID: u32 = 2097152;
pub const CLONE_DETACHED: u32 = 4194304;
pub const CLONE_UNTRACED: u32 = 8388608;
pub const CLONE_CHILD_SETTID: u32 = 16777216;
pub const CLONE_NEWCGROUP: u32 = 33554432;
pub const CLONE_NEWUTS: u32 = 67108864;
pub const CLONE_NEWIPC: u32 = 134217728;
pub const CLONE_NEWUSER: u32 = 268435456;
pub const CLONE_NEWPID: u32 = 536870912;
pub const CLONE_NEWNET: u32 = 1073741824;
pub const CLONE_IO: u32 = 2147483648;
pub const CLONE_CLEAR_SIGHAND: u64 = 4294967296;
pub const CLONE_INTO_CGROUP: u64 = 8589934592;
pub const CLONE_NEWTIME: u32 = 128;
pub const CLONE_ARGS_SIZE_VER0: u32 = 64;
pub const CLONE_ARGS_SIZE_VER1: u32 = 80;
pub const CLONE_ARGS_SIZE_VER2: u32 = 88;
pub const SCHED_NORMAL: u32 = 0;
pub const SCHED_FIFO: u32 = 1;
pub const SCHED_RR: u32 = 2;
pub const SCHED_BATCH: u32 = 3;
pub const SCHED_IDLE: u32 = 5;
pub const SCHED_DEADLINE: u32 = 6;
pub const SCHED_EXT: u32 = 7;
pub const SCHED_RESET_ON_FORK: u32 = 1073741824;
pub const SCHED_FLAG_RESET_ON_FORK: u32 = 1;
pub const SCHED_FLAG_RECLAIM: u32 = 2;
pub const SCHED_FLAG_DL_OVERRUN: u32 = 4;
pub const SCHED_FLAG_KEEP_POLICY: u32 = 8;
pub const SCHED_FLAG_KEEP_PARAMS: u32 = 16;
pub const SCHED_FLAG_UTIL_CLAMP_MIN: u32 = 32;
pub const SCHED_FLAG_UTIL_CLAMP_MAX: u32 = 64;
pub const SCHED_FLAG_KEEP_ALL: u32 = 24;
pub const SCHED_FLAG_UTIL_CLAMP: u32 = 96;
pub const SCHED_FLAG_ALL: u32 = 127;
pub const SCHED_ATTR_SIZE_VER0: u32 = 48;
pub const SCHED_ATTR_SIZE_VER1: u32 = 56;
pub const SCHED_OTHER: u32 = 0;
pub const PTHREAD_ONCE_INIT: u32 = 0;
pub const PTHREAD_BARRIER_SERIAL_THREAD: i32 = -1;
pub const PTHREAD_STACK_MIN: u32 = 8192;
pub const PTHREAD_CREATE_DETACHED: u32 = 1;
pub const PTHREAD_CREATE_JOINABLE: u32 = 0;
pub const PTHREAD_EXPLICIT_SCHED: u32 = 0;
pub const PTHREAD_INHERIT_SCHED: u32 = 1;
pub const PTHREAD_PRIO_NONE: u32 = 0;
pub const PTHREAD_PRIO_INHERIT: u32 = 1;
pub const PTHREAD_PROCESS_PRIVATE: u32 = 0;
pub const PTHREAD_PROCESS_SHARED: u32 = 1;
pub const PTHREAD_SCOPE_SYSTEM: u32 = 0;
pub const PTHREAD_SCOPE_PROCESS: u32 = 1;
pub const NATIVE_APP_GLUE_MAX_INPUT_BUFFERS: u32 = 2;
unsafe extern "C" {
    pub fn android_get_device_api_level() -> ::std::os::raw::c_int;
}
pub type wchar_t = ::std::os::raw::c_int;
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone)]
pub struct max_align_t {
    pub __clang_max_align_nonce1: ::std::os::raw::c_longlong,
    pub __clang_max_align_nonce2: f64,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of max_align_t"][::std::mem::size_of::<max_align_t>() - 16usize];
    ["Alignment of max_align_t"][::std::mem::align_of::<max_align_t>() - 8usize];
    ["Offset of field: max_align_t::__clang_max_align_nonce1"]
        [::std::mem::offset_of!(max_align_t, __clang_max_align_nonce1) - 0usize];
    ["Offset of field: max_align_t::__clang_max_align_nonce2"]
        [::std::mem::offset_of!(max_align_t, __clang_max_align_nonce2) - 8usize];
};
pub type __int8_t = ::std::os::raw::c_schar;
pub type __uint8_t = ::std::os::raw::c_uchar;
pub type __int16_t = ::std::os::raw::c_short;
pub type __uint16_t = ::std::os::raw::c_ushort;
pub type __int32_t = ::std::os::raw::c_int;
pub type __uint32_t = ::std::os::raw::c_uint;
pub type __int64_t = ::std::os::raw::c_longlong;
pub type __uint64_t = ::std::os::raw::c_ulonglong;
pub type __intptr_t = ::std::os::raw::c_int;
pub type __uintptr_t = ::std::os::raw::c_uint;
pub type int_least8_t = i8;
pub type uint_least8_t = u8;
pub type int_least16_t = i16;
pub type uint_least16_t = u16;
pub type int_least32_t = i32;
pub type uint_least32_t = u32;
pub type int_least64_t = i64;
pub type uint_least64_t = u64;
pub type int_fast8_t = i8;
pub type uint_fast8_t = u8;
pub type int_fast64_t = i64;
pub type uint_fast64_t = u64;
pub type int_fast16_t = i32;
pub type uint_fast16_t = u32;
pub type int_fast32_t = i32;
pub type uint_fast32_t = u32;
pub type uintmax_t = u64;
pub type intmax_t = i64;
pub type __s8 = ::std::os::raw::c_schar;
pub type __u8 = ::std::os::raw::c_uchar;
pub type __s16 = ::std::os::raw::c_short;
pub type __u16 = ::std::os::raw::c_ushort;
pub type __s32 = ::std::os::raw::c_int;
pub type __u32 = ::std::os::raw::c_uint;
pub type __s64 = ::std::os::raw::c_longlong;
pub type __u64 = ::std::os::raw::c_ulonglong;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __kernel_fd_set {
    pub fds_bits: [::std::os::raw::c_ulong; 32usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_fd_set"][::std::mem::size_of::<__kernel_fd_set>() - 128usize];
    ["Alignment of __kernel_fd_set"][::std::mem::align_of::<__kernel_fd_set>() - 4usize];
    ["Offset of field: __kernel_fd_set::fds_bits"]
        [::std::mem::offset_of!(__kernel_fd_set, fds_bits) - 0usize];
};
pub type __kernel_sighandler_t =
    ::std::option::Option<unsafe extern "C" fn(arg1: ::std::os::raw::c_int)>;
pub type __kernel_key_t = ::std::os::raw::c_int;
pub type __kernel_mqd_t = ::std::os::raw::c_int;
pub type __kernel_mode_t = ::std::os::raw::c_ushort;
pub type __kernel_ipc_pid_t = ::std::os::raw::c_ushort;
pub type __kernel_uid_t = ::std::os::raw::c_ushort;
pub type __kernel_gid_t = ::std::os::raw::c_ushort;
pub type __kernel_old_dev_t = ::std::os::raw::c_ushort;
pub type __kernel_long_t = ::std::os::raw::c_long;
pub type __kernel_ulong_t = ::std::os::raw::c_ulong;
pub type __kernel_ino_t = __kernel_ulong_t;
pub type __kernel_pid_t = ::std::os::raw::c_int;
pub type __kernel_suseconds_t = __kernel_long_t;
pub type __kernel_daddr_t = ::std::os::raw::c_int;
pub type __kernel_uid32_t = ::std::os::raw::c_uint;
pub type __kernel_gid32_t = ::std::os::raw::c_uint;
pub type __kernel_old_uid_t = __kernel_uid_t;
pub type __kernel_old_gid_t = __kernel_gid_t;
pub type __kernel_size_t = ::std::os::raw::c_uint;
pub type __kernel_ssize_t = ::std::os::raw::c_int;
pub type __kernel_ptrdiff_t = ::std::os::raw::c_int;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __kernel_fsid_t {
    pub val: [::std::os::raw::c_int; 2usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_fsid_t"][::std::mem::size_of::<__kernel_fsid_t>() - 8usize];
    ["Alignment of __kernel_fsid_t"][::std::mem::align_of::<__kernel_fsid_t>() - 4usize];
    ["Offset of field: __kernel_fsid_t::val"]
        [::std::mem::offset_of!(__kernel_fsid_t, val) - 0usize];
};
pub type __kernel_off_t = __kernel_long_t;
pub type __kernel_loff_t = ::std::os::raw::c_longlong;
pub type __kernel_old_time_t = __kernel_long_t;
pub type __kernel_time_t = __kernel_long_t;
pub type __kernel_time64_t = ::std::os::raw::c_longlong;
pub type __kernel_clock_t = __kernel_long_t;
pub type __kernel_timer_t = ::std::os::raw::c_int;
pub type __kernel_clockid_t = ::std::os::raw::c_int;
pub type __kernel_caddr_t = *mut ::std::os::raw::c_char;
pub type __kernel_uid16_t = ::std::os::raw::c_ushort;
pub type __kernel_gid16_t = ::std::os::raw::c_ushort;
pub type __le16 = __u16;
pub type __be16 = __u16;
pub type __le32 = __u32;
pub type __be32 = __u32;
pub type __le64 = __u64;
pub type __be64 = __u64;
pub type __sum16 = __u16;
pub type __wsum = __u32;
pub type __poll_t = ::std::os::raw::c_uint;
pub type __gid_t = __kernel_gid32_t;
pub type gid_t = __gid_t;
pub type __uid_t = __kernel_uid32_t;
pub type uid_t = __uid_t;
pub type __pid_t = __kernel_pid_t;
pub type pid_t = __pid_t;
pub type __id_t = u32;
pub type id_t = __id_t;
pub type blkcnt_t = ::std::os::raw::c_ulong;
pub type blksize_t = ::std::os::raw::c_ulong;
pub type caddr_t = __kernel_caddr_t;
pub type clock_t = __kernel_clock_t;
pub type __clockid_t = __kernel_clockid_t;
pub type clockid_t = __clockid_t;
pub type daddr_t = __kernel_daddr_t;
pub type fsblkcnt_t = ::std::os::raw::c_ulong;
pub type fsfilcnt_t = ::std::os::raw::c_ulong;
pub type __mode_t = __kernel_mode_t;
pub type mode_t = __mode_t;
pub type __key_t = __kernel_key_t;
pub type key_t = __key_t;
pub type __ino_t = __kernel_ino_t;
pub type ino_t = __ino_t;
pub type ino64_t = u64;
pub type __nlink_t = u32;
pub type nlink_t = __nlink_t;
pub type __timer_t = *mut ::std::os::raw::c_void;
pub type timer_t = __timer_t;
pub type __suseconds_t = __kernel_suseconds_t;
pub type suseconds_t = __suseconds_t;
pub type __useconds_t = u32;
pub type useconds_t = __useconds_t;
pub type dev_t = u32;
pub type __time_t = __kernel_time_t;
pub type time_t = __time_t;
pub type off_t = __kernel_off_t;
pub type loff_t = __kernel_loff_t;
pub type off64_t = loff_t;
pub type __socklen_t = i32;
pub type socklen_t = __socklen_t;
pub type __va_list = __builtin_va_list;
pub type uint_t = ::std::os::raw::c_uint;
pub type uint = ::std::os::raw::c_uint;
pub type u_char = ::std::os::raw::c_uchar;
pub type u_short = ::std::os::raw::c_ushort;
pub type u_int = ::std::os::raw::c_uint;
pub type u_long = ::std::os::raw::c_ulong;
pub type u_int32_t = u32;
pub type u_int16_t = u16;
pub type u_int8_t = u8;
pub type u_int64_t = u64;
pub const AASSET_MODE_UNKNOWN: _bindgen_ty_1 = 0;
pub const AASSET_MODE_RANDOM: _bindgen_ty_1 = 1;
pub const AASSET_MODE_STREAMING: _bindgen_ty_1 = 2;
pub const AASSET_MODE_BUFFER: _bindgen_ty_1 = 3;
pub type _bindgen_ty_1 = ::std::os::raw::c_uint;
pub const AKEYCODE_UNKNOWN: _bindgen_ty_2 = 0;
pub const AKEYCODE_SOFT_LEFT: _bindgen_ty_2 = 1;
pub const AKEYCODE_SOFT_RIGHT: _bindgen_ty_2 = 2;
pub const AKEYCODE_HOME: _bindgen_ty_2 = 3;
pub const AKEYCODE_BACK: _bindgen_ty_2 = 4;
pub const AKEYCODE_CALL: _bindgen_ty_2 = 5;
pub const AKEYCODE_ENDCALL: _bindgen_ty_2 = 6;
pub const AKEYCODE_0: _bindgen_ty_2 = 7;
pub const AKEYCODE_1: _bindgen_ty_2 = 8;
pub const AKEYCODE_2: _bindgen_ty_2 = 9;
pub const AKEYCODE_3: _bindgen_ty_2 = 10;
pub const AKEYCODE_4: _bindgen_ty_2 = 11;
pub const AKEYCODE_5: _bindgen_ty_2 = 12;
pub const AKEYCODE_6: _bindgen_ty_2 = 13;
pub const AKEYCODE_7: _bindgen_ty_2 = 14;
pub const AKEYCODE_8: _bindgen_ty_2 = 15;
pub const AKEYCODE_9: _bindgen_ty_2 = 16;
pub const AKEYCODE_STAR: _bindgen_ty_2 = 17;
pub const AKEYCODE_POUND: _bindgen_ty_2 = 18;
pub const AKEYCODE_DPAD_UP: _bindgen_ty_2 = 19;
pub const AKEYCODE_DPAD_DOWN: _bindgen_ty_2 = 20;
pub const AKEYCODE_DPAD_LEFT: _bindgen_ty_2 = 21;
pub const AKEYCODE_DPAD_RIGHT: _bindgen_ty_2 = 22;
pub const AKEYCODE_DPAD_CENTER: _bindgen_ty_2 = 23;
pub const AKEYCODE_VOLUME_UP: _bindgen_ty_2 = 24;
pub const AKEYCODE_VOLUME_DOWN: _bindgen_ty_2 = 25;
pub const AKEYCODE_POWER: _bindgen_ty_2 = 26;
pub const AKEYCODE_CAMERA: _bindgen_ty_2 = 27;
pub const AKEYCODE_CLEAR: _bindgen_ty_2 = 28;
pub const AKEYCODE_A: _bindgen_ty_2 = 29;
pub const AKEYCODE_B: _bindgen_ty_2 = 30;
pub const AKEYCODE_C: _bindgen_ty_2 = 31;
pub const AKEYCODE_D: _bindgen_ty_2 = 32;
pub const AKEYCODE_E: _bindgen_ty_2 = 33;
pub const AKEYCODE_F: _bindgen_ty_2 = 34;
pub const AKEYCODE_G: _bindgen_ty_2 = 35;
pub const AKEYCODE_H: _bindgen_ty_2 = 36;
pub const AKEYCODE_I: _bindgen_ty_2 = 37;
pub const AKEYCODE_J: _bindgen_ty_2 = 38;
pub const AKEYCODE_K: _bindgen_ty_2 = 39;
pub const AKEYCODE_L: _bindgen_ty_2 = 40;
pub const AKEYCODE_M: _bindgen_ty_2 = 41;
pub const AKEYCODE_N: _bindgen_ty_2 = 42;
pub const AKEYCODE_O: _bindgen_ty_2 = 43;
pub const AKEYCODE_P: _bindgen_ty_2 = 44;
pub const AKEYCODE_Q: _bindgen_ty_2 = 45;
pub const AKEYCODE_R: _bindgen_ty_2 = 46;
pub const AKEYCODE_S: _bindgen_ty_2 = 47;
pub const AKEYCODE_T: _bindgen_ty_2 = 48;
pub const AKEYCODE_U: _bindgen_ty_2 = 49;
pub const AKEYCODE_V: _bindgen_ty_2 = 50;
pub const AKEYCODE_W: _bindgen_ty_2 = 51;
pub const AKEYCODE_X: _bindgen_ty_2 = 52;
pub const AKEYCODE_Y: _bindgen_ty_2 = 53;
pub const AKEYCODE_Z: _bindgen_ty_2 = 54;
pub const AKEYCODE_COMMA: _bindgen_ty_2 = 55;
pub const AKEYCODE_PERIOD: _bindgen_ty_2 = 56;
pub const AKEYCODE_ALT_LEFT: _bindgen_ty_2 = 57;
pub const AKEYCODE_ALT_RIGHT: _bindgen_ty_2 = 58;
pub const AKEYCODE_SHIFT_LEFT: _bindgen_ty_2 = 59;
pub const AKEYCODE_SHIFT_RIGHT: _bindgen_ty_2 = 60;
pub const AKEYCODE_TAB: _bindgen_ty_2 = 61;
pub const AKEYCODE_SPACE: _bindgen_ty_2 = 62;
pub const AKEYCODE_SYM: _bindgen_ty_2 = 63;
pub const AKEYCODE_EXPLORER: _bindgen_ty_2 = 64;
pub const AKEYCODE_ENVELOPE: _bindgen_ty_2 = 65;
pub const AKEYCODE_ENTER: _bindgen_ty_2 = 66;
pub const AKEYCODE_DEL: _bindgen_ty_2 = 67;
pub const AKEYCODE_GRAVE: _bindgen_ty_2 = 68;
pub const AKEYCODE_MINUS: _bindgen_ty_2 = 69;
pub const AKEYCODE_EQUALS: _bindgen_ty_2 = 70;
pub const AKEYCODE_LEFT_BRACKET: _bindgen_ty_2 = 71;
pub const AKEYCODE_RIGHT_BRACKET: _bindgen_ty_2 = 72;
pub const AKEYCODE_BACKSLASH: _bindgen_ty_2 = 73;
pub const AKEYCODE_SEMICOLON: _bindgen_ty_2 = 74;
pub const AKEYCODE_APOSTROPHE: _bindgen_ty_2 = 75;
pub const AKEYCODE_SLASH: _bindgen_ty_2 = 76;
pub const AKEYCODE_AT: _bindgen_ty_2 = 77;
pub const AKEYCODE_NUM: _bindgen_ty_2 = 78;
pub const AKEYCODE_HEADSETHOOK: _bindgen_ty_2 = 79;
pub const AKEYCODE_FOCUS: _bindgen_ty_2 = 80;
pub const AKEYCODE_PLUS: _bindgen_ty_2 = 81;
pub const AKEYCODE_MENU: _bindgen_ty_2 = 82;
pub const AKEYCODE_NOTIFICATION: _bindgen_ty_2 = 83;
pub const AKEYCODE_SEARCH: _bindgen_ty_2 = 84;
pub const AKEYCODE_MEDIA_PLAY_PAUSE: _bindgen_ty_2 = 85;
pub const AKEYCODE_MEDIA_STOP: _bindgen_ty_2 = 86;
pub const AKEYCODE_MEDIA_NEXT: _bindgen_ty_2 = 87;
pub const AKEYCODE_MEDIA_PREVIOUS: _bindgen_ty_2 = 88;
pub const AKEYCODE_MEDIA_REWIND: _bindgen_ty_2 = 89;
pub const AKEYCODE_MEDIA_FAST_FORWARD: _bindgen_ty_2 = 90;
pub const AKEYCODE_MUTE: _bindgen_ty_2 = 91;
pub const AKEYCODE_PAGE_UP: _bindgen_ty_2 = 92;
pub const AKEYCODE_PAGE_DOWN: _bindgen_ty_2 = 93;
pub const AKEYCODE_PICTSYMBOLS: _bindgen_ty_2 = 94;
pub const AKEYCODE_SWITCH_CHARSET: _bindgen_ty_2 = 95;
pub const AKEYCODE_BUTTON_A: _bindgen_ty_2 = 96;
pub const AKEYCODE_BUTTON_B: _bindgen_ty_2 = 97;
pub const AKEYCODE_BUTTON_C: _bindgen_ty_2 = 98;
pub const AKEYCODE_BUTTON_X: _bindgen_ty_2 = 99;
pub const AKEYCODE_BUTTON_Y: _bindgen_ty_2 = 100;
pub const AKEYCODE_BUTTON_Z: _bindgen_ty_2 = 101;
pub const AKEYCODE_BUTTON_L1: _bindgen_ty_2 = 102;
pub const AKEYCODE_BUTTON_R1: _bindgen_ty_2 = 103;
pub const AKEYCODE_BUTTON_L2: _bindgen_ty_2 = 104;
pub const AKEYCODE_BUTTON_R2: _bindgen_ty_2 = 105;
pub const AKEYCODE_BUTTON_THUMBL: _bindgen_ty_2 = 106;
pub const AKEYCODE_BUTTON_THUMBR: _bindgen_ty_2 = 107;
pub const AKEYCODE_BUTTON_START: _bindgen_ty_2 = 108;
pub const AKEYCODE_BUTTON_SELECT: _bindgen_ty_2 = 109;
pub const AKEYCODE_BUTTON_MODE: _bindgen_ty_2 = 110;
pub const AKEYCODE_ESCAPE: _bindgen_ty_2 = 111;
pub const AKEYCODE_FORWARD_DEL: _bindgen_ty_2 = 112;
pub const AKEYCODE_CTRL_LEFT: _bindgen_ty_2 = 113;
pub const AKEYCODE_CTRL_RIGHT: _bindgen_ty_2 = 114;
pub const AKEYCODE_CAPS_LOCK: _bindgen_ty_2 = 115;
pub const AKEYCODE_SCROLL_LOCK: _bindgen_ty_2 = 116;
pub const AKEYCODE_META_LEFT: _bindgen_ty_2 = 117;
pub const AKEYCODE_META_RIGHT: _bindgen_ty_2 = 118;
pub const AKEYCODE_FUNCTION: _bindgen_ty_2 = 119;
pub const AKEYCODE_SYSRQ: _bindgen_ty_2 = 120;
pub const AKEYCODE_BREAK: _bindgen_ty_2 = 121;
pub const AKEYCODE_MOVE_HOME: _bindgen_ty_2 = 122;
pub const AKEYCODE_MOVE_END: _bindgen_ty_2 = 123;
pub const AKEYCODE_INSERT: _bindgen_ty_2 = 124;
pub const AKEYCODE_FORWARD: _bindgen_ty_2 = 125;
pub const AKEYCODE_MEDIA_PLAY: _bindgen_ty_2 = 126;
pub const AKEYCODE_MEDIA_PAUSE: _bindgen_ty_2 = 127;
pub const AKEYCODE_MEDIA_CLOSE: _bindgen_ty_2 = 128;
pub const AKEYCODE_MEDIA_EJECT: _bindgen_ty_2 = 129;
pub const AKEYCODE_MEDIA_RECORD: _bindgen_ty_2 = 130;
pub const AKEYCODE_F1: _bindgen_ty_2 = 131;
pub const AKEYCODE_F2: _bindgen_ty_2 = 132;
pub const AKEYCODE_F3: _bindgen_ty_2 = 133;
pub const AKEYCODE_F4: _bindgen_ty_2 = 134;
pub const AKEYCODE_F5: _bindgen_ty_2 = 135;
pub const AKEYCODE_F6: _bindgen_ty_2 = 136;
pub const AKEYCODE_F7: _bindgen_ty_2 = 137;
pub const AKEYCODE_F8: _bindgen_ty_2 = 138;
pub const AKEYCODE_F9: _bindgen_ty_2 = 139;
pub const AKEYCODE_F10: _bindgen_ty_2 = 140;
pub const AKEYCODE_F11: _bindgen_ty_2 = 141;
pub const AKEYCODE_F12: _bindgen_ty_2 = 142;
pub const AKEYCODE_NUM_LOCK: _bindgen_ty_2 = 143;
pub const AKEYCODE_NUMPAD_0: _bindgen_ty_2 = 144;
pub const AKEYCODE_NUMPAD_1: _bindgen_ty_2 = 145;
pub const AKEYCODE_NUMPAD_2: _bindgen_ty_2 = 146;
pub const AKEYCODE_NUMPAD_3: _bindgen_ty_2 = 147;
pub const AKEYCODE_NUMPAD_4: _bindgen_ty_2 = 148;
pub const AKEYCODE_NUMPAD_5: _bindgen_ty_2 = 149;
pub const AKEYCODE_NUMPAD_6: _bindgen_ty_2 = 150;
pub const AKEYCODE_NUMPAD_7: _bindgen_ty_2 = 151;
pub const AKEYCODE_NUMPAD_8: _bindgen_ty_2 = 152;
pub const AKEYCODE_NUMPAD_9: _bindgen_ty_2 = 153;
pub const AKEYCODE_NUMPAD_DIVIDE: _bindgen_ty_2 = 154;
pub const AKEYCODE_NUMPAD_MULTIPLY: _bindgen_ty_2 = 155;
pub const AKEYCODE_NUMPAD_SUBTRACT: _bindgen_ty_2 = 156;
pub const AKEYCODE_NUMPAD_ADD: _bindgen_ty_2 = 157;
pub const AKEYCODE_NUMPAD_DOT: _bindgen_ty_2 = 158;
pub const AKEYCODE_NUMPAD_COMMA: _bindgen_ty_2 = 159;
pub const AKEYCODE_NUMPAD_ENTER: _bindgen_ty_2 = 160;
pub const AKEYCODE_NUMPAD_EQUALS: _bindgen_ty_2 = 161;
pub const AKEYCODE_NUMPAD_LEFT_PAREN: _bindgen_ty_2 = 162;
pub const AKEYCODE_NUMPAD_RIGHT_PAREN: _bindgen_ty_2 = 163;
pub const AKEYCODE_VOLUME_MUTE: _bindgen_ty_2 = 164;
pub const AKEYCODE_INFO: _bindgen_ty_2 = 165;
pub const AKEYCODE_CHANNEL_UP: _bindgen_ty_2 = 166;
pub const AKEYCODE_CHANNEL_DOWN: _bindgen_ty_2 = 167;
pub const AKEYCODE_ZOOM_IN: _bindgen_ty_2 = 168;
pub const AKEYCODE_ZOOM_OUT: _bindgen_ty_2 = 169;
pub const AKEYCODE_TV: _bindgen_ty_2 = 170;
pub const AKEYCODE_WINDOW: _bindgen_ty_2 = 171;
pub const AKEYCODE_GUIDE: _bindgen_ty_2 = 172;
pub const AKEYCODE_DVR: _bindgen_ty_2 = 173;
pub const AKEYCODE_BOOKMARK: _bindgen_ty_2 = 174;
pub const AKEYCODE_CAPTIONS: _bindgen_ty_2 = 175;
pub const AKEYCODE_SETTINGS: _bindgen_ty_2 = 176;
pub const AKEYCODE_TV_POWER: _bindgen_ty_2 = 177;
pub const AKEYCODE_TV_INPUT: _bindgen_ty_2 = 178;
pub const AKEYCODE_STB_POWER: _bindgen_ty_2 = 179;
pub const AKEYCODE_STB_INPUT: _bindgen_ty_2 = 180;
pub const AKEYCODE_AVR_POWER: _bindgen_ty_2 = 181;
pub const AKEYCODE_AVR_INPUT: _bindgen_ty_2 = 182;
pub const AKEYCODE_PROG_RED: _bindgen_ty_2 = 183;
pub const AKEYCODE_PROG_GREEN: _bindgen_ty_2 = 184;
pub const AKEYCODE_PROG_YELLOW: _bindgen_ty_2 = 185;
pub const AKEYCODE_PROG_BLUE: _bindgen_ty_2 = 186;
pub const AKEYCODE_APP_SWITCH: _bindgen_ty_2 = 187;
pub const AKEYCODE_BUTTON_1: _bindgen_ty_2 = 188;
pub const AKEYCODE_BUTTON_2: _bindgen_ty_2 = 189;
pub const AKEYCODE_BUTTON_3: _bindgen_ty_2 = 190;
pub const AKEYCODE_BUTTON_4: _bindgen_ty_2 = 191;
pub const AKEYCODE_BUTTON_5: _bindgen_ty_2 = 192;
pub const AKEYCODE_BUTTON_6: _bindgen_ty_2 = 193;
pub const AKEYCODE_BUTTON_7: _bindgen_ty_2 = 194;
pub const AKEYCODE_BUTTON_8: _bindgen_ty_2 = 195;
pub const AKEYCODE_BUTTON_9: _bindgen_ty_2 = 196;
pub const AKEYCODE_BUTTON_10: _bindgen_ty_2 = 197;
pub const AKEYCODE_BUTTON_11: _bindgen_ty_2 = 198;
pub const AKEYCODE_BUTTON_12: _bindgen_ty_2 = 199;
pub const AKEYCODE_BUTTON_13: _bindgen_ty_2 = 200;
pub const AKEYCODE_BUTTON_14: _bindgen_ty_2 = 201;
pub const AKEYCODE_BUTTON_15: _bindgen_ty_2 = 202;
pub const AKEYCODE_BUTTON_16: _bindgen_ty_2 = 203;
pub const AKEYCODE_LANGUAGE_SWITCH: _bindgen_ty_2 = 204;
pub const AKEYCODE_MANNER_MODE: _bindgen_ty_2 = 205;
pub const AKEYCODE_3D_MODE: _bindgen_ty_2 = 206;
pub const AKEYCODE_CONTACTS: _bindgen_ty_2 = 207;
pub const AKEYCODE_CALENDAR: _bindgen_ty_2 = 208;
pub const AKEYCODE_MUSIC: _bindgen_ty_2 = 209;
pub const AKEYCODE_CALCULATOR: _bindgen_ty_2 = 210;
pub const AKEYCODE_ZENKAKU_HANKAKU: _bindgen_ty_2 = 211;
pub const AKEYCODE_EISU: _bindgen_ty_2 = 212;
pub const AKEYCODE_MUHENKAN: _bindgen_ty_2 = 213;
pub const AKEYCODE_HENKAN: _bindgen_ty_2 = 214;
pub const AKEYCODE_KATAKANA_HIRAGANA: _bindgen_ty_2 = 215;
pub const AKEYCODE_YEN: _bindgen_ty_2 = 216;
pub const AKEYCODE_RO: _bindgen_ty_2 = 217;
pub const AKEYCODE_KANA: _bindgen_ty_2 = 218;
pub const AKEYCODE_ASSIST: _bindgen_ty_2 = 219;
pub const AKEYCODE_BRIGHTNESS_DOWN: _bindgen_ty_2 = 220;
pub const AKEYCODE_BRIGHTNESS_UP: _bindgen_ty_2 = 221;
pub const AKEYCODE_MEDIA_AUDIO_TRACK: _bindgen_ty_2 = 222;
pub const AKEYCODE_SLEEP: _bindgen_ty_2 = 223;
pub const AKEYCODE_WAKEUP: _bindgen_ty_2 = 224;
pub const AKEYCODE_PAIRING: _bindgen_ty_2 = 225;
pub const AKEYCODE_MEDIA_TOP_MENU: _bindgen_ty_2 = 226;
pub const AKEYCODE_11: _bindgen_ty_2 = 227;
pub const AKEYCODE_12: _bindgen_ty_2 = 228;
pub const AKEYCODE_LAST_CHANNEL: _bindgen_ty_2 = 229;
pub const AKEYCODE_TV_DATA_SERVICE: _bindgen_ty_2 = 230;
pub const AKEYCODE_VOICE_ASSIST: _bindgen_ty_2 = 231;
pub const AKEYCODE_TV_RADIO_SERVICE: _bindgen_ty_2 = 232;
pub const AKEYCODE_TV_TELETEXT: _bindgen_ty_2 = 233;
pub const AKEYCODE_TV_NUMBER_ENTRY: _bindgen_ty_2 = 234;
pub const AKEYCODE_TV_TERRESTRIAL_ANALOG: _bindgen_ty_2 = 235;
pub const AKEYCODE_TV_TERRESTRIAL_DIGITAL: _bindgen_ty_2 = 236;
pub const AKEYCODE_TV_SATELLITE: _bindgen_ty_2 = 237;
pub const AKEYCODE_TV_SATELLITE_BS: _bindgen_ty_2 = 238;
pub const AKEYCODE_TV_SATELLITE_CS: _bindgen_ty_2 = 239;
pub const AKEYCODE_TV_SATELLITE_SERVICE: _bindgen_ty_2 = 240;
pub const AKEYCODE_TV_NETWORK: _bindgen_ty_2 = 241;
pub const AKEYCODE_TV_ANTENNA_CABLE: _bindgen_ty_2 = 242;
pub const AKEYCODE_TV_INPUT_HDMI_1: _bindgen_ty_2 = 243;
pub const AKEYCODE_TV_INPUT_HDMI_2: _bindgen_ty_2 = 244;
pub const AKEYCODE_TV_INPUT_HDMI_3: _bindgen_ty_2 = 245;
pub const AKEYCODE_TV_INPUT_HDMI_4: _bindgen_ty_2 = 246;
pub const AKEYCODE_TV_INPUT_COMPOSITE_1: _bindgen_ty_2 = 247;
pub const AKEYCODE_TV_INPUT_COMPOSITE_2: _bindgen_ty_2 = 248;
pub const AKEYCODE_TV_INPUT_COMPONENT_1: _bindgen_ty_2 = 249;
pub const AKEYCODE_TV_INPUT_COMPONENT_2: _bindgen_ty_2 = 250;
pub const AKEYCODE_TV_INPUT_VGA_1: _bindgen_ty_2 = 251;
pub const AKEYCODE_TV_AUDIO_DESCRIPTION: _bindgen_ty_2 = 252;
pub const AKEYCODE_TV_AUDIO_DESCRIPTION_MIX_UP: _bindgen_ty_2 = 253;
pub const AKEYCODE_TV_AUDIO_DESCRIPTION_MIX_DOWN: _bindgen_ty_2 = 254;
pub const AKEYCODE_TV_ZOOM_MODE: _bindgen_ty_2 = 255;
pub const AKEYCODE_TV_CONTENTS_MENU: _bindgen_ty_2 = 256;
pub const AKEYCODE_TV_MEDIA_CONTEXT_MENU: _bindgen_ty_2 = 257;
pub const AKEYCODE_TV_TIMER_PROGRAMMING: _bindgen_ty_2 = 258;
pub const AKEYCODE_HELP: _bindgen_ty_2 = 259;
pub const AKEYCODE_NAVIGATE_PREVIOUS: _bindgen_ty_2 = 260;
pub const AKEYCODE_NAVIGATE_NEXT: _bindgen_ty_2 = 261;
pub const AKEYCODE_NAVIGATE_IN: _bindgen_ty_2 = 262;
pub const AKEYCODE_NAVIGATE_OUT: _bindgen_ty_2 = 263;
pub const AKEYCODE_STEM_PRIMARY: _bindgen_ty_2 = 264;
pub const AKEYCODE_STEM_1: _bindgen_ty_2 = 265;
pub const AKEYCODE_STEM_2: _bindgen_ty_2 = 266;
pub const AKEYCODE_STEM_3: _bindgen_ty_2 = 267;
pub const AKEYCODE_DPAD_UP_LEFT: _bindgen_ty_2 = 268;
pub const AKEYCODE_DPAD_DOWN_LEFT: _bindgen_ty_2 = 269;
pub const AKEYCODE_DPAD_UP_RIGHT: _bindgen_ty_2 = 270;
pub const AKEYCODE_DPAD_DOWN_RIGHT: _bindgen_ty_2 = 271;
pub const AKEYCODE_MEDIA_SKIP_FORWARD: _bindgen_ty_2 = 272;
pub const AKEYCODE_MEDIA_SKIP_BACKWARD: _bindgen_ty_2 = 273;
pub const AKEYCODE_MEDIA_STEP_FORWARD: _bindgen_ty_2 = 274;
pub const AKEYCODE_MEDIA_STEP_BACKWARD: _bindgen_ty_2 = 275;
pub const AKEYCODE_SOFT_SLEEP: _bindgen_ty_2 = 276;
pub const AKEYCODE_CUT: _bindgen_ty_2 = 277;
pub const AKEYCODE_COPY: _bindgen_ty_2 = 278;
pub const AKEYCODE_PASTE: _bindgen_ty_2 = 279;
pub const AKEYCODE_SYSTEM_NAVIGATION_UP: _bindgen_ty_2 = 280;
pub const AKEYCODE_SYSTEM_NAVIGATION_DOWN: _bindgen_ty_2 = 281;
pub const AKEYCODE_SYSTEM_NAVIGATION_LEFT: _bindgen_ty_2 = 282;
pub const AKEYCODE_SYSTEM_NAVIGATION_RIGHT: _bindgen_ty_2 = 283;
pub const AKEYCODE_ALL_APPS: _bindgen_ty_2 = 284;
pub const AKEYCODE_REFRESH: _bindgen_ty_2 = 285;
pub const AKEYCODE_THUMBS_UP: _bindgen_ty_2 = 286;
pub const AKEYCODE_THUMBS_DOWN: _bindgen_ty_2 = 287;
pub const AKEYCODE_PROFILE_SWITCH: _bindgen_ty_2 = 288;
pub const AKEYCODE_VIDEO_APP_1: _bindgen_ty_2 = 289;
pub const AKEYCODE_VIDEO_APP_2: _bindgen_ty_2 = 290;
pub const AKEYCODE_VIDEO_APP_3: _bindgen_ty_2 = 291;
pub const AKEYCODE_VIDEO_APP_4: _bindgen_ty_2 = 292;
pub const AKEYCODE_VIDEO_APP_5: _bindgen_ty_2 = 293;
pub const AKEYCODE_VIDEO_APP_6: _bindgen_ty_2 = 294;
pub const AKEYCODE_VIDEO_APP_7: _bindgen_ty_2 = 295;
pub const AKEYCODE_VIDEO_APP_8: _bindgen_ty_2 = 296;
pub const AKEYCODE_FEATURED_APP_1: _bindgen_ty_2 = 297;
pub const AKEYCODE_FEATURED_APP_2: _bindgen_ty_2 = 298;
pub const AKEYCODE_FEATURED_APP_3: _bindgen_ty_2 = 299;
pub const AKEYCODE_FEATURED_APP_4: _bindgen_ty_2 = 300;
pub const AKEYCODE_DEMO_APP_1: _bindgen_ty_2 = 301;
pub const AKEYCODE_DEMO_APP_2: _bindgen_ty_2 = 302;
pub const AKEYCODE_DEMO_APP_3: _bindgen_ty_2 = 303;
pub const AKEYCODE_DEMO_APP_4: _bindgen_ty_2 = 304;
pub const AKEYCODE_KEYBOARD_BACKLIGHT_DOWN: _bindgen_ty_2 = 305;
pub const AKEYCODE_KEYBOARD_BACKLIGHT_UP: _bindgen_ty_2 = 306;
pub const AKEYCODE_KEYBOARD_BACKLIGHT_TOGGLE: _bindgen_ty_2 = 307;
pub const AKEYCODE_STYLUS_BUTTON_PRIMARY: _bindgen_ty_2 = 308;
pub const AKEYCODE_STYLUS_BUTTON_SECONDARY: _bindgen_ty_2 = 309;
pub const AKEYCODE_STYLUS_BUTTON_TERTIARY: _bindgen_ty_2 = 310;
pub const AKEYCODE_STYLUS_BUTTON_TAIL: _bindgen_ty_2 = 311;
pub const AKEYCODE_RECENT_APPS: _bindgen_ty_2 = 312;
pub const AKEYCODE_MACRO_1: _bindgen_ty_2 = 313;
pub const AKEYCODE_MACRO_2: _bindgen_ty_2 = 314;
pub const AKEYCODE_MACRO_3: _bindgen_ty_2 = 315;
pub const AKEYCODE_MACRO_4: _bindgen_ty_2 = 316;
pub const AKEYCODE_EMOJI_PICKER: _bindgen_ty_2 = 317;
pub const AKEYCODE_SCREENSHOT: _bindgen_ty_2 = 318;
pub type _bindgen_ty_2 = ::std::os::raw::c_uint;
pub const ALOOPER_PREPARE_ALLOW_NON_CALLBACKS: _bindgen_ty_3 = 1;
pub type _bindgen_ty_3 = ::std::os::raw::c_uint;
pub const ALOOPER_POLL_WAKE: _bindgen_ty_4 = -1;
pub const ALOOPER_POLL_CALLBACK: _bindgen_ty_4 = -2;
pub const ALOOPER_POLL_TIMEOUT: _bindgen_ty_4 = -3;
pub const ALOOPER_POLL_ERROR: _bindgen_ty_4 = -4;
pub type _bindgen_ty_4 = ::std::os::raw::c_int;
pub const ALOOPER_EVENT_INPUT: _bindgen_ty_5 = 1;
pub const ALOOPER_EVENT_OUTPUT: _bindgen_ty_5 = 2;
pub const ALOOPER_EVENT_ERROR: _bindgen_ty_5 = 4;
pub const ALOOPER_EVENT_HANGUP: _bindgen_ty_5 = 8;
pub const ALOOPER_EVENT_INVALID: _bindgen_ty_5 = 16;
pub type _bindgen_ty_5 = ::std::os::raw::c_uint;
pub type __gnuc_va_list = __builtin_va_list;
pub type va_list = __builtin_va_list;
#[repr(C)]
pub struct JavaVMAttachArgs {
    pub version: jint,
    pub name: *const ::std::os::raw::c_char,
    pub group: jobject,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of JavaVMAttachArgs"][::std::mem::size_of::<JavaVMAttachArgs>() - 12usize];
    ["Alignment of JavaVMAttachArgs"][::std::mem::align_of::<JavaVMAttachArgs>() - 4usize];
    ["Offset of field: JavaVMAttachArgs::version"]
        [::std::mem::offset_of!(JavaVMAttachArgs, version) - 0usize];
    ["Offset of field: JavaVMAttachArgs::name"]
        [::std::mem::offset_of!(JavaVMAttachArgs, name) - 4usize];
    ["Offset of field: JavaVMAttachArgs::group"]
        [::std::mem::offset_of!(JavaVMAttachArgs, group) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct JavaVMOption {
    pub optionString: *const ::std::os::raw::c_char,
    pub extraInfo: *mut ::std::os::raw::c_void,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of JavaVMOption"][::std::mem::size_of::<JavaVMOption>() - 8usize];
    ["Alignment of JavaVMOption"][::std::mem::align_of::<JavaVMOption>() - 4usize];
    ["Offset of field: JavaVMOption::optionString"]
        [::std::mem::offset_of!(JavaVMOption, optionString) - 0usize];
    ["Offset of field: JavaVMOption::extraInfo"]
        [::std::mem::offset_of!(JavaVMOption, extraInfo) - 4usize];
};
#[repr(C)]
pub struct JavaVMInitArgs {
    pub version: jint,
    pub nOptions: jint,
    pub options: *mut JavaVMOption,
    pub ignoreUnrecognized: jboolean,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of JavaVMInitArgs"][::std::mem::size_of::<JavaVMInitArgs>() - 16usize];
    ["Alignment of JavaVMInitArgs"][::std::mem::align_of::<JavaVMInitArgs>() - 4usize];
    ["Offset of field: JavaVMInitArgs::version"]
        [::std::mem::offset_of!(JavaVMInitArgs, version) - 0usize];
    ["Offset of field: JavaVMInitArgs::nOptions"]
        [::std::mem::offset_of!(JavaVMInitArgs, nOptions) - 4usize];
    ["Offset of field: JavaVMInitArgs::options"]
        [::std::mem::offset_of!(JavaVMInitArgs, options) - 8usize];
    ["Offset of field: JavaVMInitArgs::ignoreUnrecognized"]
        [::std::mem::offset_of!(JavaVMInitArgs, ignoreUnrecognized) - 12usize];
};
pub const AKEY_STATE_UNKNOWN: _bindgen_ty_6 = -1;
pub const AKEY_STATE_UP: _bindgen_ty_6 = 0;
pub const AKEY_STATE_DOWN: _bindgen_ty_6 = 1;
pub const AKEY_STATE_VIRTUAL: _bindgen_ty_6 = 2;
pub type _bindgen_ty_6 = ::std::os::raw::c_int;
pub const AMETA_NONE: _bindgen_ty_7 = 0;
pub const AMETA_ALT_ON: _bindgen_ty_7 = 2;
pub const AMETA_ALT_LEFT_ON: _bindgen_ty_7 = 16;
pub const AMETA_ALT_RIGHT_ON: _bindgen_ty_7 = 32;
pub const AMETA_SHIFT_ON: _bindgen_ty_7 = 1;
pub const AMETA_SHIFT_LEFT_ON: _bindgen_ty_7 = 64;
pub const AMETA_SHIFT_RIGHT_ON: _bindgen_ty_7 = 128;
pub const AMETA_SYM_ON: _bindgen_ty_7 = 4;
pub const AMETA_FUNCTION_ON: _bindgen_ty_7 = 8;
pub const AMETA_CTRL_ON: _bindgen_ty_7 = 4096;
pub const AMETA_CTRL_LEFT_ON: _bindgen_ty_7 = 8192;
pub const AMETA_CTRL_RIGHT_ON: _bindgen_ty_7 = 16384;
pub const AMETA_META_ON: _bindgen_ty_7 = 65536;
pub const AMETA_META_LEFT_ON: _bindgen_ty_7 = 131072;
pub const AMETA_META_RIGHT_ON: _bindgen_ty_7 = 262144;
pub const AMETA_CAPS_LOCK_ON: _bindgen_ty_7 = 1048576;
pub const AMETA_NUM_LOCK_ON: _bindgen_ty_7 = 2097152;
pub const AMETA_SCROLL_LOCK_ON: _bindgen_ty_7 = 4194304;
pub type _bindgen_ty_7 = ::std::os::raw::c_uint;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct AInputEvent {
    _unused: [u8; 0],
}
pub const AINPUT_EVENT_TYPE_KEY: _bindgen_ty_8 = 1;
pub const AINPUT_EVENT_TYPE_MOTION: _bindgen_ty_8 = 2;
pub const AINPUT_EVENT_TYPE_FOCUS: _bindgen_ty_8 = 3;
pub const AINPUT_EVENT_TYPE_CAPTURE: _bindgen_ty_8 = 4;
pub const AINPUT_EVENT_TYPE_DRAG: _bindgen_ty_8 = 5;
pub const AINPUT_EVENT_TYPE_TOUCH_MODE: _bindgen_ty_8 = 6;
pub type _bindgen_ty_8 = ::std::os::raw::c_uint;
pub const AKEY_EVENT_ACTION_DOWN: _bindgen_ty_9 = 0;
pub const AKEY_EVENT_ACTION_UP: _bindgen_ty_9 = 1;
pub const AKEY_EVENT_ACTION_MULTIPLE: _bindgen_ty_9 = 2;
pub type _bindgen_ty_9 = ::std::os::raw::c_uint;
pub const AKEY_EVENT_FLAG_WOKE_HERE: _bindgen_ty_10 = 1;
pub const AKEY_EVENT_FLAG_SOFT_KEYBOARD: _bindgen_ty_10 = 2;
pub const AKEY_EVENT_FLAG_KEEP_TOUCH_MODE: _bindgen_ty_10 = 4;
pub const AKEY_EVENT_FLAG_FROM_SYSTEM: _bindgen_ty_10 = 8;
pub const AKEY_EVENT_FLAG_EDITOR_ACTION: _bindgen_ty_10 = 16;
pub const AKEY_EVENT_FLAG_CANCELED: _bindgen_ty_10 = 32;
pub const AKEY_EVENT_FLAG_VIRTUAL_HARD_KEY: _bindgen_ty_10 = 64;
pub const AKEY_EVENT_FLAG_LONG_PRESS: _bindgen_ty_10 = 128;
pub const AKEY_EVENT_FLAG_CANCELED_LONG_PRESS: _bindgen_ty_10 = 256;
pub const AKEY_EVENT_FLAG_TRACKING: _bindgen_ty_10 = 512;
pub const AKEY_EVENT_FLAG_FALLBACK: _bindgen_ty_10 = 1024;
pub type _bindgen_ty_10 = ::std::os::raw::c_uint;
pub const AMOTION_EVENT_ACTION_MASK: _bindgen_ty_11 = 255;
pub const AMOTION_EVENT_ACTION_POINTER_INDEX_MASK: _bindgen_ty_11 = 65280;
pub const AMOTION_EVENT_ACTION_DOWN: _bindgen_ty_11 = 0;
pub const AMOTION_EVENT_ACTION_UP: _bindgen_ty_11 = 1;
pub const AMOTION_EVENT_ACTION_MOVE: _bindgen_ty_11 = 2;
pub const AMOTION_EVENT_ACTION_CANCEL: _bindgen_ty_11 = 3;
pub const AMOTION_EVENT_ACTION_OUTSIDE: _bindgen_ty_11 = 4;
pub const AMOTION_EVENT_ACTION_POINTER_DOWN: _bindgen_ty_11 = 5;
pub const AMOTION_EVENT_ACTION_POINTER_UP: _bindgen_ty_11 = 6;
pub const AMOTION_EVENT_ACTION_HOVER_MOVE: _bindgen_ty_11 = 7;
pub const AMOTION_EVENT_ACTION_SCROLL: _bindgen_ty_11 = 8;
pub const AMOTION_EVENT_ACTION_HOVER_ENTER: _bindgen_ty_11 = 9;
pub const AMOTION_EVENT_ACTION_HOVER_EXIT: _bindgen_ty_11 = 10;
pub const AMOTION_EVENT_ACTION_BUTTON_PRESS: _bindgen_ty_11 = 11;
pub const AMOTION_EVENT_ACTION_BUTTON_RELEASE: _bindgen_ty_11 = 12;
pub type _bindgen_ty_11 = ::std::os::raw::c_uint;
pub const AMOTION_EVENT_FLAG_WINDOW_IS_OBSCURED: _bindgen_ty_12 = 1;
pub type _bindgen_ty_12 = ::std::os::raw::c_uint;
pub const AMOTION_EVENT_EDGE_FLAG_NONE: _bindgen_ty_13 = 0;
pub const AMOTION_EVENT_EDGE_FLAG_TOP: _bindgen_ty_13 = 1;
pub const AMOTION_EVENT_EDGE_FLAG_BOTTOM: _bindgen_ty_13 = 2;
pub const AMOTION_EVENT_EDGE_FLAG_LEFT: _bindgen_ty_13 = 4;
pub const AMOTION_EVENT_EDGE_FLAG_RIGHT: _bindgen_ty_13 = 8;
pub type _bindgen_ty_13 = ::std::os::raw::c_uint;
pub const AMOTION_EVENT_AXIS_X: _bindgen_ty_14 = 0;
pub const AMOTION_EVENT_AXIS_Y: _bindgen_ty_14 = 1;
pub const AMOTION_EVENT_AXIS_PRESSURE: _bindgen_ty_14 = 2;
pub const AMOTION_EVENT_AXIS_SIZE: _bindgen_ty_14 = 3;
pub const AMOTION_EVENT_AXIS_TOUCH_MAJOR: _bindgen_ty_14 = 4;
pub const AMOTION_EVENT_AXIS_TOUCH_MINOR: _bindgen_ty_14 = 5;
pub const AMOTION_EVENT_AXIS_TOOL_MAJOR: _bindgen_ty_14 = 6;
pub const AMOTION_EVENT_AXIS_TOOL_MINOR: _bindgen_ty_14 = 7;
pub const AMOTION_EVENT_AXIS_ORIENTATION: _bindgen_ty_14 = 8;
pub const AMOTION_EVENT_AXIS_VSCROLL: _bindgen_ty_14 = 9;
pub const AMOTION_EVENT_AXIS_HSCROLL: _bindgen_ty_14 = 10;
pub const AMOTION_EVENT_AXIS_Z: _bindgen_ty_14 = 11;
pub const AMOTION_EVENT_AXIS_RX: _bindgen_ty_14 = 12;
pub const AMOTION_EVENT_AXIS_RY: _bindgen_ty_14 = 13;
pub const AMOTION_EVENT_AXIS_RZ: _bindgen_ty_14 = 14;
pub const AMOTION_EVENT_AXIS_HAT_X: _bindgen_ty_14 = 15;
pub const AMOTION_EVENT_AXIS_HAT_Y: _bindgen_ty_14 = 16;
pub const AMOTION_EVENT_AXIS_LTRIGGER: _bindgen_ty_14 = 17;
pub const AMOTION_EVENT_AXIS_RTRIGGER: _bindgen_ty_14 = 18;
pub const AMOTION_EVENT_AXIS_THROTTLE: _bindgen_ty_14 = 19;
pub const AMOTION_EVENT_AXIS_RUDDER: _bindgen_ty_14 = 20;
pub const AMOTION_EVENT_AXIS_WHEEL: _bindgen_ty_14 = 21;
pub const AMOTION_EVENT_AXIS_GAS: _bindgen_ty_14 = 22;
pub const AMOTION_EVENT_AXIS_BRAKE: _bindgen_ty_14 = 23;
pub const AMOTION_EVENT_AXIS_DISTANCE: _bindgen_ty_14 = 24;
pub const AMOTION_EVENT_AXIS_TILT: _bindgen_ty_14 = 25;
pub const AMOTION_EVENT_AXIS_SCROLL: _bindgen_ty_14 = 26;
pub const AMOTION_EVENT_AXIS_RELATIVE_X: _bindgen_ty_14 = 27;
pub const AMOTION_EVENT_AXIS_RELATIVE_Y: _bindgen_ty_14 = 28;
pub const AMOTION_EVENT_AXIS_GENERIC_1: _bindgen_ty_14 = 32;
pub const AMOTION_EVENT_AXIS_GENERIC_2: _bindgen_ty_14 = 33;
pub const AMOTION_EVENT_AXIS_GENERIC_3: _bindgen_ty_14 = 34;
pub const AMOTION_EVENT_AXIS_GENERIC_4: _bindgen_ty_14 = 35;
pub const AMOTION_EVENT_AXIS_GENERIC_5: _bindgen_ty_14 = 36;
pub const AMOTION_EVENT_AXIS_GENERIC_6: _bindgen_ty_14 = 37;
pub const AMOTION_EVENT_AXIS_GENERIC_7: _bindgen_ty_14 = 38;
pub const AMOTION_EVENT_AXIS_GENERIC_8: _bindgen_ty_14 = 39;
pub const AMOTION_EVENT_AXIS_GENERIC_9: _bindgen_ty_14 = 40;
pub const AMOTION_EVENT_AXIS_GENERIC_10: _bindgen_ty_14 = 41;
pub const AMOTION_EVENT_AXIS_GENERIC_11: _bindgen_ty_14 = 42;
pub const AMOTION_EVENT_AXIS_GENERIC_12: _bindgen_ty_14 = 43;
pub const AMOTION_EVENT_AXIS_GENERIC_13: _bindgen_ty_14 = 44;
pub const AMOTION_EVENT_AXIS_GENERIC_14: _bindgen_ty_14 = 45;
pub const AMOTION_EVENT_AXIS_GENERIC_15: _bindgen_ty_14 = 46;
pub const AMOTION_EVENT_AXIS_GENERIC_16: _bindgen_ty_14 = 47;
pub const AMOTION_EVENT_AXIS_GESTURE_X_OFFSET: _bindgen_ty_14 = 48;
pub const AMOTION_EVENT_AXIS_GESTURE_Y_OFFSET: _bindgen_ty_14 = 49;
pub const AMOTION_EVENT_AXIS_GESTURE_SCROLL_X_DISTANCE: _bindgen_ty_14 = 50;
pub const AMOTION_EVENT_AXIS_GESTURE_SCROLL_Y_DISTANCE: _bindgen_ty_14 = 51;
pub const AMOTION_EVENT_AXIS_GESTURE_PINCH_SCALE_FACTOR: _bindgen_ty_14 = 52;
pub const AMOTION_EVENT_AXIS_GESTURE_SWIPE_FINGER_COUNT: _bindgen_ty_14 = 53;
pub const AMOTION_EVENT_MAXIMUM_VALID_AXIS_VALUE: _bindgen_ty_14 = 53;
pub type _bindgen_ty_14 = ::std::os::raw::c_uint;
pub const AMOTION_EVENT_BUTTON_PRIMARY: _bindgen_ty_15 = 1;
pub const AMOTION_EVENT_BUTTON_SECONDARY: _bindgen_ty_15 = 2;
pub const AMOTION_EVENT_BUTTON_TERTIARY: _bindgen_ty_15 = 4;
pub const AMOTION_EVENT_BUTTON_BACK: _bindgen_ty_15 = 8;
pub const AMOTION_EVENT_BUTTON_FORWARD: _bindgen_ty_15 = 16;
pub const AMOTION_EVENT_BUTTON_STYLUS_PRIMARY: _bindgen_ty_15 = 32;
pub const AMOTION_EVENT_BUTTON_STYLUS_SECONDARY: _bindgen_ty_15 = 64;
pub type _bindgen_ty_15 = ::std::os::raw::c_uint;
pub const AMOTION_EVENT_TOOL_TYPE_UNKNOWN: _bindgen_ty_16 = 0;
pub const AMOTION_EVENT_TOOL_TYPE_FINGER: _bindgen_ty_16 = 1;
pub const AMOTION_EVENT_TOOL_TYPE_STYLUS: _bindgen_ty_16 = 2;
pub const AMOTION_EVENT_TOOL_TYPE_MOUSE: _bindgen_ty_16 = 3;
pub const AMOTION_EVENT_TOOL_TYPE_ERASER: _bindgen_ty_16 = 4;
pub const AMOTION_EVENT_TOOL_TYPE_PALM: _bindgen_ty_16 = 5;
pub type _bindgen_ty_16 = ::std::os::raw::c_uint;
pub const AMotionClassification_AMOTION_EVENT_CLASSIFICATION_NONE: AMotionClassification = 0;
pub const AMotionClassification_AMOTION_EVENT_CLASSIFICATION_AMBIGUOUS_GESTURE:
    AMotionClassification = 1;
pub const AMotionClassification_AMOTION_EVENT_CLASSIFICATION_DEEP_PRESS: AMotionClassification = 2;
pub const AMotionClassification_AMOTION_EVENT_CLASSIFICATION_TWO_FINGER_SWIPE:
    AMotionClassification = 3;
pub const AMotionClassification_AMOTION_EVENT_CLASSIFICATION_MULTI_FINGER_SWIPE:
    AMotionClassification = 4;
pub const AMotionClassification_AMOTION_EVENT_CLASSIFICATION_PINCH: AMotionClassification = 5;
pub type AMotionClassification = u32;
pub const AINPUT_SOURCE_CLASS_MASK: _bindgen_ty_17 = 255;
pub const AINPUT_SOURCE_CLASS_NONE: _bindgen_ty_17 = 0;
pub const AINPUT_SOURCE_CLASS_BUTTON: _bindgen_ty_17 = 1;
pub const AINPUT_SOURCE_CLASS_POINTER: _bindgen_ty_17 = 2;
pub const AINPUT_SOURCE_CLASS_NAVIGATION: _bindgen_ty_17 = 4;
pub const AINPUT_SOURCE_CLASS_POSITION: _bindgen_ty_17 = 8;
pub const AINPUT_SOURCE_CLASS_JOYSTICK: _bindgen_ty_17 = 16;
pub type _bindgen_ty_17 = ::std::os::raw::c_uint;
pub const AINPUT_SOURCE_UNKNOWN: _bindgen_ty_18 = 0;
pub const AINPUT_SOURCE_KEYBOARD: _bindgen_ty_18 = 257;
pub const AINPUT_SOURCE_DPAD: _bindgen_ty_18 = 513;
pub const AINPUT_SOURCE_GAMEPAD: _bindgen_ty_18 = 1025;
pub const AINPUT_SOURCE_TOUCHSCREEN: _bindgen_ty_18 = 4098;
pub const AINPUT_SOURCE_MOUSE: _bindgen_ty_18 = 8194;
pub const AINPUT_SOURCE_STYLUS: _bindgen_ty_18 = 16386;
pub const AINPUT_SOURCE_BLUETOOTH_STYLUS: _bindgen_ty_18 = 49154;
pub const AINPUT_SOURCE_TRACKBALL: _bindgen_ty_18 = 65540;
pub const AINPUT_SOURCE_MOUSE_RELATIVE: _bindgen_ty_18 = 131076;
pub const AINPUT_SOURCE_TOUCHPAD: _bindgen_ty_18 = 1048584;
pub const AINPUT_SOURCE_TOUCH_NAVIGATION: _bindgen_ty_18 = 2097152;
pub const AINPUT_SOURCE_JOYSTICK: _bindgen_ty_18 = 16777232;
pub const AINPUT_SOURCE_HDMI: _bindgen_ty_18 = 33554433;
pub const AINPUT_SOURCE_SENSOR: _bindgen_ty_18 = 67108864;
pub const AINPUT_SOURCE_ROTARY_ENCODER: _bindgen_ty_18 = 4194304;
pub const AINPUT_SOURCE_ANY: _bindgen_ty_18 = 4294967040;
pub type _bindgen_ty_18 = ::std::os::raw::c_uint;
pub const AINPUT_KEYBOARD_TYPE_NONE: _bindgen_ty_19 = 0;
pub const AINPUT_KEYBOARD_TYPE_NON_ALPHABETIC: _bindgen_ty_19 = 1;
pub const AINPUT_KEYBOARD_TYPE_ALPHABETIC: _bindgen_ty_19 = 2;
pub type _bindgen_ty_19 = ::std::os::raw::c_uint;
pub const AINPUT_MOTION_RANGE_X: _bindgen_ty_20 = 0;
pub const AINPUT_MOTION_RANGE_Y: _bindgen_ty_20 = 1;
pub const AINPUT_MOTION_RANGE_PRESSURE: _bindgen_ty_20 = 2;
pub const AINPUT_MOTION_RANGE_SIZE: _bindgen_ty_20 = 3;
pub const AINPUT_MOTION_RANGE_TOUCH_MAJOR: _bindgen_ty_20 = 4;
pub const AINPUT_MOTION_RANGE_TOUCH_MINOR: _bindgen_ty_20 = 5;
pub const AINPUT_MOTION_RANGE_TOOL_MAJOR: _bindgen_ty_20 = 6;
pub const AINPUT_MOTION_RANGE_TOOL_MINOR: _bindgen_ty_20 = 7;
pub const AINPUT_MOTION_RANGE_ORIENTATION: _bindgen_ty_20 = 8;
pub type _bindgen_ty_20 = ::std::os::raw::c_uint;
unsafe extern "C" {
    pub fn AInputEvent_getType(event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AInputEvent_getDeviceId(event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AInputEvent_getSource(event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AInputEvent_release(event: *const AInputEvent);
}
unsafe extern "C" {
    pub fn AKeyEvent_getAction(key_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AKeyEvent_getFlags(key_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AKeyEvent_getKeyCode(key_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AKeyEvent_getScanCode(key_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AKeyEvent_getMetaState(key_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AKeyEvent_getRepeatCount(key_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AKeyEvent_getDownTime(key_event: *const AInputEvent) -> i64;
}
unsafe extern "C" {
    pub fn AKeyEvent_getEventTime(key_event: *const AInputEvent) -> i64;
}
unsafe extern "C" {
    pub fn AKeyEvent_fromJava(env: *mut JNIEnv, keyEvent: jobject) -> *const AInputEvent;
}
unsafe extern "C" {
    pub fn AMotionEvent_getAction(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getFlags(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getMetaState(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getButtonState(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getEdgeFlags(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getDownTime(motion_event: *const AInputEvent) -> i64;
}
unsafe extern "C" {
    pub fn AMotionEvent_getEventTime(motion_event: *const AInputEvent) -> i64;
}
unsafe extern "C" {
    pub fn AMotionEvent_getXOffset(motion_event: *const AInputEvent) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getYOffset(motion_event: *const AInputEvent) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getXPrecision(motion_event: *const AInputEvent) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getYPrecision(motion_event: *const AInputEvent) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getPointerCount(motion_event: *const AInputEvent) -> usize;
}
unsafe extern "C" {
    pub fn AMotionEvent_getPointerId(motion_event: *const AInputEvent, pointer_index: usize)
        -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getToolType(motion_event: *const AInputEvent, pointer_index: usize) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getRawX(motion_event: *const AInputEvent, pointer_index: usize) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getRawY(motion_event: *const AInputEvent, pointer_index: usize) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getX(motion_event: *const AInputEvent, pointer_index: usize) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getY(motion_event: *const AInputEvent, pointer_index: usize) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getPressure(motion_event: *const AInputEvent, pointer_index: usize) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getSize(motion_event: *const AInputEvent, pointer_index: usize) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getTouchMajor(
        motion_event: *const AInputEvent,
        pointer_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getTouchMinor(
        motion_event: *const AInputEvent,
        pointer_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getToolMajor(motion_event: *const AInputEvent, pointer_index: usize)
        -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getToolMinor(motion_event: *const AInputEvent, pointer_index: usize)
        -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getOrientation(
        motion_event: *const AInputEvent,
        pointer_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getAxisValue(
        motion_event: *const AInputEvent,
        axis: i32,
        pointer_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistorySize(motion_event: *const AInputEvent) -> usize;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalEventTime(
        motion_event: *const AInputEvent,
        history_index: usize,
    ) -> i64;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalRawX(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalRawY(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalX(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalY(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalPressure(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalSize(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalTouchMajor(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalTouchMinor(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalToolMajor(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalToolMinor(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalOrientation(
        motion_event: *const AInputEvent,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getHistoricalAxisValue(
        motion_event: *const AInputEvent,
        axis: i32,
        pointer_index: usize,
        history_index: usize,
    ) -> f32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getActionButton(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_getClassification(motion_event: *const AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AMotionEvent_fromJava(env: *mut JNIEnv, motionEvent: jobject) -> *const AInputEvent;
}
unsafe extern "C" {
    pub fn AInputEvent_toJava(env: *mut JNIEnv, aInputEvent: *const AInputEvent) -> jobject;
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct AInputQueue {
    _unused: [u8; 0],
}
unsafe extern "C" {
    pub fn AInputQueue_attachLooper(
        queue: *mut AInputQueue,
        looper: *mut ALooper,
        ident: ::std::os::raw::c_int,
        callback: ALooper_callbackFunc,
        data: *mut ::std::os::raw::c_void,
    );
}
unsafe extern "C" {
    pub fn AInputQueue_detachLooper(queue: *mut AInputQueue);
}
unsafe extern "C" {
    pub fn AInputQueue_hasEvents(queue: *mut AInputQueue) -> i32;
}
unsafe extern "C" {
    pub fn AInputQueue_getEvent(queue: *mut AInputQueue, outEvent: *mut *mut AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AInputQueue_preDispatchEvent(queue: *mut AInputQueue, event: *mut AInputEvent) -> i32;
}
unsafe extern "C" {
    pub fn AInputQueue_finishEvent(
        queue: *mut AInputQueue,
        event: *mut AInputEvent,
        handled: ::std::os::raw::c_int,
    );
}
unsafe extern "C" {
    pub fn AInputQueue_fromJava(env: *mut JNIEnv, inputQueue: jobject) -> *mut AInputQueue;
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct imaxdiv_t {
    pub quot: intmax_t,
    pub rem: intmax_t,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of imaxdiv_t"][::std::mem::size_of::<imaxdiv_t>() - 16usize];
    ["Alignment of imaxdiv_t"][::std::mem::align_of::<imaxdiv_t>() - 4usize];
    ["Offset of field: imaxdiv_t::quot"][::std::mem::offset_of!(imaxdiv_t, quot) - 0usize];
    ["Offset of field: imaxdiv_t::rem"][::std::mem::offset_of!(imaxdiv_t, rem) - 8usize];
};
unsafe extern "C" {
    pub fn imaxabs(__i: intmax_t) -> intmax_t;
}
unsafe extern "C" {
    pub fn imaxdiv(__numerator: intmax_t, __denominator: intmax_t) -> imaxdiv_t;
}
unsafe extern "C" {
    pub fn strtoimax(
        __s: *const ::std::os::raw::c_char,
        __end_ptr: *mut *mut ::std::os::raw::c_char,
        __base: ::std::os::raw::c_int,
    ) -> intmax_t;
}
unsafe extern "C" {
    pub fn strtoumax(
        __s: *const ::std::os::raw::c_char,
        __end_ptr: *mut *mut ::std::os::raw::c_char,
        __base: ::std::os::raw::c_int,
    ) -> uintmax_t;
}
unsafe extern "C" {
    pub fn wcstoimax(
        __s: *const wchar_t,
        __end_ptr: *mut *mut wchar_t,
        __base: ::std::os::raw::c_int,
    ) -> intmax_t;
}
unsafe extern "C" {
    pub fn wcstoumax(
        __s: *const wchar_t,
        __end_ptr: *mut *mut wchar_t,
        __base: ::std::os::raw::c_int,
    ) -> uintmax_t;
}
pub const ADataSpace_ADATASPACE_UNKNOWN: ADataSpace = 0;
pub const ADataSpace_ADATASPACE_STANDARD_MASK: ADataSpace = 4128768;
pub const ADataSpace_ADATASPACE_STANDARD_UNSPECIFIED: ADataSpace = 0;
pub const ADataSpace_ADATASPACE_STANDARD_BT709: ADataSpace = 65536;
pub const ADataSpace_ADATASPACE_STANDARD_BT601_625: ADataSpace = 131072;
pub const ADataSpace_ADATASPACE_STANDARD_BT601_625_UNADJUSTED: ADataSpace = 196608;
pub const ADataSpace_ADATASPACE_STANDARD_BT601_525: ADataSpace = 262144;
pub const ADataSpace_ADATASPACE_STANDARD_BT601_525_UNADJUSTED: ADataSpace = 327680;
pub const ADataSpace_ADATASPACE_STANDARD_BT2020: ADataSpace = 393216;
pub const ADataSpace_ADATASPACE_STANDARD_BT2020_CONSTANT_LUMINANCE: ADataSpace = 458752;
pub const ADataSpace_ADATASPACE_STANDARD_BT470M: ADataSpace = 524288;
pub const ADataSpace_ADATASPACE_STANDARD_FILM: ADataSpace = 589824;
pub const ADataSpace_ADATASPACE_STANDARD_DCI_P3: ADataSpace = 655360;
pub const ADataSpace_ADATASPACE_STANDARD_ADOBE_RGB: ADataSpace = 720896;
pub const ADataSpace_ADATASPACE_TRANSFER_MASK: ADataSpace = 130023424;
pub const ADataSpace_ADATASPACE_TRANSFER_UNSPECIFIED: ADataSpace = 0;
pub const ADataSpace_ADATASPACE_TRANSFER_LINEAR: ADataSpace = 4194304;
pub const ADataSpace_ADATASPACE_TRANSFER_SRGB: ADataSpace = 8388608;
pub const ADataSpace_ADATASPACE_TRANSFER_SMPTE_170M: ADataSpace = 12582912;
pub const ADataSpace_ADATASPACE_TRANSFER_GAMMA2_2: ADataSpace = 16777216;
pub const ADataSpace_ADATASPACE_TRANSFER_GAMMA2_6: ADataSpace = 20971520;
pub const ADataSpace_ADATASPACE_TRANSFER_GAMMA2_8: ADataSpace = 25165824;
pub const ADataSpace_ADATASPACE_TRANSFER_ST2084: ADataSpace = 29360128;
pub const ADataSpace_ADATASPACE_TRANSFER_HLG: ADataSpace = 33554432;
pub const ADataSpace_ADATASPACE_RANGE_MASK: ADataSpace = 939524096;
pub const ADataSpace_ADATASPACE_RANGE_UNSPECIFIED: ADataSpace = 0;
pub const ADataSpace_ADATASPACE_RANGE_FULL: ADataSpace = 134217728;
pub const ADataSpace_ADATASPACE_RANGE_LIMITED: ADataSpace = 268435456;
pub const ADataSpace_ADATASPACE_RANGE_EXTENDED: ADataSpace = 402653184;
pub const ADataSpace_ADATASPACE_SCRGB_LINEAR: ADataSpace = 406913024;
pub const ADataSpace_ADATASPACE_SRGB: ADataSpace = 142671872;
pub const ADataSpace_ADATASPACE_SCRGB: ADataSpace = 411107328;
pub const ADataSpace_ADATASPACE_DISPLAY_P3: ADataSpace = 143261696;
pub const ADataSpace_ADATASPACE_BT2020_PQ: ADataSpace = 163971072;
pub const ADataSpace_ADATASPACE_BT2020_ITU_PQ: ADataSpace = 298188800;
pub const ADataSpace_ADATASPACE_ADOBE_RGB: ADataSpace = 151715840;
pub const ADataSpace_ADATASPACE_JFIF: ADataSpace = 146931712;
pub const ADataSpace_ADATASPACE_BT601_625: ADataSpace = 281149440;
pub const ADataSpace_ADATASPACE_BT601_525: ADataSpace = 281280512;
pub const ADataSpace_ADATASPACE_BT2020: ADataSpace = 147193856;
pub const ADataSpace_ADATASPACE_BT709: ADataSpace = 281083904;
pub const ADataSpace_ADATASPACE_DCI_P3: ADataSpace = 155844608;
pub const ADataSpace_ADATASPACE_SRGB_LINEAR: ADataSpace = 138477568;
pub const ADataSpace_ADATASPACE_BT2020_HLG: ADataSpace = 168165376;
pub const ADataSpace_ADATASPACE_BT2020_ITU_HLG: ADataSpace = 302383104;
pub const ADataSpace_ADATASPACE_DEPTH: ADataSpace = 4096;
pub const ADataSpace_ADATASPACE_DYNAMIC_DEPTH: ADataSpace = 4098;
pub const ADataSpace_STANDARD_MASK: ADataSpace = 4128768;
pub const ADataSpace_STANDARD_UNSPECIFIED: ADataSpace = 0;
pub const ADataSpace_STANDARD_BT709: ADataSpace = 65536;
pub const ADataSpace_STANDARD_BT601_625: ADataSpace = 131072;
pub const ADataSpace_STANDARD_BT601_625_UNADJUSTED: ADataSpace = 196608;
pub const ADataSpace_STANDARD_BT601_525: ADataSpace = 262144;
pub const ADataSpace_STANDARD_BT601_525_UNADJUSTED: ADataSpace = 327680;
pub const ADataSpace_STANDARD_BT470M: ADataSpace = 524288;
pub const ADataSpace_STANDARD_BT2020: ADataSpace = 393216;
pub const ADataSpace_STANDARD_FILM: ADataSpace = 589824;
pub const ADataSpace_STANDARD_DCI_P3: ADataSpace = 655360;
pub const ADataSpace_STANDARD_ADOBE_RGB: ADataSpace = 720896;
pub const ADataSpace_TRANSFER_MASK: ADataSpace = 130023424;
pub const ADataSpace_TRANSFER_UNSPECIFIED: ADataSpace = 0;
pub const ADataSpace_TRANSFER_LINEAR: ADataSpace = 4194304;
pub const ADataSpace_TRANSFER_SMPTE_170M: ADataSpace = 12582912;
pub const ADataSpace_TRANSFER_GAMMA2_2: ADataSpace = 16777216;
pub const ADataSpace_TRANSFER_GAMMA2_6: ADataSpace = 20971520;
pub const ADataSpace_TRANSFER_GAMMA2_8: ADataSpace = 25165824;
pub const ADataSpace_TRANSFER_SRGB: ADataSpace = 8388608;
pub const ADataSpace_TRANSFER_ST2084: ADataSpace = 29360128;
pub const ADataSpace_TRANSFER_HLG: ADataSpace = 33554432;
pub const ADataSpace_RANGE_MASK: ADataSpace = 939524096;
pub const ADataSpace_RANGE_UNSPECIFIED: ADataSpace = 0;
pub const ADataSpace_RANGE_FULL: ADataSpace = 134217728;
pub const ADataSpace_RANGE_LIMITED: ADataSpace = 268435456;
pub const ADataSpace_RANGE_EXTENDED: ADataSpace = 402653184;
pub type ADataSpace = i32;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R8G8B8A8_UNORM: AHardwareBuffer_Format = 1;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R8G8B8X8_UNORM: AHardwareBuffer_Format = 2;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R8G8B8_UNORM: AHardwareBuffer_Format = 3;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R5G6B5_UNORM: AHardwareBuffer_Format = 4;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R16G16B16A16_FLOAT: AHardwareBuffer_Format =
    22;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R10G10B10A2_UNORM: AHardwareBuffer_Format =
    43;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_BLOB: AHardwareBuffer_Format = 33;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_D16_UNORM: AHardwareBuffer_Format = 48;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_D24_UNORM: AHardwareBuffer_Format = 49;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_D24_UNORM_S8_UINT: AHardwareBuffer_Format =
    50;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_D32_FLOAT: AHardwareBuffer_Format = 51;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_D32_FLOAT_S8_UINT: AHardwareBuffer_Format =
    52;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_S8_UINT: AHardwareBuffer_Format = 53;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_Y8Cb8Cr8_420: AHardwareBuffer_Format = 35;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_YCbCr_P010: AHardwareBuffer_Format = 54;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_YCbCr_P210: AHardwareBuffer_Format = 60;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R8_UNORM: AHardwareBuffer_Format = 56;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R16_UINT: AHardwareBuffer_Format = 57;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R16G16_UINT: AHardwareBuffer_Format = 58;
pub const AHardwareBuffer_Format_AHARDWAREBUFFER_FORMAT_R10G10B10A10_UNORM: AHardwareBuffer_Format =
    59;
pub type AHardwareBuffer_Format = ::std::os::raw::c_uint;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_READ_NEVER:
    AHardwareBuffer_UsageFlags = 0;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_READ_RARELY:
    AHardwareBuffer_UsageFlags = 2;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_READ_OFTEN:
    AHardwareBuffer_UsageFlags = 3;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_READ_MASK:
    AHardwareBuffer_UsageFlags = 15;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_WRITE_NEVER:
    AHardwareBuffer_UsageFlags = 0;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_WRITE_RARELY:
    AHardwareBuffer_UsageFlags = 32;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_WRITE_OFTEN:
    AHardwareBuffer_UsageFlags = 48;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_CPU_WRITE_MASK:
    AHardwareBuffer_UsageFlags = 240;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_GPU_SAMPLED_IMAGE:
    AHardwareBuffer_UsageFlags = 256;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_GPU_FRAMEBUFFER:
    AHardwareBuffer_UsageFlags = 512;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_GPU_COLOR_OUTPUT:
    AHardwareBuffer_UsageFlags = 512;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_COMPOSER_OVERLAY:
    AHardwareBuffer_UsageFlags = 2048;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_PROTECTED_CONTENT:
    AHardwareBuffer_UsageFlags = 16384;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VIDEO_ENCODE:
    AHardwareBuffer_UsageFlags = 65536;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_SENSOR_DIRECT_DATA:
    AHardwareBuffer_UsageFlags = 8388608;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_GPU_DATA_BUFFER:
    AHardwareBuffer_UsageFlags = 16777216;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_GPU_CUBE_MAP:
    AHardwareBuffer_UsageFlags = 33554432;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_GPU_MIPMAP_COMPLETE:
    AHardwareBuffer_UsageFlags = 67108864;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_FRONT_BUFFER:
    AHardwareBuffer_UsageFlags = 4294967296;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_0: AHardwareBuffer_UsageFlags =
    268435456;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_1: AHardwareBuffer_UsageFlags =
    536870912;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_2: AHardwareBuffer_UsageFlags =
    1073741824;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_3: AHardwareBuffer_UsageFlags =
    2147483648;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_4: AHardwareBuffer_UsageFlags =
    281474976710656;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_5: AHardwareBuffer_UsageFlags =
    562949953421312;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_6: AHardwareBuffer_UsageFlags =
    1125899906842624;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_7: AHardwareBuffer_UsageFlags =
    2251799813685248;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_8: AHardwareBuffer_UsageFlags =
    4503599627370496;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_9: AHardwareBuffer_UsageFlags =
    9007199254740992;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_10: AHardwareBuffer_UsageFlags =
    18014398509481984;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_11: AHardwareBuffer_UsageFlags =
    36028797018963968;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_12: AHardwareBuffer_UsageFlags =
    72057594037927936;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_13: AHardwareBuffer_UsageFlags =
    144115188075855872;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_14: AHardwareBuffer_UsageFlags =
    288230376151711744;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_15: AHardwareBuffer_UsageFlags =
    576460752303423488;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_16: AHardwareBuffer_UsageFlags =
    1152921504606846976;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_17: AHardwareBuffer_UsageFlags =
    2305843009213693952;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_18: AHardwareBuffer_UsageFlags =
    4611686018427387904;
pub const AHardwareBuffer_UsageFlags_AHARDWAREBUFFER_USAGE_VENDOR_19: AHardwareBuffer_UsageFlags =
    9223372036854775808;
pub type AHardwareBuffer_UsageFlags = ::std::os::raw::c_ulonglong;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct AHardwareBuffer_Desc {
    pub width: u32,
    pub height: u32,
    pub layers: u32,
    pub format: u32,
    pub usage: u64,
    pub stride: u32,
    pub rfu0: u32,
    pub rfu1: u64,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of AHardwareBuffer_Desc"][::std::mem::size_of::<AHardwareBuffer_Desc>() - 40usize];
    ["Alignment of AHardwareBuffer_Desc"][::std::mem::align_of::<AHardwareBuffer_Desc>() - 4usize];
    ["Offset of field: AHardwareBuffer_Desc::width"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, width) - 0usize];
    ["Offset of field: AHardwareBuffer_Desc::height"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, height) - 4usize];
    ["Offset of field: AHardwareBuffer_Desc::layers"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, layers) - 8usize];
    ["Offset of field: AHardwareBuffer_Desc::format"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, format) - 12usize];
    ["Offset of field: AHardwareBuffer_Desc::usage"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, usage) - 16usize];
    ["Offset of field: AHardwareBuffer_Desc::stride"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, stride) - 24usize];
    ["Offset of field: AHardwareBuffer_Desc::rfu0"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, rfu0) - 28usize];
    ["Offset of field: AHardwareBuffer_Desc::rfu1"]
        [::std::mem::offset_of!(AHardwareBuffer_Desc, rfu1) - 32usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct AHardwareBuffer_Plane {
    pub data: *mut ::std::os::raw::c_void,
    pub pixelStride: u32,
    pub rowStride: u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of AHardwareBuffer_Plane"][::std::mem::size_of::<AHardwareBuffer_Plane>() - 12usize];
    ["Alignment of AHardwareBuffer_Plane"]
        [::std::mem::align_of::<AHardwareBuffer_Plane>() - 4usize];
    ["Offset of field: AHardwareBuffer_Plane::data"]
        [::std::mem::offset_of!(AHardwareBuffer_Plane, data) - 0usize];
    ["Offset of field: AHardwareBuffer_Plane::pixelStride"]
        [::std::mem::offset_of!(AHardwareBuffer_Plane, pixelStride) - 4usize];
    ["Offset of field: AHardwareBuffer_Plane::rowStride"]
        [::std::mem::offset_of!(AHardwareBuffer_Plane, rowStride) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct AHardwareBuffer_Planes {
    pub planeCount: u32,
    pub planes: [AHardwareBuffer_Plane; 4usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of AHardwareBuffer_Planes"][::std::mem::size_of::<AHardwareBuffer_Planes>() - 52usize];
    ["Alignment of AHardwareBuffer_Planes"]
        [::std::mem::align_of::<AHardwareBuffer_Planes>() - 4usize];
    ["Offset of field: AHardwareBuffer_Planes::planeCount"]
        [::std::mem::offset_of!(AHardwareBuffer_Planes, planeCount) - 0usize];
    ["Offset of field: AHardwareBuffer_Planes::planes"]
        [::std::mem::offset_of!(AHardwareBuffer_Planes, planes) - 4usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct AHardwareBuffer {
    _unused: [u8; 0],
}
unsafe extern "C" {
    pub fn AHardwareBuffer_allocate(
        desc: *const AHardwareBuffer_Desc,
        outBuffer: *mut *mut AHardwareBuffer,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_acquire(buffer: *mut AHardwareBuffer);
}
unsafe extern "C" {
    pub fn AHardwareBuffer_release(buffer: *mut AHardwareBuffer);
}
unsafe extern "C" {
    pub fn AHardwareBuffer_describe(
        buffer: *const AHardwareBuffer,
        outDesc: *mut AHardwareBuffer_Desc,
    );
}
unsafe extern "C" {
    pub fn AHardwareBuffer_lock(
        buffer: *mut AHardwareBuffer,
        usage: u64,
        fence: i32,
        rect: *const ARect,
        outVirtualAddress: *mut *mut ::std::os::raw::c_void,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_unlock(
        buffer: *mut AHardwareBuffer,
        fence: *mut i32,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_sendHandleToUnixSocket(
        buffer: *const AHardwareBuffer,
        socketFd: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_recvHandleFromUnixSocket(
        socketFd: ::std::os::raw::c_int,
        outBuffer: *mut *mut AHardwareBuffer,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_lockPlanes(
        buffer: *mut AHardwareBuffer,
        usage: u64,
        fence: i32,
        rect: *const ARect,
        outPlanes: *mut AHardwareBuffer_Planes,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_isSupported(desc: *const AHardwareBuffer_Desc) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_lockAndGetInfo(
        buffer: *mut AHardwareBuffer,
        usage: u64,
        fence: i32,
        rect: *const ARect,
        outVirtualAddress: *mut *mut ::std::os::raw::c_void,
        outBytesPerPixel: *mut i32,
        outBytesPerStride: *mut i32,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn AHardwareBuffer_getId(
        buffer: *const AHardwareBuffer,
        outId: *mut u64,
    ) -> ::std::os::raw::c_int;
}
#[doc = " \\brief Describe information about a pointer, found in a\n GameActivityMotionEvent.\n\n You can read values directly from this structure, or use helper functions\n (`GameActivityPointerAxes_getX`, `GameActivityPointerAxes_getY` and\n `GameActivityPointerAxes_getAxisValue`).\n\n The X axis and Y axis are enabled by default but any other axis that you want\n to read **must** be enabled first, using\n `GameActivityPointerAxes_enableAxis`.\n\n \\see GameActivityMotionEvent"]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameActivityPointerAxes {
    pub id: i32,
    pub toolType: i32,
    pub axisValues: [f32; 48usize],
    pub rawX: f32,
    pub rawY: f32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameActivityPointerAxes"]
        [::std::mem::size_of::<GameActivityPointerAxes>() - 208usize];
    ["Alignment of GameActivityPointerAxes"]
        [::std::mem::align_of::<GameActivityPointerAxes>() - 4usize];
    ["Offset of field: GameActivityPointerAxes::id"]
        [::std::mem::offset_of!(GameActivityPointerAxes, id) - 0usize];
    ["Offset of field: GameActivityPointerAxes::toolType"]
        [::std::mem::offset_of!(GameActivityPointerAxes, toolType) - 4usize];
    ["Offset of field: GameActivityPointerAxes::axisValues"]
        [::std::mem::offset_of!(GameActivityPointerAxes, axisValues) - 8usize];
    ["Offset of field: GameActivityPointerAxes::rawX"]
        [::std::mem::offset_of!(GameActivityPointerAxes, rawX) - 200usize];
    ["Offset of field: GameActivityPointerAxes::rawY"]
        [::std::mem::offset_of!(GameActivityPointerAxes, rawY) - 204usize];
};
unsafe extern "C" {
    #[doc = " \\brief Enable the specified axis, so that its value is reported in the\n GameActivityPointerAxes structures stored in a motion event.\n\n You must enable any axis that you want to read, apart from\n `AMOTION_EVENT_AXIS_X` and `AMOTION_EVENT_AXIS_Y` that are enabled by\n default.\n\n If the axis index is out of range, nothing is done."]
    pub fn GameActivityPointerAxes_enableAxis(axis: i32);
}
unsafe extern "C" {
    #[doc = " \\brief Disable the specified axis. Its value won't be reported in the\n GameActivityPointerAxes structures stored in a motion event anymore.\n\n Apart from X and Y, any axis that you want to read **must** be enabled first,\n using `GameActivityPointerAxes_enableAxis`.\n\n If the axis index is out of range, nothing is done."]
    pub fn GameActivityPointerAxes_disableAxis(axis: i32);
}
unsafe extern "C" {
    #[doc = " \\brief Get the value of the requested axis.\n\n Apart from X and Y, any axis that you want to read **must** be enabled first,\n using `GameActivityPointerAxes_enableAxis`.\n\n Find the valid enums for the axis (`AMOTION_EVENT_AXIS_X`,\n `AMOTION_EVENT_AXIS_Y`, `AMOTION_EVENT_AXIS_PRESSURE`...)\n in https://developer.android.com/ndk/reference/group/input.\n\n @param pointerInfo The structure containing information about the pointer,\n obtained from GameActivityMotionEvent.\n @param axis The axis to get the value from\n @return The value of the axis, or 0 if the axis is invalid or was not\n enabled."]
    pub fn GameActivityPointerAxes_getAxisValue(
        pointerInfo: *const GameActivityPointerAxes,
        axis: i32,
    ) -> f32;
}
#[doc = " \\brief Describe a motion event that happened on the GameActivity SurfaceView.\n\n This is 1:1 mapping to the information contained in a Java `MotionEvent`\n (see https://developer.android.com/reference/android/view/MotionEvent)."]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameActivityMotionEvent {
    pub deviceId: i32,
    pub source: i32,
    pub action: i32,
    pub eventTime: i64,
    pub downTime: i64,
    pub flags: i32,
    pub metaState: i32,
    pub actionButton: i32,
    pub buttonState: i32,
    pub classification: i32,
    pub edgeFlags: i32,
    pub pointerCount: u32,
    pub pointers: [GameActivityPointerAxes; 8usize],
    pub historySize: ::std::os::raw::c_int,
    pub historicalEventTimesMillis: *mut i64,
    pub historicalEventTimesNanos: *mut i64,
    pub historicalAxisValues: *mut f32,
    pub precisionX: f32,
    pub precisionY: f32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameActivityMotionEvent"]
        [::std::mem::size_of::<GameActivityMotionEvent>() - 1744usize];
    ["Alignment of GameActivityMotionEvent"]
        [::std::mem::align_of::<GameActivityMotionEvent>() - 4usize];
    ["Offset of field: GameActivityMotionEvent::deviceId"]
        [::std::mem::offset_of!(GameActivityMotionEvent, deviceId) - 0usize];
    ["Offset of field: GameActivityMotionEvent::source"]
        [::std::mem::offset_of!(GameActivityMotionEvent, source) - 4usize];
    ["Offset of field: GameActivityMotionEvent::action"]
        [::std::mem::offset_of!(GameActivityMotionEvent, action) - 8usize];
    ["Offset of field: GameActivityMotionEvent::eventTime"]
        [::std::mem::offset_of!(GameActivityMotionEvent, eventTime) - 12usize];
    ["Offset of field: GameActivityMotionEvent::downTime"]
        [::std::mem::offset_of!(GameActivityMotionEvent, downTime) - 20usize];
    ["Offset of field: GameActivityMotionEvent::flags"]
        [::std::mem::offset_of!(GameActivityMotionEvent, flags) - 28usize];
    ["Offset of field: GameActivityMotionEvent::metaState"]
        [::std::mem::offset_of!(GameActivityMotionEvent, metaState) - 32usize];
    ["Offset of field: GameActivityMotionEvent::actionButton"]
        [::std::mem::offset_of!(GameActivityMotionEvent, actionButton) - 36usize];
    ["Offset of field: GameActivityMotionEvent::buttonState"]
        [::std::mem::offset_of!(GameActivityMotionEvent, buttonState) - 40usize];
    ["Offset of field: GameActivityMotionEvent::classification"]
        [::std::mem::offset_of!(GameActivityMotionEvent, classification) - 44usize];
    ["Offset of field: GameActivityMotionEvent::edgeFlags"]
        [::std::mem::offset_of!(GameActivityMotionEvent, edgeFlags) - 48usize];
    ["Offset of field: GameActivityMotionEvent::pointerCount"]
        [::std::mem::offset_of!(GameActivityMotionEvent, pointerCount) - 52usize];
    ["Offset of field: GameActivityMotionEvent::pointers"]
        [::std::mem::offset_of!(GameActivityMotionEvent, pointers) - 56usize];
    ["Offset of field: GameActivityMotionEvent::historySize"]
        [::std::mem::offset_of!(GameActivityMotionEvent, historySize) - 1720usize];
    ["Offset of field: GameActivityMotionEvent::historicalEventTimesMillis"]
        [::std::mem::offset_of!(GameActivityMotionEvent, historicalEventTimesMillis) - 1724usize];
    ["Offset of field: GameActivityMotionEvent::historicalEventTimesNanos"]
        [::std::mem::offset_of!(GameActivityMotionEvent, historicalEventTimesNanos) - 1728usize];
    ["Offset of field: GameActivityMotionEvent::historicalAxisValues"]
        [::std::mem::offset_of!(GameActivityMotionEvent, historicalAxisValues) - 1732usize];
    ["Offset of field: GameActivityMotionEvent::precisionX"]
        [::std::mem::offset_of!(GameActivityMotionEvent, precisionX) - 1736usize];
    ["Offset of field: GameActivityMotionEvent::precisionY"]
        [::std::mem::offset_of!(GameActivityMotionEvent, precisionY) - 1740usize];
};
unsafe extern "C" {
    pub fn GameActivityMotionEvent_getHistoricalAxisValue(
        event: *const GameActivityMotionEvent,
        axis: ::std::os::raw::c_int,
        pointerIndex: ::std::os::raw::c_int,
        historyPos: ::std::os::raw::c_int,
    ) -> f32;
}
unsafe extern "C" {
    #[doc = " \\brief Handle the freeing of the GameActivityMotionEvent struct."]
    pub fn GameActivityMotionEvent_destroy(c_event: *mut GameActivityMotionEvent);
}
#[doc = " \\brief Describe a key event that happened on the GameActivity SurfaceView.\n\n This is 1:1 mapping to the information contained in a Java `KeyEvent`\n (see https://developer.android.com/reference/android/view/KeyEvent).\n The only exception is the event times, which are reported as\n nanoseconds in this struct."]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameActivityKeyEvent {
    pub deviceId: i32,
    pub source: i32,
    pub action: i32,
    pub eventTime: i64,
    pub downTime: i64,
    pub flags: i32,
    pub metaState: i32,
    pub modifiers: i32,
    pub repeatCount: i32,
    pub keyCode: i32,
    pub scanCode: i32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameActivityKeyEvent"][::std::mem::size_of::<GameActivityKeyEvent>() - 52usize];
    ["Alignment of GameActivityKeyEvent"][::std::mem::align_of::<GameActivityKeyEvent>() - 4usize];
    ["Offset of field: GameActivityKeyEvent::deviceId"]
        [::std::mem::offset_of!(GameActivityKeyEvent, deviceId) - 0usize];
    ["Offset of field: GameActivityKeyEvent::source"]
        [::std::mem::offset_of!(GameActivityKeyEvent, source) - 4usize];
    ["Offset of field: GameActivityKeyEvent::action"]
        [::std::mem::offset_of!(GameActivityKeyEvent, action) - 8usize];
    ["Offset of field: GameActivityKeyEvent::eventTime"]
        [::std::mem::offset_of!(GameActivityKeyEvent, eventTime) - 12usize];
    ["Offset of field: GameActivityKeyEvent::downTime"]
        [::std::mem::offset_of!(GameActivityKeyEvent, downTime) - 20usize];
    ["Offset of field: GameActivityKeyEvent::flags"]
        [::std::mem::offset_of!(GameActivityKeyEvent, flags) - 28usize];
    ["Offset of field: GameActivityKeyEvent::metaState"]
        [::std::mem::offset_of!(GameActivityKeyEvent, metaState) - 32usize];
    ["Offset of field: GameActivityKeyEvent::modifiers"]
        [::std::mem::offset_of!(GameActivityKeyEvent, modifiers) - 36usize];
    ["Offset of field: GameActivityKeyEvent::repeatCount"]
        [::std::mem::offset_of!(GameActivityKeyEvent, repeatCount) - 40usize];
    ["Offset of field: GameActivityKeyEvent::keyCode"]
        [::std::mem::offset_of!(GameActivityKeyEvent, keyCode) - 44usize];
    ["Offset of field: GameActivityKeyEvent::scanCode"]
        [::std::mem::offset_of!(GameActivityKeyEvent, scanCode) - 48usize];
};
#[doc = " This struct holds a span within a region of text from start (inclusive) to\n end (exclusive). An empty span or cursor position is specified with\n start==end. An undefined span is specified with start = end = SPAN_UNDEFINED."]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameTextInputSpan {
    #[doc = " The start of the region (inclusive)."]
    pub start: i32,
    #[doc = " The end of the region (exclusive)."]
    pub end: i32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameTextInputSpan"][::std::mem::size_of::<GameTextInputSpan>() - 8usize];
    ["Alignment of GameTextInputSpan"][::std::mem::align_of::<GameTextInputSpan>() - 4usize];
    ["Offset of field: GameTextInputSpan::start"]
        [::std::mem::offset_of!(GameTextInputSpan, start) - 0usize];
    ["Offset of field: GameTextInputSpan::end"]
        [::std::mem::offset_of!(GameTextInputSpan, end) - 4usize];
};
pub const GameTextInputSpanFlag_SPAN_UNDEFINED: GameTextInputSpanFlag = -1;
#[doc = " Values with special meaning in a GameTextInputSpan."]
pub type GameTextInputSpanFlag = i32;
#[doc = " This struct holds the state of an editable section of text.\n The text can have a selection and a composing region defined on it.\n A composing region is used by IMEs that allow input using multiple steps to\n compose a glyph or word. Use functions GameTextInput_getState and\n GameTextInput_setState to read and modify the state that an IME is editing."]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameTextInputState {
    #[doc = " Text owned by the state, as a modified UTF-8 string. Null-terminated.\n https://en.wikipedia.org/wiki/UTF-8#Modified_UTF-8"]
    pub text_UTF8: *const ::std::os::raw::c_char,
    #[doc = " Length in bytes of text_UTF8, *not* including the null at end."]
    pub text_length: i32,
    #[doc = " A selection defined on the text."]
    pub selection: GameTextInputSpan,
    #[doc = " A composing region defined on the text."]
    pub composingRegion: GameTextInputSpan,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameTextInputState"][::std::mem::size_of::<GameTextInputState>() - 24usize];
    ["Alignment of GameTextInputState"][::std::mem::align_of::<GameTextInputState>() - 4usize];
    ["Offset of field: GameTextInputState::text_UTF8"]
        [::std::mem::offset_of!(GameTextInputState, text_UTF8) - 0usize];
    ["Offset of field: GameTextInputState::text_length"]
        [::std::mem::offset_of!(GameTextInputState, text_length) - 4usize];
    ["Offset of field: GameTextInputState::selection"]
        [::std::mem::offset_of!(GameTextInputState, selection) - 8usize];
    ["Offset of field: GameTextInputState::composingRegion"]
        [::std::mem::offset_of!(GameTextInputState, composingRegion) - 16usize];
};
#[doc = " A callback called by GameTextInput_getState.\n @param context User-defined context.\n @param state State, owned by the library, that will be valid for the duration\n of the callback."]
pub type GameTextInputGetStateCallback = ::std::option::Option<
    unsafe extern "C" fn(context: *mut ::std::os::raw::c_void, state: *const GameTextInputState),
>;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameTextInput {
    _unused: [u8; 0],
}
unsafe extern "C" {
    #[doc = " Initialize the GameTextInput library.\n If called twice without GameTextInput_destroy being called, the same pointer\n will be returned and a warning will be issued.\n @param env A JNI env valid on the calling thread. All other calls to the resulting GameTextInput\n object must be done on the same calling thread.\n @param max_string_size The maximum length of a string that can be edited. If\n zero, the maximum defaults to 65536 bytes. A buffer of this size is allocated\n at initialization.\n @return A handle to the library."]
    pub fn GameTextInput_init(env: *mut JNIEnv, max_string_size: u32) -> *mut GameTextInput;
}
unsafe extern "C" {
    #[doc = " When using GameTextInput, you need to create a gametextinput.InputConnection\n on the Java side and pass it using this function to the library, unless using\n GameActivity in which case this will be done for you. See the GameActivity\n source code or GameTextInput samples for examples of usage.\n @param input A valid GameTextInput library handle.\n @param inputConnection A gametextinput.InputConnection object."]
    pub fn GameTextInput_setInputConnection(input: *mut GameTextInput, inputConnection: jobject);
}
unsafe extern "C" {
    #[doc = " Unless using GameActivity, it is required to call this function from your\n Java gametextinput.Listener.stateChanged method to convert eventState and\n trigger any event callbacks. When using GameActivity, this does not need to\n be called as event processing is handled by the Activity.\n @param input A valid GameTextInput library handle.\n @param eventState A Java gametextinput.State object."]
    pub fn GameTextInput_processEvent(input: *mut GameTextInput, eventState: jobject);
}
unsafe extern "C" {
    #[doc = " Free any resources owned by the GameTextInput library.\n Any subsequent calls to the library will fail until GameTextInput_init is\n called again.\n @param input A valid GameTextInput library handle."]
    pub fn GameTextInput_destroy(input: *mut GameTextInput);
}
pub const ShowImeFlags_SHOW_IME_UNDEFINED: ShowImeFlags = 0;
pub const ShowImeFlags_SHOW_IMPLICIT: ShowImeFlags = 1;
pub const ShowImeFlags_SHOW_FORCED: ShowImeFlags = 2;
#[doc = " Flags to be passed to GameTextInput_showIme."]
pub type ShowImeFlags = u32;
unsafe extern "C" {
    #[doc = " Show the IME. Calls InputMethodManager.showSoftInput().\n @param input A valid GameTextInput library handle.\n @param flags Defined in ShowImeFlags above. For more information see:\n https://developer.android.com/reference/android/view/inputmethod/InputMethodManager"]
    pub fn GameTextInput_showIme(input: *mut GameTextInput, flags: u32);
}
pub const HideImeFlags_HIDE_IME_UNDEFINED: HideImeFlags = 0;
pub const HideImeFlags_HIDE_IMPLICIT_ONLY: HideImeFlags = 1;
pub const HideImeFlags_HIDE_NOT_ALWAYS: HideImeFlags = 2;
#[doc = " Flags to be passed to GameTextInput_hideIme."]
pub type HideImeFlags = u32;
unsafe extern "C" {
    #[doc = " Hide the IME. Calls InputMethodManager.hideSoftInputFromWindow().\n @param input A valid GameTextInput library handle.\n @param flags Defined in HideImeFlags above. For more information see:\n https://developer.android.com/reference/android/view/inputmethod/InputMethodManager"]
    pub fn GameTextInput_hideIme(input: *mut GameTextInput, flags: u32);
}
unsafe extern "C" {
    #[doc = " Restarts the input method. Calls InputMethodManager.restartInput().\n @param input A valid GameTextInput library handle."]
    pub fn GameTextInput_restartInput(input: *mut GameTextInput);
}
unsafe extern "C" {
    #[doc = " Call a callback with the current GameTextInput state, which may have been\n modified by changes in the IME and calls to GameTextInput_setState. We use a\n callback rather than returning the state in order to simplify ownership of\n text_UTF8 strings. These strings are only valid during the calling of the\n callback.\n @param input A valid GameTextInput library handle.\n @param callback A function that will be called with valid state.\n @param context Context used by the callback."]
    pub fn GameTextInput_getState(
        input: *mut GameTextInput,
        callback: GameTextInputGetStateCallback,
        context: *mut ::std::os::raw::c_void,
    );
}
unsafe extern "C" {
    #[doc = " Set the current GameTextInput state. This state is reflected to any active\n IME.\n @param input A valid GameTextInput library handle.\n @param state The state to set. Ownership is maintained by the caller and must\n remain valid for the duration of the call."]
    pub fn GameTextInput_setState(input: *mut GameTextInput, state: *const GameTextInputState);
}
#[doc = " Type of the callback needed by GameTextInput_setEventCallback that will be\n called every time the IME state changes.\n @param context User-defined context set in GameTextInput_setEventCallback.\n @param current_state Current IME state, owned by the library and valid during\n the callback."]
pub type GameTextInputEventCallback = ::std::option::Option<
    unsafe extern "C" fn(
        context: *mut ::std::os::raw::c_void,
        current_state: *const GameTextInputState,
    ),
>;
unsafe extern "C" {
    #[doc = " Optionally set a callback to be called whenever the IME state changes.\n Not necessary if you are using GameActivity, which handles these callbacks\n for you.\n @param input A valid GameTextInput library handle.\n @param callback Called by the library when the IME state changes.\n @param context Context passed as first argument to the callback.\n <b>This function is deprecated. Don't perform any complex processing inside\n the callback other than copying the state variable. Using any synchronization\n primitives inside this callback may cause a deadlock.</b>"]
    pub fn GameTextInput_setEventCallback(
        input: *mut GameTextInput,
        callback: GameTextInputEventCallback,
        context: *mut ::std::os::raw::c_void,
    );
}
#[doc = " Type of the callback needed by GameTextInput_setImeInsetsCallback that will\n be called every time the IME window insets change.\n @param context User-defined context set in\n GameTextInput_setImeWIndowInsetsCallback.\n @param current_insets Current IME insets, owned by the library and valid\n during the callback."]
pub type GameTextInputImeInsetsCallback = ::std::option::Option<
    unsafe extern "C" fn(context: *mut ::std::os::raw::c_void, current_insets: *const ARect),
>;
unsafe extern "C" {
    #[doc = " Optionally set a callback to be called whenever the IME insets change.\n Not necessary if you are using GameActivity, which handles these callbacks\n for you.\n @param input A valid GameTextInput library handle.\n @param callback Called by the library when the IME insets change.\n @param context Context passed as first argument to the callback."]
    pub fn GameTextInput_setImeInsetsCallback(
        input: *mut GameTextInput,
        callback: GameTextInputImeInsetsCallback,
        context: *mut ::std::os::raw::c_void,
    );
}
unsafe extern "C" {
    #[doc = " Get the current window insets for the IME.\n @param input A valid GameTextInput library handle.\n @param insets Filled with the current insets by this function."]
    pub fn GameTextInput_getImeInsets(input: *const GameTextInput, insets: *mut ARect);
}
unsafe extern "C" {
    #[doc = " Unless using GameActivity, it is required to call this function from your\n Java gametextinput.Listener.onImeInsetsChanged method to\n trigger any event callbacks. When using GameActivity, this does not need to\n be called as insets processing is handled by the Activity.\n @param input A valid GameTextInput library handle.\n @param eventState A Java gametextinput.State object."]
    pub fn GameTextInput_processImeInsets(input: *mut GameTextInput, insets: *const ARect);
}
unsafe extern "C" {
    #[doc = " Convert a GameTextInputState struct to a Java gametextinput.State object.\n Don't forget to delete the returned Java local ref when you're done.\n @param input A valid GameTextInput library handle.\n @param state Input state to convert.\n @return A Java object of class gametextinput.State. The caller is required to\n delete this local reference."]
    pub fn GameTextInputState_toJava(
        input: *const GameTextInput,
        state: *const GameTextInputState,
    ) -> jobject;
}
unsafe extern "C" {
    #[doc = " Convert from a Java gametextinput.State object into a C GameTextInputState\n struct.\n @param input A valid GameTextInput library handle.\n @param state A Java gametextinput.State object.\n @param callback A function called with the C struct, valid for the duration\n of the call.\n @param context Context passed to the callback."]
    pub fn GameTextInputState_fromJava(
        input: *const GameTextInput,
        state: jobject,
        callback: GameTextInputGetStateCallback,
        context: *mut ::std::os::raw::c_void,
    );
}
#[doc = " Mask of bits that determine the overall class\n of text being given.  Currently supported classes are:\n {@link #TYPE_CLASS_TEXT}, {@link #TYPE_CLASS_NUMBER},\n {@link #TYPE_CLASS_PHONE}, {@link #TYPE_CLASS_DATETIME}.\n <p>IME authors: If the class is not one you\n understand, assume {@link #TYPE_CLASS_TEXT} with NO variation\n or flags.<p>"]
pub const GameTextInputType_TYPE_MASK_CLASS: GameTextInputType = 15;
#[doc = " Mask of bits that determine the variation of\n the base content class."]
pub const GameTextInputType_TYPE_MASK_VARIATION: GameTextInputType = 4080;
#[doc = " Mask of bits that provide addition bit flags\n of options."]
pub const GameTextInputType_TYPE_MASK_FLAGS: GameTextInputType = 16773120;
#[doc = " Special content type for when no explicit type has been specified.\n This should be interpreted to mean that the target input connection\n is not rich, it can not process and show things like candidate text nor\n retrieve the current text, so the input method will need to run in a\n limited \"generate key events\" mode, if it supports it. Note that some\n input methods may not support it, for example a voice-based input\n method will likely not be able to generate key events even if this\n flag is set."]
pub const GameTextInputType_TYPE_NULL: GameTextInputType = 0;
#[doc = " Class for normal text.  This class supports the following flags (only\n one of which should be set):\n {@link #TYPE_TEXT_FLAG_CAP_CHARACTERS},\n {@link #TYPE_TEXT_FLAG_CAP_WORDS}, and.\n {@link #TYPE_TEXT_FLAG_CAP_SENTENCES}.  It also supports the\n following variations:\n {@link #TYPE_TEXT_VARIATION_NORMAL}, and\n {@link #TYPE_TEXT_VARIATION_URI}.  If you do not recognize the\n variation, normal should be assumed."]
pub const GameTextInputType_TYPE_CLASS_TEXT: GameTextInputType = 1;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: capitalize all characters.  Overrides\n {@link #TYPE_TEXT_FLAG_CAP_WORDS} and\n {@link #TYPE_TEXT_FLAG_CAP_SENTENCES}.  This value is explicitly defined\n to be the same as {@link TextUtils#CAP_MODE_CHARACTERS}. Of course,\n this only affects languages where there are upper-case and lower-case\n letters."]
pub const GameTextInputType_TYPE_TEXT_FLAG_CAP_CHARACTERS: GameTextInputType = 4096;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: capitalize the first character of\n every word.  Overrides {@link #TYPE_TEXT_FLAG_CAP_SENTENCES}.  This\n value is explicitly defined\n to be the same as {@link TextUtils#CAP_MODE_WORDS}. Of course,\n this only affects languages where there are upper-case and lower-case\n letters."]
pub const GameTextInputType_TYPE_TEXT_FLAG_CAP_WORDS: GameTextInputType = 8192;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: capitalize the first character of\n each sentence.  This value is explicitly defined\n to be the same as {@link TextUtils#CAP_MODE_SENTENCES}. For example\n in English it means to capitalize after a period and a space (note that\n other languages may have different characters for period, or not use\n spaces, or use different grammatical rules). Of course, this only affects\n languages where there are upper-case and lower-case letters."]
pub const GameTextInputType_TYPE_TEXT_FLAG_CAP_SENTENCES: GameTextInputType = 16384;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: the user is entering free-form\n text that should have auto-correction applied to it. Without this flag,\n the IME will not try to correct typos. You should always set this flag\n unless you really expect users to type non-words in this field, for\n example to choose a name for a character in a game.\n Contrast this with {@link #TYPE_TEXT_FLAG_AUTO_COMPLETE} and\n {@link #TYPE_TEXT_FLAG_NO_SUGGESTIONS}:\n {@code TYPE_TEXT_FLAG_AUTO_CORRECT} means that the IME will try to\n auto-correct typos as the user is typing, but does not define whether\n the IME offers an interface to show suggestions."]
pub const GameTextInputType_TYPE_TEXT_FLAG_AUTO_CORRECT: GameTextInputType = 32768;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: the text editor (which means\n the application) is performing auto-completion of the text being entered\n based on its own semantics, which it will present to the user as they type.\n This generally means that the input method should not be showing\n candidates itself, but can expect the editor to supply its own\n completions/candidates from\n {@link android.view.inputmethod.InputMethodSession#displayCompletions\n InputMethodSession.displayCompletions()} as a result of the editor calling\n {@link android.view.inputmethod.InputMethodManager#displayCompletions\n InputMethodManager.displayCompletions()}.\n Note the contrast with {@link #TYPE_TEXT_FLAG_AUTO_CORRECT} and\n {@link #TYPE_TEXT_FLAG_NO_SUGGESTIONS}:\n {@code TYPE_TEXT_FLAG_AUTO_COMPLETE} means the editor should show an\n interface for displaying suggestions, but instead of supplying its own\n it will rely on the Editor to pass completions/corrections."]
pub const GameTextInputType_TYPE_TEXT_FLAG_AUTO_COMPLETE: GameTextInputType = 65536;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: multiple lines of text can be\n entered into the field.  If this flag is not set, the text field\n will be constrained to a single line. The IME may also choose not to\n display an enter key when this flag is not set, as there should be no\n need to create new lines."]
pub const GameTextInputType_TYPE_TEXT_FLAG_MULTI_LINE: GameTextInputType = 131072;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: the regular text view associated\n with this should not be multi-line, but when a fullscreen input method\n is providing text it should use multiple lines if it can."]
pub const GameTextInputType_TYPE_TEXT_FLAG_IME_MULTI_LINE: GameTextInputType = 262144;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: the input method does not need to\n display any dictionary-based candidates. This is useful for text views that\n do not contain words from the language and do not benefit from any\n dictionary-based completions or corrections. It overrides the\n {@link #TYPE_TEXT_FLAG_AUTO_CORRECT} value when set.\n Please avoid using this unless you are certain this is what you want.\n Many input methods need suggestions to work well, for example the ones\n based on gesture typing. Consider clearing\n {@link #TYPE_TEXT_FLAG_AUTO_CORRECT} instead if you just do not\n want the IME to correct typos.\n Note the contrast with {@link #TYPE_TEXT_FLAG_AUTO_CORRECT} and\n {@link #TYPE_TEXT_FLAG_AUTO_COMPLETE}:\n {@code TYPE_TEXT_FLAG_NO_SUGGESTIONS} means the IME does not need to\n show an interface to display suggestions. Most IMEs will also take this to\n mean they do not need to try to auto-correct what the user is typing."]
pub const GameTextInputType_TYPE_TEXT_FLAG_NO_SUGGESTIONS: GameTextInputType = 524288;
#[doc = " Flag for {@link #TYPE_CLASS_TEXT}: Let the IME know the text conversion\n suggestions are required by the application. Text conversion suggestion is\n for the transliteration languages which has pronunciation characters and\n target characters. When the user is typing the pronunciation charactes, the\n IME could provide the possible target characters to the user. When this\n flag is set, the IME should insert the text conversion suggestions through\n {@link Builder#setTextConversionSuggestions(List)} and\n the {@link TextAttribute} with initialized with the text conversion\n suggestions is provided by the IME to the application. To receive the\n additional information, the application needs to implement {@link\n InputConnection#setComposingText(CharSequence, int, TextAttribute)},\n {@link InputConnection#setComposingRegion(int, int, TextAttribute)}, and\n {@link InputConnection#commitText(CharSequence, int, TextAttribute)}."]
pub const GameTextInputType_TYPE_TEXT_FLAG_ENABLE_TEXT_CONVERSION_SUGGESTIONS: GameTextInputType =
    1048576;
#[doc = " Default variation of {@link #TYPE_CLASS_TEXT}: plain old normal text."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_NORMAL: GameTextInputType = 0;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering a URI."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_URI: GameTextInputType = 16;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering an e-mail address."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_EMAIL_ADDRESS: GameTextInputType = 32;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering the subject line of\n an e-mail."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_EMAIL_SUBJECT: GameTextInputType = 48;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering a short, possibly informal\n message such as an instant message or a text message."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_SHORT_MESSAGE: GameTextInputType = 64;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering the content of a long,\n possibly formal message such as the body of an e-mail."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_LONG_MESSAGE: GameTextInputType = 80;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering the name of a person."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_PERSON_NAME: GameTextInputType = 96;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering a postal mailing address."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_POSTAL_ADDRESS: GameTextInputType = 112;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering a password."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_PASSWORD: GameTextInputType = 128;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering a password, which should\n be visible to the user."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_VISIBLE_PASSWORD: GameTextInputType = 144;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering text inside of a web form."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_WEB_EDIT_TEXT: GameTextInputType = 160;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering text to filter contents\n of a list etc."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_FILTER: GameTextInputType = 176;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering text for phonetic\n pronunciation, such as a phonetic name field in contacts. This is mostly\n useful for languages where one spelling may have several phonetic\n readings, like Japanese."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_PHONETIC: GameTextInputType = 192;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering e-mail address inside\n of a web form.  This was added in\n {@link android.os.Build.VERSION_CODES#HONEYCOMB}.  An IME must target\n this API version or later to see this input type; if it doesn't, a request\n for this type will be seen as {@link #TYPE_TEXT_VARIATION_EMAIL_ADDRESS}\n when passed through {@link\n android.view.inputmethod.EditorInfo#makeCompatible(int)\n EditorInfo.makeCompatible(int)}."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_WEB_EMAIL_ADDRESS: GameTextInputType = 208;
#[doc = " Variation of {@link #TYPE_CLASS_TEXT}: entering password inside\n of a web form.  This was added in\n {@link android.os.Build.VERSION_CODES#HONEYCOMB}.  An IME must target\n this API version or later to see this input type; if it doesn't, a request\n for this type will be seen as {@link #TYPE_TEXT_VARIATION_PASSWORD}\n when passed through {@link\n android.view.inputmethod.EditorInfo#makeCompatible(int)\n EditorInfo.makeCompatible(int)}."]
pub const GameTextInputType_TYPE_TEXT_VARIATION_WEB_PASSWORD: GameTextInputType = 224;
#[doc = " Class for numeric text.  This class supports the following flags:\n {@link #TYPE_NUMBER_FLAG_SIGNED} and\n {@link #TYPE_NUMBER_FLAG_DECIMAL}.  It also supports the following\n variations: {@link #TYPE_NUMBER_VARIATION_NORMAL} and\n {@link #TYPE_NUMBER_VARIATION_PASSWORD}.\n <p>IME authors: If you do not recognize\n the variation, normal should be assumed.</p>"]
pub const GameTextInputType_TYPE_CLASS_NUMBER: GameTextInputType = 2;
#[doc = " Flag of {@link #TYPE_CLASS_NUMBER}: the number is signed, allowing\n a positive or negative sign at the start."]
pub const GameTextInputType_TYPE_NUMBER_FLAG_SIGNED: GameTextInputType = 4096;
#[doc = " Flag of {@link #TYPE_CLASS_NUMBER}: the number is decimal, allowing\n a decimal point to provide fractional values."]
pub const GameTextInputType_TYPE_NUMBER_FLAG_DECIMAL: GameTextInputType = 8192;
#[doc = " Default variation of {@link #TYPE_CLASS_NUMBER}: plain normal\n numeric text.  This was added in\n {@link android.os.Build.VERSION_CODES#HONEYCOMB}.  An IME must target\n this API version or later to see this input type; if it doesn't, a request\n for this type will be dropped when passed through\n {@link android.view.inputmethod.EditorInfo#makeCompatible(int)\n EditorInfo.makeCompatible(int)}."]
pub const GameTextInputType_TYPE_NUMBER_VARIATION_NORMAL: GameTextInputType = 0;
#[doc = " Variation of {@link #TYPE_CLASS_NUMBER}: entering a numeric password.\n This was added in {@link android.os.Build.VERSION_CODES#HONEYCOMB}.  An\n IME must target this API version or later to see this input type; if it\n doesn't, a request for this type will be dropped when passed\n through {@link android.view.inputmethod.EditorInfo#makeCompatible(int)\n EditorInfo.makeCompatible(int)}."]
pub const GameTextInputType_TYPE_NUMBER_VARIATION_PASSWORD: GameTextInputType = 16;
#[doc = " Class for a phone number.  This class currently supports no variations\n or flags."]
pub const GameTextInputType_TYPE_CLASS_PHONE: GameTextInputType = 3;
#[doc = " Class for dates and times.  It supports the\n following variations:\n {@link #TYPE_DATETIME_VARIATION_NORMAL}\n {@link #TYPE_DATETIME_VARIATION_DATE}, and\n {@link #TYPE_DATETIME_VARIATION_TIME}."]
pub const GameTextInputType_TYPE_CLASS_DATETIME: GameTextInputType = 4;
#[doc = " Default variation of {@link #TYPE_CLASS_DATETIME}: allows entering\n both a date and time."]
pub const GameTextInputType_TYPE_DATETIME_VARIATION_NORMAL: GameTextInputType = 0;
#[doc = " Default variation of {@link #TYPE_CLASS_DATETIME}: allows entering\n only a date."]
pub const GameTextInputType_TYPE_DATETIME_VARIATION_DATE: GameTextInputType = 16;
#[doc = " Default variation of {@link #TYPE_CLASS_DATETIME}: allows entering\n only a time."]
pub const GameTextInputType_TYPE_DATETIME_VARIATION_TIME: GameTextInputType = 32;
#[doc = " Definitions for inputType argument of GameActivity_setImeEditorInfo()\n\n <pre>\n |-------|-------|-------|-------|\n                              1111 TYPE_MASK_CLASS\n                      11111111     TYPE_MASK_VARIATION\n          111111111111             TYPE_MASK_FLAGS\n |-------|-------|-------|-------|\n                                   TYPE_NULL\n |-------|-------|-------|-------|\n                                 1 TYPE_CLASS_TEXT\n                             1     TYPE_TEXT_VARIATION_URI\n                            1      TYPE_TEXT_VARIATION_EMAIL_ADDRESS\n                            11     TYPE_TEXT_VARIATION_EMAIL_SUBJECT\n                           1       TYPE_TEXT_VARIATION_SHORT_MESSAGE\n                           1 1     TYPE_TEXT_VARIATION_LONG_MESSAGE\n                           11      TYPE_TEXT_VARIATION_PERSON_NAME\n                           111     TYPE_TEXT_VARIATION_POSTAL_ADDRESS\n                          1        TYPE_TEXT_VARIATION_PASSWORD\n                          1  1     TYPE_TEXT_VARIATION_VISIBLE_PASSWORD\n                          1 1      TYPE_TEXT_VARIATION_WEB_EDIT_TEXT\n                          1 11     TYPE_TEXT_VARIATION_FILTER\n                          11       TYPE_TEXT_VARIATION_PHONETIC\n                          11 1     TYPE_TEXT_VARIATION_WEB_EMAIL_ADDRESS\n                          111      TYPE_TEXT_VARIATION_WEB_PASSWORD\n                     1             TYPE_TEXT_FLAG_CAP_CHARACTERS\n                    1              TYPE_TEXT_FLAG_CAP_WORDS\n                   1               TYPE_TEXT_FLAG_CAP_SENTENCES\n                  1                TYPE_TEXT_FLAG_AUTO_CORRECT\n                 1                 TYPE_TEXT_FLAG_AUTO_COMPLETE\n                1                  TYPE_TEXT_FLAG_MULTI_LINE\n               1                   TYPE_TEXT_FLAG_IME_MULTI_LINE\n              1                    TYPE_TEXT_FLAG_NO_SUGGESTIONS\n             1 TYPE_TEXT_FLAG_ENABLE_TEXT_CONVERSION_SUGGESTIONS\n |-------|-------|-------|-------|\n                                1  TYPE_CLASS_NUMBER\n                             1     TYPE_NUMBER_VARIATION_PASSWORD\n                     1             TYPE_NUMBER_FLAG_SIGNED\n                    1              TYPE_NUMBER_FLAG_DECIMAL\n |-------|-------|-------|-------|\n                                11 TYPE_CLASS_PHONE\n |-------|-------|-------|-------|\n                               1   TYPE_CLASS_DATETIME\n                             1     TYPE_DATETIME_VARIATION_DATE\n                            1      TYPE_DATETIME_VARIATION_TIME\n |-------|-------|-------|-------|</pre>"]
pub type GameTextInputType = u32;
#[doc = " Set of bits in {@link #imeOptions} that provide alternative actions\n associated with the \"enter\" key.  This both helps the IME provide\n better feedback about what the enter key will do, and also allows it\n to provide alternative mechanisms for providing that command."]
pub const GameTextInputActionType_IME_MASK_ACTION: GameTextInputActionType = 255;
#[doc = " Bits of {@link #IME_MASK_ACTION}: no specific action has been\n associated with this editor, let the editor come up with its own if\n it can."]
pub const GameTextInputActionType_IME_ACTION_UNSPECIFIED: GameTextInputActionType = 0;
#[doc = " Bits of {@link #IME_MASK_ACTION}: there is no available action."]
pub const GameTextInputActionType_IME_ACTION_NONE: GameTextInputActionType = 1;
#[doc = " Bits of {@link #IME_MASK_ACTION}: the action key performs a \"go\"\n operation to take the user to the target of the text they typed.\n Typically used, for example, when entering a URL."]
pub const GameTextInputActionType_IME_ACTION_GO: GameTextInputActionType = 2;
#[doc = " Bits of {@link #IME_MASK_ACTION}: the action key performs a \"search\"\n operation, taking the user to the results of searching for the text\n they have typed (in whatever context is appropriate)."]
pub const GameTextInputActionType_IME_ACTION_SEARCH: GameTextInputActionType = 3;
#[doc = " Bits of {@link #IME_MASK_ACTION}: the action key performs a \"send\"\n operation, delivering the text to its target.  This is typically used\n when composing a message in IM or SMS where sending is immediate."]
pub const GameTextInputActionType_IME_ACTION_SEND: GameTextInputActionType = 4;
#[doc = " Bits of {@link #IME_MASK_ACTION}: the action key performs a \"next\"\n operation, taking the user to the next field that will accept text."]
pub const GameTextInputActionType_IME_ACTION_NEXT: GameTextInputActionType = 5;
#[doc = " Bits of {@link #IME_MASK_ACTION}: the action key performs a \"done\"\n operation, typically meaning there is nothing more to input and the\n IME will be closed."]
pub const GameTextInputActionType_IME_ACTION_DONE: GameTextInputActionType = 6;
#[doc = " Bits of {@link #IME_MASK_ACTION}: like {@link #IME_ACTION_NEXT}, but\n for moving to the previous field.  This will normally not be used to\n specify an action (since it precludes {@link #IME_ACTION_NEXT}), but\n can be returned to the app if it sets {@link #IME_FLAG_NAVIGATE_PREVIOUS}."]
pub const GameTextInputActionType_IME_ACTION_PREVIOUS: GameTextInputActionType = 7;
#[doc = " actionId and imeOptions argument of GameActivity_setImeEditorInfo().\n\n <pre>\n |-------|-------|-------|-------|\n                              1111 IME_MASK_ACTION\n |-------|-------|-------|-------|\n                                   IME_ACTION_UNSPECIFIED\n                                 1 IME_ACTION_NONE\n                                1  IME_ACTION_GO\n                                11 IME_ACTION_SEARCH\n                               1   IME_ACTION_SEND\n                               1 1 IME_ACTION_NEXT\n                               11  IME_ACTION_DONE\n                               111 IME_ACTION_PREVIOUS\n         1                         IME_FLAG_NO_PERSONALIZED_LEARNING\n        1                          IME_FLAG_NO_FULLSCREEN\n       1                           IME_FLAG_NAVIGATE_PREVIOUS\n      1                            IME_FLAG_NAVIGATE_NEXT\n     1                             IME_FLAG_NO_EXTRACT_UI\n    1                              IME_FLAG_NO_ACCESSORY_ACTION\n   1                               IME_FLAG_NO_ENTER_ACTION\n  1                                IME_FLAG_FORCE_ASCII\n |-------|-------|-------|-------|</pre>"]
pub type GameTextInputActionType = u32;
#[doc = " Flag of {@link #imeOptions}: used to request that the IME should not update\n any personalized data such as typing history and personalized language\n model based on what the user typed on this text editing object.  Typical\n use cases are: <ul> <li>When the application is in a special mode, where\n user's activities are expected to be not recorded in the application's\n history.  Some web browsers and chat applications may have this kind of\n modes.</li> <li>When storing typing history does not make much sense.\n Specifying this flag in typing games may help to avoid typing history from\n being filled up with words that the user is less likely to type in their\n daily life.  Another example is that when the application already knows\n that the expected input is not a valid word (e.g. a promotion code that is\n     not a valid word in any natural language).</li>\n </ul>\n\n <p>Applications need to be aware that the flag is not a guarantee, and some\n IMEs may not respect it.</p>"]
pub const GameTextInputImeOptions_IME_FLAG_NO_PERSONALIZED_LEARNING: GameTextInputImeOptions =
    16777216;
#[doc = " Flag of {@link #imeOptions}: used to request that the IME never go\n into fullscreen mode.\n By default, IMEs may go into full screen mode when they think\n it's appropriate, for example on small screens in landscape\n orientation where displaying a software keyboard may occlude\n such a large portion of the screen that the remaining part is\n too small to meaningfully display the application UI.\n If this flag is set, compliant IMEs will never go into full screen mode,\n and always leave some space to display the application UI.\n Applications need to be aware that the flag is not a guarantee, and\n some IMEs may ignore it."]
pub const GameTextInputImeOptions_IME_FLAG_NO_FULLSCREEN: GameTextInputImeOptions = 33554432;
#[doc = " Flag of {@link #imeOptions}: like {@link #IME_FLAG_NAVIGATE_NEXT}, but\n specifies there is something interesting that a backward navigation\n can focus on.  If the user selects the IME's facility to backward\n navigate, this will show up in the application as an {@link\n #IME_ACTION_PREVIOUS} at {@link InputConnection#performEditorAction(int)\n InputConnection.performEditorAction(int)}."]
pub const GameTextInputImeOptions_IME_FLAG_NAVIGATE_PREVIOUS: GameTextInputImeOptions = 67108864;
#[doc = " Flag of {@link #imeOptions}: used to specify that there is something\n interesting that a forward navigation can focus on. This is like using\n {@link #IME_ACTION_NEXT}, except allows the IME to be multiline (with\n an enter key) as well as provide forward navigation.  Note that some\n IMEs may not be able to do this, especially when running on a small\n screen where there is little space.  In that case it does not need to\n present a UI for this option.  Like {@link #IME_ACTION_NEXT}, if the\n user selects the IME's facility to forward navigate, this will show up\n in the application at {@link InputConnection#performEditorAction(int)\n InputConnection.performEditorAction(int)}."]
pub const GameTextInputImeOptions_IME_FLAG_NAVIGATE_NEXT: GameTextInputImeOptions = 134217728;
#[doc = " Flag of {@link #imeOptions}: used to specify that the IME does not need\n to show its extracted text UI.  For input methods that may be fullscreen,\n often when in landscape mode, this allows them to be smaller and let part\n of the application be shown behind, through transparent UI parts in the\n fullscreen IME. The part of the UI visible to the user may not be\n responsive to touch because the IME will receive touch events, which may\n confuse the user; use {@link #IME_FLAG_NO_FULLSCREEN} instead for a better\n experience. Using this flag is discouraged and it may become deprecated in\n the future. Its meaning is unclear in some situations and it may not work\n appropriately on older versions of the platform."]
pub const GameTextInputImeOptions_IME_FLAG_NO_EXTRACT_UI: GameTextInputImeOptions = 268435456;
#[doc = " Flag of {@link #imeOptions}: used in conjunction with one of the actions\n masked by {@link #IME_MASK_ACTION}, this indicates that the action\n should not be available as an accessory button on the right of the\n extracted text when the input method is full-screen. Note that by setting\n this flag, there can be cases where the action is simply never available to\n the user. Setting this generally means that you think that in fullscreen\n mode, where there is little space to show the text, it's not worth taking\n some screen real estate to display the action and it should be used instead\n to show more text."]
pub const GameTextInputImeOptions_IME_FLAG_NO_ACCESSORY_ACTION: GameTextInputImeOptions = 536870912;
#[doc = " Flag of {@link #imeOptions}: used in conjunction with one of the actions\n masked by {@link #IME_MASK_ACTION}. If this flag is not set, IMEs will\n normally replace the \"enter\" key with the action supplied. This flag\n indicates that the action should not be available in-line as a replacement\n for the \"enter\" key. Typically this is because the action has such a\n significant impact or is not recoverable enough that accidentally hitting\n it should be avoided, such as sending a message. Note that\n {@link android.widget.TextView} will automatically set this flag for you\n on multi-line text views."]
pub const GameTextInputImeOptions_IME_FLAG_NO_ENTER_ACTION: GameTextInputImeOptions = 1073741824;
#[doc = " Flag of {@link #imeOptions}: used to request an IME that is capable of\n inputting ASCII characters.  The intention of this flag is to ensure that\n the user can type Roman alphabet characters in a {@link\n android.widget.TextView}. It is typically used for an account ID or\n password input. A lot of the time, IMEs are already able to input ASCII\n even without being told so (such IMEs already respect this flag in a\n sense), but there are cases when this is not the default. For instance,\n users of languages using a different script like Arabic, Greek, Hebrew or\n Russian typically have a keyboard that can't input ASCII characters by\n default. Applications need to be aware that the flag is not a guarantee,\n and some IMEs may not respect it. However, it is strongly recommended for\n IME authors to respect this flag especially when their IME could end up\n with a state where only languages using non-ASCII are enabled."]
pub const GameTextInputImeOptions_IME_FLAG_FORCE_ASCII: GameTextInputImeOptions = 2147483648;
#[doc = " Flag of {@link #internalImeOptions}: flag is set when app window containing\n this\n {@link EditorInfo} is using {@link Configuration#ORIENTATION_PORTRAIT}\n mode.\n @hide"]
pub const GameTextInputImeOptions_IME_INTERNAL_FLAG_APP_WINDOW_PORTRAIT: GameTextInputImeOptions =
    1;
#[doc = " Generic unspecified type for {@link #imeOptions}."]
pub const GameTextInputImeOptions_IME_NULL: GameTextInputImeOptions = 0;
pub type GameTextInputImeOptions = u32;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_CAPTION_BAR: GameCommonInsetsType = 0;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_DISPLAY_CUTOUT: GameCommonInsetsType = 1;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_IME: GameCommonInsetsType = 2;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_MANDATORY_SYSTEM_GESTURES:
    GameCommonInsetsType = 3;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_NAVIGATION_BARS: GameCommonInsetsType = 4;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_STATUS_BARS: GameCommonInsetsType = 5;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_SYSTEM_BARS: GameCommonInsetsType = 6;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_SYSTEM_GESTURES: GameCommonInsetsType = 7;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_TAPABLE_ELEMENT: GameCommonInsetsType = 8;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_WATERFALL: GameCommonInsetsType = 9;
pub const GameCommonInsetsType_GAMECOMMON_INSETS_TYPE_COUNT: GameCommonInsetsType = 10;
#[doc = " The type of a component for which to retrieve insets. See\n https://developer.android.com/reference/androidx/core/view/WindowInsetsCompat.Type"]
pub type GameCommonInsetsType = u8;
#[doc = " This structure defines the native side of an android.app.GameActivity.\n It is created by the framework, and handed to the application's native\n code as it is being launched."]
#[repr(C)]
pub struct GameActivity {
    #[doc = " Pointer to the callback function table of the native application.\n You can set the functions here to your own callbacks.  The callbacks\n pointer itself here should not be changed; it is allocated and managed\n for you by the framework."]
    pub callbacks: *mut GameActivityCallbacks,
    #[doc = " The global handle on the process's Java VM."]
    pub vm: *mut JavaVM,
    #[doc = " JNI context for the main thread of the app.  Note that this field\n can ONLY be used from the main thread of the process; that is, the\n thread that calls into the GameActivityCallbacks."]
    pub env: *mut JNIEnv,
    #[doc = " The GameActivity object handle."]
    pub javaGameActivity: jobject,
    #[doc = " Path to this application's internal data directory."]
    pub internalDataPath: *const ::std::os::raw::c_char,
    #[doc = " Path to this application's external (removable/mountable) data directory."]
    pub externalDataPath: *const ::std::os::raw::c_char,
    #[doc = " The platform's SDK version code."]
    pub sdkVersion: i32,
    #[doc = " This is the native instance of the application.  It is not used by\n the framework, but can be set by the application to its own instance\n state."]
    pub instance: *mut ::std::os::raw::c_void,
    #[doc = " Pointer to the Asset Manager instance for the application.  The\n application uses this to access binary assets bundled inside its own .apk\n file."]
    pub assetManager: *mut AAssetManager,
    #[doc = " Available starting with Honeycomb: path to the directory containing\n the application's OBB files (if any).  If the app doesn't have any\n OBB files, this directory may not exist."]
    pub obbPath: *const ::std::os::raw::c_char,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameActivity"][::std::mem::size_of::<GameActivity>() - 40usize];
    ["Alignment of GameActivity"][::std::mem::align_of::<GameActivity>() - 4usize];
    ["Offset of field: GameActivity::callbacks"]
        [::std::mem::offset_of!(GameActivity, callbacks) - 0usize];
    ["Offset of field: GameActivity::vm"][::std::mem::offset_of!(GameActivity, vm) - 4usize];
    ["Offset of field: GameActivity::env"][::std::mem::offset_of!(GameActivity, env) - 8usize];
    ["Offset of field: GameActivity::javaGameActivity"]
        [::std::mem::offset_of!(GameActivity, javaGameActivity) - 12usize];
    ["Offset of field: GameActivity::internalDataPath"]
        [::std::mem::offset_of!(GameActivity, internalDataPath) - 16usize];
    ["Offset of field: GameActivity::externalDataPath"]
        [::std::mem::offset_of!(GameActivity, externalDataPath) - 20usize];
    ["Offset of field: GameActivity::sdkVersion"]
        [::std::mem::offset_of!(GameActivity, sdkVersion) - 24usize];
    ["Offset of field: GameActivity::instance"]
        [::std::mem::offset_of!(GameActivity, instance) - 28usize];
    ["Offset of field: GameActivity::assetManager"]
        [::std::mem::offset_of!(GameActivity, assetManager) - 32usize];
    ["Offset of field: GameActivity::obbPath"]
        [::std::mem::offset_of!(GameActivity, obbPath) - 36usize];
};
#[doc = " A function the user should call from their callback with the data, its length\n and the library- supplied context."]
pub type SaveInstanceStateRecallback = ::std::option::Option<
    unsafe extern "C" fn(
        bytes: *const ::std::os::raw::c_char,
        len: ::std::os::raw::c_int,
        context: *mut ::std::os::raw::c_void,
    ),
>;
#[doc = " {@link GameActivityCallbacks}"]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct GameActivityCallbacks {
    #[doc = " GameActivity has started.  See Java documentation for Activity.onStart()\n for more information."]
    pub onStart: ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " GameActivity has resumed.  See Java documentation for Activity.onResume()\n for more information."]
    pub onResume: ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " The framework is asking GameActivity to save its current instance state.\n See the Java documentation for Activity.onSaveInstanceState() for more\n information. The user should call the recallback with their data, its\n length and the provided context; they retain ownership of the data. Note\n that the saved state will be persisted, so it can not contain any active\n entities (pointers to memory, file descriptors, etc)."]
    pub onSaveInstanceState: ::std::option::Option<
        unsafe extern "C" fn(
            activity: *mut GameActivity,
            recallback: SaveInstanceStateRecallback,
            context: *mut ::std::os::raw::c_void,
        ),
    >,
    #[doc = " GameActivity has paused.  See Java documentation for Activity.onPause()\n for more information."]
    pub onPause: ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " GameActivity has stopped.  See Java documentation for Activity.onStop()\n for more information."]
    pub onStop: ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " GameActivity is being destroyed.  See Java documentation for\n Activity.onDestroy() for more information."]
    pub onDestroy: ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " Focus has changed in this GameActivity's window.  This is often used,\n for example, to pause a game when it loses input focus."]
    pub onWindowFocusChanged:
        ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity, hasFocus: bool)>,
    #[doc = " The drawing window for this native activity has been created.  You\n can use the given native window object to start drawing."]
    pub onNativeWindowCreated: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, window: *mut ANativeWindow),
    >,
    #[doc = " The drawing window for this native activity has been resized.  You should\n retrieve the new size from the window and ensure that your rendering in\n it now matches."]
    pub onNativeWindowResized: ::std::option::Option<
        unsafe extern "C" fn(
            activity: *mut GameActivity,
            window: *mut ANativeWindow,
            newWidth: i32,
            newHeight: i32,
        ),
    >,
    #[doc = " The drawing window for this native activity needs to be redrawn.  To\n avoid transient artifacts during screen changes (such resizing after\n rotation), applications should not return from this function until they\n have finished drawing their window in its current state."]
    pub onNativeWindowRedrawNeeded: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, window: *mut ANativeWindow),
    >,
    #[doc = " The drawing window for this native activity is going to be destroyed.\n You MUST ensure that you do not touch the window object after returning\n from this function: in the common case of drawing to the window from\n another thread, that means the implementation of this callback must\n properly synchronize with the other thread to stop its drawing before\n returning from here."]
    pub onNativeWindowDestroyed: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, window: *mut ANativeWindow),
    >,
    #[doc = " The current device AConfiguration has changed. The new configuration can\n be retrieved from assetManager."]
    pub onConfigurationChanged:
        ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " The system is running low on memory.  Use this callback to release\n resources you do not need, to help the system avoid killing more\n important processes."]
    pub onTrimMemory: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, level: ::std::os::raw::c_int),
    >,
    #[doc = " Callback called for every MotionEvent done on the GameActivity\n SurfaceView. Ownership of `event` is maintained by the library and it is\n only valid during the callback."]
    pub onTouchEvent: ::std::option::Option<
        unsafe extern "C" fn(
            activity: *mut GameActivity,
            event: *const GameActivityMotionEvent,
        ) -> bool,
    >,
    #[doc = " Callback called for every key down event on the GameActivity SurfaceView.\n Ownership of `event` is maintained by the library and it is only valid\n during the callback."]
    pub onKeyDown: ::std::option::Option<
        unsafe extern "C" fn(
            activity: *mut GameActivity,
            event: *const GameActivityKeyEvent,
        ) -> bool,
    >,
    #[doc = " Callback called for every key up event on the GameActivity SurfaceView.\n Ownership of `event` is maintained by the library and it is only valid\n during the callback."]
    pub onKeyUp: ::std::option::Option<
        unsafe extern "C" fn(
            activity: *mut GameActivity,
            event: *const GameActivityKeyEvent,
        ) -> bool,
    >,
    #[doc = " Callback called for every soft-keyboard text input event.\n Ownership of `state` is maintained by the library and it is only valid\n during the callback."]
    pub onTextInputEvent: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, state: *const GameTextInputState),
    >,
    #[doc = " Callback called when WindowInsets of the main app window have changed.\n Call GameActivity_getWindowInsets to retrieve the insets themselves."]
    pub onWindowInsetsChanged:
        ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity)>,
    #[doc = " Callback called when the rectangle in the window where the content\n should be placed has changed."]
    pub onContentRectChanged: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, rect: *const ARect),
    >,
    #[doc = " Callback called when the software keyboard is shown or hidden."]
    pub onSoftwareKeyboardVisibilityChanged:
        ::std::option::Option<unsafe extern "C" fn(activity: *mut GameActivity, visible: bool)>,
    #[doc = " Callback called when the software keyboard is shown or hidden."]
    pub onEditorAction: ::std::option::Option<
        unsafe extern "C" fn(activity: *mut GameActivity, action: ::std::os::raw::c_int) -> bool,
    >,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of GameActivityCallbacks"][::std::mem::size_of::<GameActivityCallbacks>() - 84usize];
    ["Alignment of GameActivityCallbacks"]
        [::std::mem::align_of::<GameActivityCallbacks>() - 4usize];
    ["Offset of field: GameActivityCallbacks::onStart"]
        [::std::mem::offset_of!(GameActivityCallbacks, onStart) - 0usize];
    ["Offset of field: GameActivityCallbacks::onResume"]
        [::std::mem::offset_of!(GameActivityCallbacks, onResume) - 4usize];
    ["Offset of field: GameActivityCallbacks::onSaveInstanceState"]
        [::std::mem::offset_of!(GameActivityCallbacks, onSaveInstanceState) - 8usize];
    ["Offset of field: GameActivityCallbacks::onPause"]
        [::std::mem::offset_of!(GameActivityCallbacks, onPause) - 12usize];
    ["Offset of field: GameActivityCallbacks::onStop"]
        [::std::mem::offset_of!(GameActivityCallbacks, onStop) - 16usize];
    ["Offset of field: GameActivityCallbacks::onDestroy"]
        [::std::mem::offset_of!(GameActivityCallbacks, onDestroy) - 20usize];
    ["Offset of field: GameActivityCallbacks::onWindowFocusChanged"]
        [::std::mem::offset_of!(GameActivityCallbacks, onWindowFocusChanged) - 24usize];
    ["Offset of field: GameActivityCallbacks::onNativeWindowCreated"]
        [::std::mem::offset_of!(GameActivityCallbacks, onNativeWindowCreated) - 28usize];
    ["Offset of field: GameActivityCallbacks::onNativeWindowResized"]
        [::std::mem::offset_of!(GameActivityCallbacks, onNativeWindowResized) - 32usize];
    ["Offset of field: GameActivityCallbacks::onNativeWindowRedrawNeeded"]
        [::std::mem::offset_of!(GameActivityCallbacks, onNativeWindowRedrawNeeded) - 36usize];
    ["Offset of field: GameActivityCallbacks::onNativeWindowDestroyed"]
        [::std::mem::offset_of!(GameActivityCallbacks, onNativeWindowDestroyed) - 40usize];
    ["Offset of field: GameActivityCallbacks::onConfigurationChanged"]
        [::std::mem::offset_of!(GameActivityCallbacks, onConfigurationChanged) - 44usize];
    ["Offset of field: GameActivityCallbacks::onTrimMemory"]
        [::std::mem::offset_of!(GameActivityCallbacks, onTrimMemory) - 48usize];
    ["Offset of field: GameActivityCallbacks::onTouchEvent"]
        [::std::mem::offset_of!(GameActivityCallbacks, onTouchEvent) - 52usize];
    ["Offset of field: GameActivityCallbacks::onKeyDown"]
        [::std::mem::offset_of!(GameActivityCallbacks, onKeyDown) - 56usize];
    ["Offset of field: GameActivityCallbacks::onKeyUp"]
        [::std::mem::offset_of!(GameActivityCallbacks, onKeyUp) - 60usize];
    ["Offset of field: GameActivityCallbacks::onTextInputEvent"]
        [::std::mem::offset_of!(GameActivityCallbacks, onTextInputEvent) - 64usize];
    ["Offset of field: GameActivityCallbacks::onWindowInsetsChanged"]
        [::std::mem::offset_of!(GameActivityCallbacks, onWindowInsetsChanged) - 68usize];
    ["Offset of field: GameActivityCallbacks::onContentRectChanged"]
        [::std::mem::offset_of!(GameActivityCallbacks, onContentRectChanged) - 72usize];
    ["Offset of field: GameActivityCallbacks::onSoftwareKeyboardVisibilityChanged"][::std::mem::offset_of!(
        GameActivityCallbacks,
        onSoftwareKeyboardVisibilityChanged
    ) - 76usize];
    ["Offset of field: GameActivityCallbacks::onEditorAction"]
        [::std::mem::offset_of!(GameActivityCallbacks, onEditorAction) - 80usize];
};
#[doc = " This is the function that must be in the native code to instantiate the\n application's native activity.  It is called with the activity instance (see\n above); if the code is being instantiated from a previously saved instance,\n the savedState will be non-NULL and point to the saved data.  You must make\n any copy of this data you need -- it will be released after you return from\n this function."]
pub type GameActivity_createFunc = ::std::option::Option<
    unsafe extern "C" fn(
        activity: *mut GameActivity,
        savedState: *mut ::std::os::raw::c_void,
        savedStateSize: usize,
    ),
>;
unsafe extern "C" {
    #[doc = " Finish the given activity.  Its finish() method will be called, causing it\n to be stopped and destroyed.  Note that this method can be called from\n *any* thread; it will send a message to the main thread of the process\n where the Java finish call will take place."]
    pub fn GameActivity_finish(activity: *mut GameActivity);
}
#[doc = " As long as this window is visible to the user, allow the lock\n screen to activate while the screen is on.  This can be used\n independently, or in combination with {@link\n GAMEACTIVITY_FLAG_KEEP_SCREEN_ON} and/or {@link\n GAMEACTIVITY_FLAG_SHOW_WHEN_LOCKED}"]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_ALLOW_LOCK_WHILE_SCREEN_ON:
    GameActivitySetWindowFlags = 1;
#[doc = " Everything behind this window will be dimmed."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_DIM_BEHIND: GameActivitySetWindowFlags = 2;
#[doc = " Blur everything behind this window.\n @deprecated Blurring is no longer supported."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_BLUR_BEHIND: GameActivitySetWindowFlags = 4;
#[doc = " This window won't ever get key input focus, so the\n user can not send key or other button events to it.  Those will\n instead go to whatever focusable window is behind it.  This flag\n will also enable {@link GAMEACTIVITY_FLAG_NOT_TOUCH_MODAL} whether or not\n that is explicitly set.\n\n Setting this flag also implies that the window will not need to\n interact with\n a soft input method, so it will be Z-ordered and positioned\n independently of any active input method (typically this means it\n gets Z-ordered on top of the input method, so it can use the full\n screen for its content and cover the input method if needed.  You\n can use {@link GAMEACTIVITY_FLAG_ALT_FOCUSABLE_IM} to modify this\n behavior."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_NOT_FOCUSABLE: GameActivitySetWindowFlags =
    8;
#[doc = " This window can never receive touch events."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_NOT_TOUCHABLE: GameActivitySetWindowFlags =
    16;
#[doc = " Even when this window is focusable (its\n {@link GAMEACTIVITY_FLAG_NOT_FOCUSABLE} is not set), allow any pointer\n events outside of the window to be sent to the windows behind it.\n Otherwise it will consume all pointer events itself, regardless of\n whether they are inside of the window."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_NOT_TOUCH_MODAL: GameActivitySetWindowFlags =
    32;
#[doc = " When set, if the device is asleep when the touch\n screen is pressed, you will receive this first touch event.  Usually\n the first touch event is consumed by the system since the user can\n not see what they are pressing on.\n\n @deprecated This flag has no effect."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_TOUCHABLE_WHEN_WAKING:
    GameActivitySetWindowFlags = 64;
#[doc = " As long as this window is visible to the user, keep\n the device's screen turned on and bright."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_KEEP_SCREEN_ON: GameActivitySetWindowFlags =
    128;
#[doc = " Place the window within the entire screen, ignoring\n decorations around the border (such as the status bar).  The\n window must correctly position its contents to take the screen\n decoration into account."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_LAYOUT_IN_SCREEN:
    GameActivitySetWindowFlags = 256;
#[doc = " Allows the window to extend outside of the screen."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_LAYOUT_NO_LIMITS:
    GameActivitySetWindowFlags = 512;
#[doc = " Hide all screen decorations (such as the status\n bar) while this window is displayed.  This allows the window to\n use the entire display space for itself -- the status bar will\n be hidden when an app window with this flag set is on the top\n layer. A fullscreen window will ignore a value of {@link\n GAMEACTIVITY_SOFT_INPUT_ADJUST_RESIZE}; the window will stay\n fullscreen and will not resize."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_FULLSCREEN: GameActivitySetWindowFlags =
    1024;
#[doc = " Override {@link GAMEACTIVITY_FLAG_FULLSCREEN} and force the\n screen decorations (such as the status bar) to be shown."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_FORCE_NOT_FULLSCREEN:
    GameActivitySetWindowFlags = 2048;
#[doc = " Turn on dithering when compositing this window to\n the screen.\n @deprecated This flag is no longer used."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_DITHER: GameActivitySetWindowFlags = 4096;
#[doc = " Treat the content of the window as secure, preventing\n it from appearing in screenshots or from being viewed on non-secure\n displays."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_SECURE: GameActivitySetWindowFlags = 8192;
#[doc = " A special mode where the layout parameters are used\n to perform scaling of the surface when it is composited to the\n screen."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_SCALED: GameActivitySetWindowFlags = 16384;
#[doc = " Intended for windows that will often be used when the user is\n holding the screen against their face, it will aggressively\n filter the event stream to prevent unintended presses in this\n situation that may not be desired for a particular window, when\n such an event stream is detected, the application will receive\n a {@link AMOTION_EVENT_ACTION_CANCEL} to indicate this so\n applications can handle this accordingly by taking no action on\n the event until the finger is released."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_IGNORE_CHEEK_PRESSES:
    GameActivitySetWindowFlags = 32768;
#[doc = " A special option only for use in combination with\n {@link GAMEACTIVITY_FLAG_LAYOUT_IN_SCREEN}.  When requesting layout in\n the screen your window may appear on top of or behind screen decorations\n such as the status bar.  By also including this flag, the window\n manager will report the inset rectangle needed to ensure your\n content is not covered by screen decorations."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_LAYOUT_INSET_DECOR:
    GameActivitySetWindowFlags = 65536;
#[doc = " Invert the state of {@link GAMEACTIVITY_FLAG_NOT_FOCUSABLE} with\n respect to how this window interacts with the current method.\n That is, if FLAG_NOT_FOCUSABLE is set and this flag is set,\n then the window will behave as if it needs to interact with the\n input method and thus be placed behind/away from it; if {@link\n GAMEACTIVITY_FLAG_NOT_FOCUSABLE} is not set and this flag is set,\n then the window will behave as if it doesn't need to interact\n with the input method and can be placed to use more space and\n cover the input method."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_ALT_FOCUSABLE_IM:
    GameActivitySetWindowFlags = 131072;
#[doc = " If you have set {@link GAMEACTIVITY_FLAG_NOT_TOUCH_MODAL}, you\n can set this flag to receive a single special MotionEvent with\n the action\n {@link AMOTION_EVENT_ACTION_OUTSIDE} for\n touches that occur outside of your window.  Note that you will not\n receive the full down/move/up gesture, only the location of the\n first down as an {@link AMOTION_EVENT_ACTION_OUTSIDE}."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_WATCH_OUTSIDE_TOUCH:
    GameActivitySetWindowFlags = 262144;
#[doc = " Special flag to let windows be shown when the screen\n is locked. This will let application windows take precedence over\n key guard or any other lock screens. Can be used with\n {@link GAMEACTIVITY_FLAG_KEEP_SCREEN_ON} to turn screen on and display\n windows directly before showing the key guard window.  Can be used with\n {@link GAMEACTIVITY_FLAG_DISMISS_KEYGUARD} to automatically fully\n dismisss non-secure keyguards.  This flag only applies to the top-most\n full-screen window."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_SHOW_WHEN_LOCKED:
    GameActivitySetWindowFlags = 524288;
#[doc = " Ask that the system wallpaper be shown behind\n your window.  The window surface must be translucent to be able\n to actually see the wallpaper behind it; this flag just ensures\n that the wallpaper surface will be there if this window actually\n has translucent regions."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_SHOW_WALLPAPER: GameActivitySetWindowFlags =
    1048576;
#[doc = " When set as a window is being added or made\n visible, once the window has been shown then the system will\n poke the power manager's user activity (as if the user had woken\n up the device) to turn the screen on."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_TURN_SCREEN_ON: GameActivitySetWindowFlags =
    2097152;
#[doc = " When set the window will cause the keyguard to\n be dismissed, only if it is not a secure lock keyguard.  Because such\n a keyguard is not needed for security, it will never re-appear if\n the user navigates to another window (in contrast to\n {@link GAMEACTIVITY_FLAG_SHOW_WHEN_LOCKED}, which will only temporarily\n hide both secure and non-secure keyguards but ensure they reappear\n when the user moves to another UI that doesn't hide them).\n If the keyguard is currently active and is secure (requires an\n unlock pattern) than the user will still need to confirm it before\n seeing this window, unless {@link GAMEACTIVITY_FLAG_SHOW_WHEN_LOCKED} has\n also been set."]
pub const GameActivitySetWindowFlags_GAMEACTIVITY_FLAG_DISMISS_KEYGUARD:
    GameActivitySetWindowFlags = 4194304;
#[doc = " Flags for GameActivity_setWindowFlags,\n as per the Java API at android.view.WindowManager.LayoutParams."]
pub type GameActivitySetWindowFlags = u32;
unsafe extern "C" {
    #[doc = " Change the window flags of the given activity.  Calls getWindow().setFlags()\n of the given activity.\n Note that some flags must be set before the window decoration is created,\n see\n https://developer.android.com/reference/android/view/Window#setFlags(int,%20int).\n Note also that this method can be called from\n *any* thread; it will send a message to the main thread of the process\n where the Java finish call will take place."]
    pub fn GameActivity_setWindowFlags(
        activity: *mut GameActivity,
        addFlags: u32,
        removeFlags: u32,
    );
}
#[doc = " Implicit request to show the input window, not as the result\n of a direct request by the user."]
pub const GameActivityShowSoftInputFlags_GAMEACTIVITY_SHOW_SOFT_INPUT_IMPLICIT:
    GameActivityShowSoftInputFlags = 1;
#[doc = " The user has forced the input method open (such as by\n long-pressing menu) so it should not be closed until they\n explicitly do so."]
pub const GameActivityShowSoftInputFlags_GAMEACTIVITY_SHOW_SOFT_INPUT_FORCED:
    GameActivityShowSoftInputFlags = 2;
#[doc = " Flags for GameActivity_showSoftInput; see the Java InputMethodManager\n API for documentation."]
pub type GameActivityShowSoftInputFlags = u8;
unsafe extern "C" {
    #[doc = " Show the IME while in the given activity.  Calls\n InputMethodManager.showSoftInput() for the given activity.  Note that this\n method can be called from *any* thread; it will send a message to the main\n thread of the process where the Java call will take place."]
    pub fn GameActivity_showSoftInput(activity: *mut GameActivity, flags: u32);
}
unsafe extern "C" {
    #[doc = " Restarts the input method. Calls InputMethodManager.restartInput().\n Note that this method can be called from *any* thread; it will send a message\n to the main thread of the process where the Java call will take place."]
    pub fn GameActivity_restartInput(activity: *mut GameActivity);
}
unsafe extern "C" {
    #[doc = " Set the text entry state (see documentation of the GameTextInputState struct\n in the Game Text Input library reference).\n\n Ownership of the state is maintained by the caller."]
    pub fn GameActivity_setTextInputState(
        activity: *mut GameActivity,
        state: *const GameTextInputState,
    );
}
unsafe extern "C" {
    #[doc = " Get the last-received text entry state (see documentation of the\n GameTextInputState struct in the Game Text Input library reference).\n"]
    pub fn GameActivity_getTextInputState(
        activity: *mut GameActivity,
        callback: GameTextInputGetStateCallback,
        context: *mut ::std::os::raw::c_void,
    );
}
unsafe extern "C" {
    #[doc = " Get a pointer to the GameTextInput library instance."]
    pub fn GameActivity_getTextInput(activity: *const GameActivity) -> *mut GameTextInput;
}
#[doc = " The soft input window should only be hidden if it was not\n explicitly shown by the user."]
pub const GameActivityHideSoftInputFlags_GAMEACTIVITY_HIDE_SOFT_INPUT_IMPLICIT_ONLY:
    GameActivityHideSoftInputFlags = 1;
#[doc = " The soft input window should normally be hidden, unless it was\n originally shown with {@link GAMEACTIVITY_SHOW_SOFT_INPUT_FORCED}."]
pub const GameActivityHideSoftInputFlags_GAMEACTIVITY_HIDE_SOFT_INPUT_NOT_ALWAYS:
    GameActivityHideSoftInputFlags = 2;
#[doc = " Flags for GameActivity_hideSoftInput; see the Java InputMethodManager\n API for documentation."]
pub type GameActivityHideSoftInputFlags = u16;
unsafe extern "C" {
    #[doc = " Hide the IME while in the given activity.  Calls\n InputMethodManager.hideSoftInput() for the given activity.  Note that this\n method can be called from *any* thread; it will send a message to the main\n thread of the process where the Java finish call will take place."]
    pub fn GameActivity_hideSoftInput(activity: *mut GameActivity, flags: u32);
}
unsafe extern "C" {
    #[doc = " Get the current window insets of the particular component. See\n https://developer.android.com/reference/androidx/core/view/WindowInsetsCompat.Type\n for more details.\n You can use these insets to influence what you show on the screen."]
    pub fn GameActivity_getWindowInsets(
        activity: *mut GameActivity,
        type_: GameCommonInsetsType,
        insets: *mut ARect,
    );
}
unsafe extern "C" {
    #[doc = " Tells whether the software keyboard is visible or not."]
    pub fn GameActivity_isSoftwareKeyboardVisible(activity: *mut GameActivity) -> bool;
}
unsafe extern "C" {
    #[doc = " Set options on how the IME behaves when it is requested for text input.\n See\n https://developer.android.com/reference/android/view/inputmethod/EditorInfo\n for the meaning of inputType, actionId and imeOptions.\n\n <b>Note:</b> currently only TYPE_NULL AND TYPE_CLASS_NUMBER are supported."]
    pub fn GameActivity_setImeEditorInfo(
        activity: *mut GameActivity,
        inputType: GameTextInputType,
        actionId: GameTextInputActionType,
        imeOptions: GameTextInputImeOptions,
    );
}
unsafe extern "C" {
    #[doc = " These are getters for Configuration class members. They may be called from\n any thread."]
    pub fn GameActivity_getOrientation(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getColorMode(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getDensityDpi(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getFontScale(activity: *mut GameActivity) -> f32;
}
unsafe extern "C" {
    pub fn GameActivity_getFontWeightAdjustment(
        activity: *mut GameActivity,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getHardKeyboardHidden(activity: *mut GameActivity)
        -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getKeyboard(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getKeyboardHidden(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getLocalesCount(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getMcc(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getMnc(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getNavigation(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getNavigationHidden(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getScreenHeightDp(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getScreenLayout(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getScreenWidthDp(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getSmallestScreenWidthDp(
        activity: *mut GameActivity,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getTouchscreen(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getUIMode(activity: *mut GameActivity) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    #[doc = " The functions below return Java locale information.\n\n In simple cases there will be just one locale, but it's possible tha\n there are more than one locale objects. Users are encouraged to write code\n that handles all locales and not just the first one.\n\n The functions in the block below return string values in the provided buffer.\n Return value is zero if there were no errors, otherwise it's non-zero.\n If the return value is zero, `dst` will contain a null-terminated string:\n strlen(dst) <= dst_size - 1.\n If the return value is non-zero, the content of dst is undefined.\n\n Parameters:\n\n dst, dst_size: define a receiver buffer. Locale string can be something\n short like \"EN/EN\", but it may be longer. You should be safe with a buffer\n size of 256 bytes.\n\n If the buffer is too small, ENOBUFS is returned. Try allocating a larger\n buffer in this case.\n\n localeIdx must be between 0 and the value of GameActivity_getLocalesCount().\n If localeIdx is out of range, EINVAL is returned.\n\n Refer to Java documentation of locales for more information."]
    pub fn GameActivity_getLocaleLanguage(
        dst: *mut ::std::os::raw::c_char,
        dst_size: usize,
        activity: *mut GameActivity,
        localeIdx: usize,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getLocaleScript(
        dst: *mut ::std::os::raw::c_char,
        dst_size: usize,
        activity: *mut GameActivity,
        localeIdx: usize,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getLocaleCountry(
        dst: *mut ::std::os::raw::c_char,
        dst_size: usize,
        activity: *mut GameActivity,
        localeIdx: usize,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn GameActivity_getLocaleVariant(
        dst: *mut ::std::os::raw::c_char,
        dst_size: usize,
        activity: *mut GameActivity,
        localeIdx: usize,
    ) -> ::std::os::raw::c_int;
}
pub const ACONFIGURATION_ORIENTATION_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_ORIENTATION_PORT: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_ORIENTATION_LAND: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_ORIENTATION_SQUARE: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_TOUCHSCREEN_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_TOUCHSCREEN_NOTOUCH: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_TOUCHSCREEN_STYLUS: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_TOUCHSCREEN_FINGER: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_DENSITY_DEFAULT: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_DENSITY_LOW: _bindgen_ty_21 = 120;
pub const ACONFIGURATION_DENSITY_MEDIUM: _bindgen_ty_21 = 160;
pub const ACONFIGURATION_DENSITY_TV: _bindgen_ty_21 = 213;
pub const ACONFIGURATION_DENSITY_HIGH: _bindgen_ty_21 = 240;
pub const ACONFIGURATION_DENSITY_XHIGH: _bindgen_ty_21 = 320;
pub const ACONFIGURATION_DENSITY_XXHIGH: _bindgen_ty_21 = 480;
pub const ACONFIGURATION_DENSITY_XXXHIGH: _bindgen_ty_21 = 640;
pub const ACONFIGURATION_DENSITY_ANY: _bindgen_ty_21 = 65534;
pub const ACONFIGURATION_DENSITY_NONE: _bindgen_ty_21 = 65535;
pub const ACONFIGURATION_KEYBOARD_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_KEYBOARD_NOKEYS: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_KEYBOARD_QWERTY: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_KEYBOARD_12KEY: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_NAVIGATION_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_NAVIGATION_NONAV: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_NAVIGATION_DPAD: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_NAVIGATION_TRACKBALL: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_NAVIGATION_WHEEL: _bindgen_ty_21 = 4;
pub const ACONFIGURATION_KEYSHIDDEN_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_KEYSHIDDEN_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_KEYSHIDDEN_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_KEYSHIDDEN_SOFT: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_NAVHIDDEN_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_NAVHIDDEN_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_NAVHIDDEN_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_SCREENSIZE_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_SCREENSIZE_SMALL: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_SCREENSIZE_NORMAL: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_SCREENSIZE_LARGE: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_SCREENSIZE_XLARGE: _bindgen_ty_21 = 4;
pub const ACONFIGURATION_SCREENLONG_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_SCREENLONG_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_SCREENLONG_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_SCREENROUND_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_SCREENROUND_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_SCREENROUND_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_WIDE_COLOR_GAMUT_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_WIDE_COLOR_GAMUT_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_WIDE_COLOR_GAMUT_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_HDR_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_HDR_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_HDR_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_UI_MODE_TYPE_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_UI_MODE_TYPE_NORMAL: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_UI_MODE_TYPE_DESK: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_UI_MODE_TYPE_CAR: _bindgen_ty_21 = 3;
pub const ACONFIGURATION_UI_MODE_TYPE_TELEVISION: _bindgen_ty_21 = 4;
pub const ACONFIGURATION_UI_MODE_TYPE_APPLIANCE: _bindgen_ty_21 = 5;
pub const ACONFIGURATION_UI_MODE_TYPE_WATCH: _bindgen_ty_21 = 6;
pub const ACONFIGURATION_UI_MODE_TYPE_VR_HEADSET: _bindgen_ty_21 = 7;
pub const ACONFIGURATION_UI_MODE_NIGHT_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_UI_MODE_NIGHT_NO: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_UI_MODE_NIGHT_YES: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_SCREEN_WIDTH_DP_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_SCREEN_HEIGHT_DP_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_SMALLEST_SCREEN_WIDTH_DP_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_LAYOUTDIR_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_LAYOUTDIR_LTR: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_LAYOUTDIR_RTL: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_MCC: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_MNC: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_LOCALE: _bindgen_ty_21 = 4;
pub const ACONFIGURATION_TOUCHSCREEN: _bindgen_ty_21 = 8;
pub const ACONFIGURATION_KEYBOARD: _bindgen_ty_21 = 16;
pub const ACONFIGURATION_KEYBOARD_HIDDEN: _bindgen_ty_21 = 32;
pub const ACONFIGURATION_NAVIGATION: _bindgen_ty_21 = 64;
pub const ACONFIGURATION_ORIENTATION: _bindgen_ty_21 = 128;
pub const ACONFIGURATION_DENSITY: _bindgen_ty_21 = 256;
pub const ACONFIGURATION_SCREEN_SIZE: _bindgen_ty_21 = 512;
pub const ACONFIGURATION_VERSION: _bindgen_ty_21 = 1024;
pub const ACONFIGURATION_SCREEN_LAYOUT: _bindgen_ty_21 = 2048;
pub const ACONFIGURATION_UI_MODE: _bindgen_ty_21 = 4096;
pub const ACONFIGURATION_SMALLEST_SCREEN_SIZE: _bindgen_ty_21 = 8192;
pub const ACONFIGURATION_LAYOUTDIR: _bindgen_ty_21 = 16384;
pub const ACONFIGURATION_SCREEN_ROUND: _bindgen_ty_21 = 32768;
pub const ACONFIGURATION_COLOR_MODE: _bindgen_ty_21 = 65536;
pub const ACONFIGURATION_GRAMMATICAL_GENDER: _bindgen_ty_21 = 131072;
pub const ACONFIGURATION_MNC_ZERO: _bindgen_ty_21 = 65535;
pub const ACONFIGURATION_GRAMMATICAL_GENDER_ANY: _bindgen_ty_21 = 0;
pub const ACONFIGURATION_GRAMMATICAL_GENDER_NEUTER: _bindgen_ty_21 = 1;
pub const ACONFIGURATION_GRAMMATICAL_GENDER_FEMININE: _bindgen_ty_21 = 2;
pub const ACONFIGURATION_GRAMMATICAL_GENDER_MASCULINE: _bindgen_ty_21 = 3;
pub type _bindgen_ty_21 = ::std::os::raw::c_uint;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct pollfd {
    pub fd: ::std::os::raw::c_int,
    pub events: ::std::os::raw::c_short,
    pub revents: ::std::os::raw::c_short,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of pollfd"][::std::mem::size_of::<pollfd>() - 8usize];
    ["Alignment of pollfd"][::std::mem::align_of::<pollfd>() - 4usize];
    ["Offset of field: pollfd::fd"][::std::mem::offset_of!(pollfd, fd) - 0usize];
    ["Offset of field: pollfd::events"][::std::mem::offset_of!(pollfd, events) - 4usize];
    ["Offset of field: pollfd::revents"][::std::mem::offset_of!(pollfd, revents) - 6usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _fpx_sw_bytes {
    pub magic1: __u32,
    pub extended_size: __u32,
    pub xfeatures: __u64,
    pub xstate_size: __u32,
    pub padding: [__u32; 7usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpx_sw_bytes"][::std::mem::size_of::<_fpx_sw_bytes>() - 48usize];
    ["Alignment of _fpx_sw_bytes"][::std::mem::align_of::<_fpx_sw_bytes>() - 4usize];
    ["Offset of field: _fpx_sw_bytes::magic1"]
        [::std::mem::offset_of!(_fpx_sw_bytes, magic1) - 0usize];
    ["Offset of field: _fpx_sw_bytes::extended_size"]
        [::std::mem::offset_of!(_fpx_sw_bytes, extended_size) - 4usize];
    ["Offset of field: _fpx_sw_bytes::xfeatures"]
        [::std::mem::offset_of!(_fpx_sw_bytes, xfeatures) - 8usize];
    ["Offset of field: _fpx_sw_bytes::xstate_size"]
        [::std::mem::offset_of!(_fpx_sw_bytes, xstate_size) - 16usize];
    ["Offset of field: _fpx_sw_bytes::padding"]
        [::std::mem::offset_of!(_fpx_sw_bytes, padding) - 20usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _fpreg {
    pub significand: [__u16; 4usize],
    pub exponent: __u16,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpreg"][::std::mem::size_of::<_fpreg>() - 10usize];
    ["Alignment of _fpreg"][::std::mem::align_of::<_fpreg>() - 2usize];
    ["Offset of field: _fpreg::significand"][::std::mem::offset_of!(_fpreg, significand) - 0usize];
    ["Offset of field: _fpreg::exponent"][::std::mem::offset_of!(_fpreg, exponent) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _fpxreg {
    pub significand: [__u16; 4usize],
    pub exponent: __u16,
    pub padding: [__u16; 3usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpxreg"][::std::mem::size_of::<_fpxreg>() - 16usize];
    ["Alignment of _fpxreg"][::std::mem::align_of::<_fpxreg>() - 2usize];
    ["Offset of field: _fpxreg::significand"]
        [::std::mem::offset_of!(_fpxreg, significand) - 0usize];
    ["Offset of field: _fpxreg::exponent"][::std::mem::offset_of!(_fpxreg, exponent) - 8usize];
    ["Offset of field: _fpxreg::padding"][::std::mem::offset_of!(_fpxreg, padding) - 10usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _xmmreg {
    pub element: [__u32; 4usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _xmmreg"][::std::mem::size_of::<_xmmreg>() - 16usize];
    ["Alignment of _xmmreg"][::std::mem::align_of::<_xmmreg>() - 4usize];
    ["Offset of field: _xmmreg::element"][::std::mem::offset_of!(_xmmreg, element) - 0usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct _fpstate_32 {
    pub cw: __u32,
    pub sw: __u32,
    pub tag: __u32,
    pub ipoff: __u32,
    pub cssel: __u32,
    pub dataoff: __u32,
    pub datasel: __u32,
    pub _st: [_fpreg; 8usize],
    pub status: __u16,
    pub magic: __u16,
    pub _fxsr_env: [__u32; 6usize],
    pub mxcsr: __u32,
    pub reserved: __u32,
    pub _fxsr_st: [_fpxreg; 8usize],
    pub _xmm: [_xmmreg; 8usize],
    pub __bindgen_anon_1: _fpstate_32__bindgen_ty_1,
    pub __bindgen_anon_2: _fpstate_32__bindgen_ty_2,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union _fpstate_32__bindgen_ty_1 {
    pub padding1: [__u32; 44usize],
    pub padding: [__u32; 44usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpstate_32__bindgen_ty_1"]
        [::std::mem::size_of::<_fpstate_32__bindgen_ty_1>() - 176usize];
    ["Alignment of _fpstate_32__bindgen_ty_1"]
        [::std::mem::align_of::<_fpstate_32__bindgen_ty_1>() - 4usize];
    ["Offset of field: _fpstate_32__bindgen_ty_1::padding1"]
        [::std::mem::offset_of!(_fpstate_32__bindgen_ty_1, padding1) - 0usize];
    ["Offset of field: _fpstate_32__bindgen_ty_1::padding"]
        [::std::mem::offset_of!(_fpstate_32__bindgen_ty_1, padding) - 0usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub union _fpstate_32__bindgen_ty_2 {
    pub padding2: [__u32; 12usize],
    pub sw_reserved: _fpx_sw_bytes,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpstate_32__bindgen_ty_2"]
        [::std::mem::size_of::<_fpstate_32__bindgen_ty_2>() - 48usize];
    ["Alignment of _fpstate_32__bindgen_ty_2"]
        [::std::mem::align_of::<_fpstate_32__bindgen_ty_2>() - 4usize];
    ["Offset of field: _fpstate_32__bindgen_ty_2::padding2"]
        [::std::mem::offset_of!(_fpstate_32__bindgen_ty_2, padding2) - 0usize];
    ["Offset of field: _fpstate_32__bindgen_ty_2::sw_reserved"]
        [::std::mem::offset_of!(_fpstate_32__bindgen_ty_2, sw_reserved) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpstate_32"][::std::mem::size_of::<_fpstate_32>() - 624usize];
    ["Alignment of _fpstate_32"][::std::mem::align_of::<_fpstate_32>() - 4usize];
    ["Offset of field: _fpstate_32::cw"][::std::mem::offset_of!(_fpstate_32, cw) - 0usize];
    ["Offset of field: _fpstate_32::sw"][::std::mem::offset_of!(_fpstate_32, sw) - 4usize];
    ["Offset of field: _fpstate_32::tag"][::std::mem::offset_of!(_fpstate_32, tag) - 8usize];
    ["Offset of field: _fpstate_32::ipoff"][::std::mem::offset_of!(_fpstate_32, ipoff) - 12usize];
    ["Offset of field: _fpstate_32::cssel"][::std::mem::offset_of!(_fpstate_32, cssel) - 16usize];
    ["Offset of field: _fpstate_32::dataoff"]
        [::std::mem::offset_of!(_fpstate_32, dataoff) - 20usize];
    ["Offset of field: _fpstate_32::datasel"]
        [::std::mem::offset_of!(_fpstate_32, datasel) - 24usize];
    ["Offset of field: _fpstate_32::_st"][::std::mem::offset_of!(_fpstate_32, _st) - 28usize];
    ["Offset of field: _fpstate_32::status"]
        [::std::mem::offset_of!(_fpstate_32, status) - 108usize];
    ["Offset of field: _fpstate_32::magic"][::std::mem::offset_of!(_fpstate_32, magic) - 110usize];
    ["Offset of field: _fpstate_32::_fxsr_env"]
        [::std::mem::offset_of!(_fpstate_32, _fxsr_env) - 112usize];
    ["Offset of field: _fpstate_32::mxcsr"][::std::mem::offset_of!(_fpstate_32, mxcsr) - 136usize];
    ["Offset of field: _fpstate_32::reserved"]
        [::std::mem::offset_of!(_fpstate_32, reserved) - 140usize];
    ["Offset of field: _fpstate_32::_fxsr_st"]
        [::std::mem::offset_of!(_fpstate_32, _fxsr_st) - 144usize];
    ["Offset of field: _fpstate_32::_xmm"][::std::mem::offset_of!(_fpstate_32, _xmm) - 272usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct _fpstate_64 {
    pub cwd: __u16,
    pub swd: __u16,
    pub twd: __u16,
    pub fop: __u16,
    pub rip: __u64,
    pub rdp: __u64,
    pub mxcsr: __u32,
    pub mxcsr_mask: __u32,
    pub st_space: [__u32; 32usize],
    pub xmm_space: [__u32; 64usize],
    pub reserved2: [__u32; 12usize],
    pub __bindgen_anon_1: _fpstate_64__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union _fpstate_64__bindgen_ty_1 {
    pub reserved3: [__u32; 12usize],
    pub sw_reserved: _fpx_sw_bytes,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpstate_64__bindgen_ty_1"]
        [::std::mem::size_of::<_fpstate_64__bindgen_ty_1>() - 48usize];
    ["Alignment of _fpstate_64__bindgen_ty_1"]
        [::std::mem::align_of::<_fpstate_64__bindgen_ty_1>() - 4usize];
    ["Offset of field: _fpstate_64__bindgen_ty_1::reserved3"]
        [::std::mem::offset_of!(_fpstate_64__bindgen_ty_1, reserved3) - 0usize];
    ["Offset of field: _fpstate_64__bindgen_ty_1::sw_reserved"]
        [::std::mem::offset_of!(_fpstate_64__bindgen_ty_1, sw_reserved) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _fpstate_64"][::std::mem::size_of::<_fpstate_64>() - 512usize];
    ["Alignment of _fpstate_64"][::std::mem::align_of::<_fpstate_64>() - 4usize];
    ["Offset of field: _fpstate_64::cwd"][::std::mem::offset_of!(_fpstate_64, cwd) - 0usize];
    ["Offset of field: _fpstate_64::swd"][::std::mem::offset_of!(_fpstate_64, swd) - 2usize];
    ["Offset of field: _fpstate_64::twd"][::std::mem::offset_of!(_fpstate_64, twd) - 4usize];
    ["Offset of field: _fpstate_64::fop"][::std::mem::offset_of!(_fpstate_64, fop) - 6usize];
    ["Offset of field: _fpstate_64::rip"][::std::mem::offset_of!(_fpstate_64, rip) - 8usize];
    ["Offset of field: _fpstate_64::rdp"][::std::mem::offset_of!(_fpstate_64, rdp) - 16usize];
    ["Offset of field: _fpstate_64::mxcsr"][::std::mem::offset_of!(_fpstate_64, mxcsr) - 24usize];
    ["Offset of field: _fpstate_64::mxcsr_mask"]
        [::std::mem::offset_of!(_fpstate_64, mxcsr_mask) - 28usize];
    ["Offset of field: _fpstate_64::st_space"]
        [::std::mem::offset_of!(_fpstate_64, st_space) - 32usize];
    ["Offset of field: _fpstate_64::xmm_space"]
        [::std::mem::offset_of!(_fpstate_64, xmm_space) - 160usize];
    ["Offset of field: _fpstate_64::reserved2"]
        [::std::mem::offset_of!(_fpstate_64, reserved2) - 416usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _header {
    pub xfeatures: __u64,
    pub reserved1: [__u64; 2usize],
    pub reserved2: [__u64; 5usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _header"][::std::mem::size_of::<_header>() - 64usize];
    ["Alignment of _header"][::std::mem::align_of::<_header>() - 4usize];
    ["Offset of field: _header::xfeatures"][::std::mem::offset_of!(_header, xfeatures) - 0usize];
    ["Offset of field: _header::reserved1"][::std::mem::offset_of!(_header, reserved1) - 8usize];
    ["Offset of field: _header::reserved2"][::std::mem::offset_of!(_header, reserved2) - 24usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _ymmh_state {
    pub ymmh_space: [__u32; 64usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _ymmh_state"][::std::mem::size_of::<_ymmh_state>() - 256usize];
    ["Alignment of _ymmh_state"][::std::mem::align_of::<_ymmh_state>() - 4usize];
    ["Offset of field: _ymmh_state::ymmh_space"]
        [::std::mem::offset_of!(_ymmh_state, ymmh_space) - 0usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct _xstate {
    pub fpstate: _fpstate_32,
    pub xstate_hdr: _header,
    pub ymmh: _ymmh_state,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _xstate"][::std::mem::size_of::<_xstate>() - 944usize];
    ["Alignment of _xstate"][::std::mem::align_of::<_xstate>() - 4usize];
    ["Offset of field: _xstate::fpstate"][::std::mem::offset_of!(_xstate, fpstate) - 0usize];
    ["Offset of field: _xstate::xstate_hdr"]
        [::std::mem::offset_of!(_xstate, xstate_hdr) - 624usize];
    ["Offset of field: _xstate::ymmh"][::std::mem::offset_of!(_xstate, ymmh) - 688usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sigcontext_32 {
    pub gs: __u16,
    pub __gsh: __u16,
    pub fs: __u16,
    pub __fsh: __u16,
    pub es: __u16,
    pub __esh: __u16,
    pub ds: __u16,
    pub __dsh: __u16,
    pub di: __u32,
    pub si: __u32,
    pub bp: __u32,
    pub sp: __u32,
    pub bx: __u32,
    pub dx: __u32,
    pub cx: __u32,
    pub ax: __u32,
    pub trapno: __u32,
    pub err: __u32,
    pub ip: __u32,
    pub cs: __u16,
    pub __csh: __u16,
    pub flags: __u32,
    pub sp_at_signal: __u32,
    pub ss: __u16,
    pub __ssh: __u16,
    pub fpstate: __u32,
    pub oldmask: __u32,
    pub cr2: __u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigcontext_32"][::std::mem::size_of::<sigcontext_32>() - 88usize];
    ["Alignment of sigcontext_32"][::std::mem::align_of::<sigcontext_32>() - 4usize];
    ["Offset of field: sigcontext_32::gs"][::std::mem::offset_of!(sigcontext_32, gs) - 0usize];
    ["Offset of field: sigcontext_32::__gsh"]
        [::std::mem::offset_of!(sigcontext_32, __gsh) - 2usize];
    ["Offset of field: sigcontext_32::fs"][::std::mem::offset_of!(sigcontext_32, fs) - 4usize];
    ["Offset of field: sigcontext_32::__fsh"]
        [::std::mem::offset_of!(sigcontext_32, __fsh) - 6usize];
    ["Offset of field: sigcontext_32::es"][::std::mem::offset_of!(sigcontext_32, es) - 8usize];
    ["Offset of field: sigcontext_32::__esh"]
        [::std::mem::offset_of!(sigcontext_32, __esh) - 10usize];
    ["Offset of field: sigcontext_32::ds"][::std::mem::offset_of!(sigcontext_32, ds) - 12usize];
    ["Offset of field: sigcontext_32::__dsh"]
        [::std::mem::offset_of!(sigcontext_32, __dsh) - 14usize];
    ["Offset of field: sigcontext_32::di"][::std::mem::offset_of!(sigcontext_32, di) - 16usize];
    ["Offset of field: sigcontext_32::si"][::std::mem::offset_of!(sigcontext_32, si) - 20usize];
    ["Offset of field: sigcontext_32::bp"][::std::mem::offset_of!(sigcontext_32, bp) - 24usize];
    ["Offset of field: sigcontext_32::sp"][::std::mem::offset_of!(sigcontext_32, sp) - 28usize];
    ["Offset of field: sigcontext_32::bx"][::std::mem::offset_of!(sigcontext_32, bx) - 32usize];
    ["Offset of field: sigcontext_32::dx"][::std::mem::offset_of!(sigcontext_32, dx) - 36usize];
    ["Offset of field: sigcontext_32::cx"][::std::mem::offset_of!(sigcontext_32, cx) - 40usize];
    ["Offset of field: sigcontext_32::ax"][::std::mem::offset_of!(sigcontext_32, ax) - 44usize];
    ["Offset of field: sigcontext_32::trapno"]
        [::std::mem::offset_of!(sigcontext_32, trapno) - 48usize];
    ["Offset of field: sigcontext_32::err"][::std::mem::offset_of!(sigcontext_32, err) - 52usize];
    ["Offset of field: sigcontext_32::ip"][::std::mem::offset_of!(sigcontext_32, ip) - 56usize];
    ["Offset of field: sigcontext_32::cs"][::std::mem::offset_of!(sigcontext_32, cs) - 60usize];
    ["Offset of field: sigcontext_32::__csh"]
        [::std::mem::offset_of!(sigcontext_32, __csh) - 62usize];
    ["Offset of field: sigcontext_32::flags"]
        [::std::mem::offset_of!(sigcontext_32, flags) - 64usize];
    ["Offset of field: sigcontext_32::sp_at_signal"]
        [::std::mem::offset_of!(sigcontext_32, sp_at_signal) - 68usize];
    ["Offset of field: sigcontext_32::ss"][::std::mem::offset_of!(sigcontext_32, ss) - 72usize];
    ["Offset of field: sigcontext_32::__ssh"]
        [::std::mem::offset_of!(sigcontext_32, __ssh) - 74usize];
    ["Offset of field: sigcontext_32::fpstate"]
        [::std::mem::offset_of!(sigcontext_32, fpstate) - 76usize];
    ["Offset of field: sigcontext_32::oldmask"]
        [::std::mem::offset_of!(sigcontext_32, oldmask) - 80usize];
    ["Offset of field: sigcontext_32::cr2"][::std::mem::offset_of!(sigcontext_32, cr2) - 84usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sigcontext_64 {
    pub r8: __u64,
    pub r9: __u64,
    pub r10: __u64,
    pub r11: __u64,
    pub r12: __u64,
    pub r13: __u64,
    pub r14: __u64,
    pub r15: __u64,
    pub di: __u64,
    pub si: __u64,
    pub bp: __u64,
    pub bx: __u64,
    pub dx: __u64,
    pub ax: __u64,
    pub cx: __u64,
    pub sp: __u64,
    pub ip: __u64,
    pub flags: __u64,
    pub cs: __u16,
    pub gs: __u16,
    pub fs: __u16,
    pub ss: __u16,
    pub err: __u64,
    pub trapno: __u64,
    pub oldmask: __u64,
    pub cr2: __u64,
    pub fpstate: __u64,
    pub reserved1: [__u64; 8usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigcontext_64"][::std::mem::size_of::<sigcontext_64>() - 256usize];
    ["Alignment of sigcontext_64"][::std::mem::align_of::<sigcontext_64>() - 4usize];
    ["Offset of field: sigcontext_64::r8"][::std::mem::offset_of!(sigcontext_64, r8) - 0usize];
    ["Offset of field: sigcontext_64::r9"][::std::mem::offset_of!(sigcontext_64, r9) - 8usize];
    ["Offset of field: sigcontext_64::r10"][::std::mem::offset_of!(sigcontext_64, r10) - 16usize];
    ["Offset of field: sigcontext_64::r11"][::std::mem::offset_of!(sigcontext_64, r11) - 24usize];
    ["Offset of field: sigcontext_64::r12"][::std::mem::offset_of!(sigcontext_64, r12) - 32usize];
    ["Offset of field: sigcontext_64::r13"][::std::mem::offset_of!(sigcontext_64, r13) - 40usize];
    ["Offset of field: sigcontext_64::r14"][::std::mem::offset_of!(sigcontext_64, r14) - 48usize];
    ["Offset of field: sigcontext_64::r15"][::std::mem::offset_of!(sigcontext_64, r15) - 56usize];
    ["Offset of field: sigcontext_64::di"][::std::mem::offset_of!(sigcontext_64, di) - 64usize];
    ["Offset of field: sigcontext_64::si"][::std::mem::offset_of!(sigcontext_64, si) - 72usize];
    ["Offset of field: sigcontext_64::bp"][::std::mem::offset_of!(sigcontext_64, bp) - 80usize];
    ["Offset of field: sigcontext_64::bx"][::std::mem::offset_of!(sigcontext_64, bx) - 88usize];
    ["Offset of field: sigcontext_64::dx"][::std::mem::offset_of!(sigcontext_64, dx) - 96usize];
    ["Offset of field: sigcontext_64::ax"][::std::mem::offset_of!(sigcontext_64, ax) - 104usize];
    ["Offset of field: sigcontext_64::cx"][::std::mem::offset_of!(sigcontext_64, cx) - 112usize];
    ["Offset of field: sigcontext_64::sp"][::std::mem::offset_of!(sigcontext_64, sp) - 120usize];
    ["Offset of field: sigcontext_64::ip"][::std::mem::offset_of!(sigcontext_64, ip) - 128usize];
    ["Offset of field: sigcontext_64::flags"]
        [::std::mem::offset_of!(sigcontext_64, flags) - 136usize];
    ["Offset of field: sigcontext_64::cs"][::std::mem::offset_of!(sigcontext_64, cs) - 144usize];
    ["Offset of field: sigcontext_64::gs"][::std::mem::offset_of!(sigcontext_64, gs) - 146usize];
    ["Offset of field: sigcontext_64::fs"][::std::mem::offset_of!(sigcontext_64, fs) - 148usize];
    ["Offset of field: sigcontext_64::ss"][::std::mem::offset_of!(sigcontext_64, ss) - 150usize];
    ["Offset of field: sigcontext_64::err"][::std::mem::offset_of!(sigcontext_64, err) - 152usize];
    ["Offset of field: sigcontext_64::trapno"]
        [::std::mem::offset_of!(sigcontext_64, trapno) - 160usize];
    ["Offset of field: sigcontext_64::oldmask"]
        [::std::mem::offset_of!(sigcontext_64, oldmask) - 168usize];
    ["Offset of field: sigcontext_64::cr2"][::std::mem::offset_of!(sigcontext_64, cr2) - 176usize];
    ["Offset of field: sigcontext_64::fpstate"]
        [::std::mem::offset_of!(sigcontext_64, fpstate) - 184usize];
    ["Offset of field: sigcontext_64::reserved1"]
        [::std::mem::offset_of!(sigcontext_64, reserved1) - 192usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sigcontext {
    pub gs: __u16,
    pub __gsh: __u16,
    pub fs: __u16,
    pub __fsh: __u16,
    pub es: __u16,
    pub __esh: __u16,
    pub ds: __u16,
    pub __dsh: __u16,
    pub edi: __u32,
    pub esi: __u32,
    pub ebp: __u32,
    pub esp: __u32,
    pub ebx: __u32,
    pub edx: __u32,
    pub ecx: __u32,
    pub eax: __u32,
    pub trapno: __u32,
    pub err: __u32,
    pub eip: __u32,
    pub cs: __u16,
    pub __csh: __u16,
    pub eflags: __u32,
    pub esp_at_signal: __u32,
    pub ss: __u16,
    pub __ssh: __u16,
    pub fpstate: *mut _fpstate_32,
    pub oldmask: __u32,
    pub cr2: __u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigcontext"][::std::mem::size_of::<sigcontext>() - 88usize];
    ["Alignment of sigcontext"][::std::mem::align_of::<sigcontext>() - 4usize];
    ["Offset of field: sigcontext::gs"][::std::mem::offset_of!(sigcontext, gs) - 0usize];
    ["Offset of field: sigcontext::__gsh"][::std::mem::offset_of!(sigcontext, __gsh) - 2usize];
    ["Offset of field: sigcontext::fs"][::std::mem::offset_of!(sigcontext, fs) - 4usize];
    ["Offset of field: sigcontext::__fsh"][::std::mem::offset_of!(sigcontext, __fsh) - 6usize];
    ["Offset of field: sigcontext::es"][::std::mem::offset_of!(sigcontext, es) - 8usize];
    ["Offset of field: sigcontext::__esh"][::std::mem::offset_of!(sigcontext, __esh) - 10usize];
    ["Offset of field: sigcontext::ds"][::std::mem::offset_of!(sigcontext, ds) - 12usize];
    ["Offset of field: sigcontext::__dsh"][::std::mem::offset_of!(sigcontext, __dsh) - 14usize];
    ["Offset of field: sigcontext::edi"][::std::mem::offset_of!(sigcontext, edi) - 16usize];
    ["Offset of field: sigcontext::esi"][::std::mem::offset_of!(sigcontext, esi) - 20usize];
    ["Offset of field: sigcontext::ebp"][::std::mem::offset_of!(sigcontext, ebp) - 24usize];
    ["Offset of field: sigcontext::esp"][::std::mem::offset_of!(sigcontext, esp) - 28usize];
    ["Offset of field: sigcontext::ebx"][::std::mem::offset_of!(sigcontext, ebx) - 32usize];
    ["Offset of field: sigcontext::edx"][::std::mem::offset_of!(sigcontext, edx) - 36usize];
    ["Offset of field: sigcontext::ecx"][::std::mem::offset_of!(sigcontext, ecx) - 40usize];
    ["Offset of field: sigcontext::eax"][::std::mem::offset_of!(sigcontext, eax) - 44usize];
    ["Offset of field: sigcontext::trapno"][::std::mem::offset_of!(sigcontext, trapno) - 48usize];
    ["Offset of field: sigcontext::err"][::std::mem::offset_of!(sigcontext, err) - 52usize];
    ["Offset of field: sigcontext::eip"][::std::mem::offset_of!(sigcontext, eip) - 56usize];
    ["Offset of field: sigcontext::cs"][::std::mem::offset_of!(sigcontext, cs) - 60usize];
    ["Offset of field: sigcontext::__csh"][::std::mem::offset_of!(sigcontext, __csh) - 62usize];
    ["Offset of field: sigcontext::eflags"][::std::mem::offset_of!(sigcontext, eflags) - 64usize];
    ["Offset of field: sigcontext::esp_at_signal"]
        [::std::mem::offset_of!(sigcontext, esp_at_signal) - 68usize];
    ["Offset of field: sigcontext::ss"][::std::mem::offset_of!(sigcontext, ss) - 72usize];
    ["Offset of field: sigcontext::__ssh"][::std::mem::offset_of!(sigcontext, __ssh) - 74usize];
    ["Offset of field: sigcontext::fpstate"][::std::mem::offset_of!(sigcontext, fpstate) - 76usize];
    ["Offset of field: sigcontext::oldmask"][::std::mem::offset_of!(sigcontext, oldmask) - 80usize];
    ["Offset of field: sigcontext::cr2"][::std::mem::offset_of!(sigcontext, cr2) - 84usize];
};
pub type sigset_t = ::std::os::raw::c_ulong;
pub type __signalfn_t = ::std::option::Option<unsafe extern "C" fn(arg1: ::std::os::raw::c_int)>;
pub type __sighandler_t = __signalfn_t;
pub type __restorefn_t = ::std::option::Option<unsafe extern "C" fn()>;
pub type __sigrestore_t = __restorefn_t;
#[repr(C)]
#[derive(Copy, Clone)]
pub struct __kernel_sigaction {
    pub _u: __kernel_sigaction__bindgen_ty_1,
    pub sa_mask: sigset_t,
    pub sa_flags: ::std::os::raw::c_ulong,
    pub sa_restorer: ::std::option::Option<unsafe extern "C" fn()>,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union __kernel_sigaction__bindgen_ty_1 {
    pub _sa_handler: __sighandler_t,
    pub _sa_sigaction: ::std::option::Option<
        unsafe extern "C" fn(
            arg1: ::std::os::raw::c_int,
            arg2: *mut siginfo,
            arg3: *mut ::std::os::raw::c_void,
        ),
    >,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_sigaction__bindgen_ty_1"]
        [::std::mem::size_of::<__kernel_sigaction__bindgen_ty_1>() - 4usize];
    ["Alignment of __kernel_sigaction__bindgen_ty_1"]
        [::std::mem::align_of::<__kernel_sigaction__bindgen_ty_1>() - 4usize];
    ["Offset of field: __kernel_sigaction__bindgen_ty_1::_sa_handler"]
        [::std::mem::offset_of!(__kernel_sigaction__bindgen_ty_1, _sa_handler) - 0usize];
    ["Offset of field: __kernel_sigaction__bindgen_ty_1::_sa_sigaction"]
        [::std::mem::offset_of!(__kernel_sigaction__bindgen_ty_1, _sa_sigaction) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_sigaction"][::std::mem::size_of::<__kernel_sigaction>() - 16usize];
    ["Alignment of __kernel_sigaction"][::std::mem::align_of::<__kernel_sigaction>() - 4usize];
    ["Offset of field: __kernel_sigaction::_u"]
        [::std::mem::offset_of!(__kernel_sigaction, _u) - 0usize];
    ["Offset of field: __kernel_sigaction::sa_mask"]
        [::std::mem::offset_of!(__kernel_sigaction, sa_mask) - 4usize];
    ["Offset of field: __kernel_sigaction::sa_flags"]
        [::std::mem::offset_of!(__kernel_sigaction, sa_flags) - 8usize];
    ["Offset of field: __kernel_sigaction::sa_restorer"]
        [::std::mem::offset_of!(__kernel_sigaction, sa_restorer) - 12usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sigaltstack {
    pub ss_sp: *mut ::std::os::raw::c_void,
    pub ss_flags: ::std::os::raw::c_int,
    pub ss_size: __kernel_size_t,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigaltstack"][::std::mem::size_of::<sigaltstack>() - 12usize];
    ["Alignment of sigaltstack"][::std::mem::align_of::<sigaltstack>() - 4usize];
    ["Offset of field: sigaltstack::ss_sp"][::std::mem::offset_of!(sigaltstack, ss_sp) - 0usize];
    ["Offset of field: sigaltstack::ss_flags"]
        [::std::mem::offset_of!(sigaltstack, ss_flags) - 4usize];
    ["Offset of field: sigaltstack::ss_size"]
        [::std::mem::offset_of!(sigaltstack, ss_size) - 8usize];
};
pub type stack_t = sigaltstack;
#[repr(C)]
#[derive(Copy, Clone)]
pub union sigval {
    pub sival_int: ::std::os::raw::c_int,
    pub sival_ptr: *mut ::std::os::raw::c_void,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigval"][::std::mem::size_of::<sigval>() - 4usize];
    ["Alignment of sigval"][::std::mem::align_of::<sigval>() - 4usize];
    ["Offset of field: sigval::sival_int"][::std::mem::offset_of!(sigval, sival_int) - 0usize];
    ["Offset of field: sigval::sival_ptr"][::std::mem::offset_of!(sigval, sival_ptr) - 0usize];
};
pub type sigval_t = sigval;
#[repr(C)]
#[derive(Copy, Clone)]
pub union __sifields {
    pub _kill: __sifields__bindgen_ty_1,
    pub _timer: __sifields__bindgen_ty_2,
    pub _rt: __sifields__bindgen_ty_3,
    pub _sigchld: __sifields__bindgen_ty_4,
    pub _sigfault: __sifields__bindgen_ty_5,
    pub _sigpoll: __sifields__bindgen_ty_6,
    pub _sigsys: __sifields__bindgen_ty_7,
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_1 {
    pub _pid: __kernel_pid_t,
    pub _uid: __kernel_uid32_t,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_1"]
        [::std::mem::size_of::<__sifields__bindgen_ty_1>() - 8usize];
    ["Alignment of __sifields__bindgen_ty_1"]
        [::std::mem::align_of::<__sifields__bindgen_ty_1>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_1::_pid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_1, _pid) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_1::_uid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_1, _uid) - 4usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct __sifields__bindgen_ty_2 {
    pub _tid: __kernel_timer_t,
    pub _overrun: ::std::os::raw::c_int,
    pub _sigval: sigval_t,
    pub _sys_private: ::std::os::raw::c_int,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_2"]
        [::std::mem::size_of::<__sifields__bindgen_ty_2>() - 16usize];
    ["Alignment of __sifields__bindgen_ty_2"]
        [::std::mem::align_of::<__sifields__bindgen_ty_2>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_2::_tid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_2, _tid) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_2::_overrun"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_2, _overrun) - 4usize];
    ["Offset of field: __sifields__bindgen_ty_2::_sigval"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_2, _sigval) - 8usize];
    ["Offset of field: __sifields__bindgen_ty_2::_sys_private"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_2, _sys_private) - 12usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct __sifields__bindgen_ty_3 {
    pub _pid: __kernel_pid_t,
    pub _uid: __kernel_uid32_t,
    pub _sigval: sigval_t,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_3"]
        [::std::mem::size_of::<__sifields__bindgen_ty_3>() - 12usize];
    ["Alignment of __sifields__bindgen_ty_3"]
        [::std::mem::align_of::<__sifields__bindgen_ty_3>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_3::_pid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_3, _pid) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_3::_uid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_3, _uid) - 4usize];
    ["Offset of field: __sifields__bindgen_ty_3::_sigval"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_3, _sigval) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_4 {
    pub _pid: __kernel_pid_t,
    pub _uid: __kernel_uid32_t,
    pub _status: ::std::os::raw::c_int,
    pub _utime: __kernel_clock_t,
    pub _stime: __kernel_clock_t,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_4"]
        [::std::mem::size_of::<__sifields__bindgen_ty_4>() - 20usize];
    ["Alignment of __sifields__bindgen_ty_4"]
        [::std::mem::align_of::<__sifields__bindgen_ty_4>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_4::_pid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_4, _pid) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_4::_uid"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_4, _uid) - 4usize];
    ["Offset of field: __sifields__bindgen_ty_4::_status"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_4, _status) - 8usize];
    ["Offset of field: __sifields__bindgen_ty_4::_utime"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_4, _utime) - 12usize];
    ["Offset of field: __sifields__bindgen_ty_4::_stime"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_4, _stime) - 16usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct __sifields__bindgen_ty_5 {
    pub _addr: *mut ::std::os::raw::c_void,
    pub __bindgen_anon_1: __sifields__bindgen_ty_5__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union __sifields__bindgen_ty_5__bindgen_ty_1 {
    pub _trapno: ::std::os::raw::c_int,
    pub _addr_lsb: ::std::os::raw::c_short,
    pub _addr_bnd: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1,
    pub _addr_pkey: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2,
    pub _perf: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3,
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1 {
    pub _dummy_bnd: [::std::os::raw::c_char; 4usize],
    pub _lower: *mut ::std::os::raw::c_void,
    pub _upper: *mut ::std::os::raw::c_void,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::size_of::<__sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1>() - 12usize];
    ["Alignment of __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::align_of::<__sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1::_dummy_bnd"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1,
        _dummy_bnd
    )
        - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1::_lower"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1,
        _lower
    ) - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1::_upper"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_1,
        _upper
    ) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2 {
    pub _dummy_pkey: [::std::os::raw::c_char; 4usize],
    pub _pkey: __u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2"]
        [::std::mem::size_of::<__sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2>() - 8usize];
    ["Alignment of __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2"]
        [::std::mem::align_of::<__sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2::_dummy_pkey"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2,
        _dummy_pkey
    )
        - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2::_pkey"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_2,
        _pkey
    ) - 4usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3 {
    pub _data: ::std::os::raw::c_ulong,
    pub _type: __u32,
    pub _flags: __u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3"]
        [::std::mem::size_of::<__sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3>() - 12usize];
    ["Alignment of __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3"]
        [::std::mem::align_of::<__sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3::_data"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3,
        _data
    ) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3::_type"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3,
        _type
    ) - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3::_flags"][::std::mem::offset_of!(
        __sifields__bindgen_ty_5__bindgen_ty_1__bindgen_ty_3,
        _flags
    ) - 8usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_5__bindgen_ty_1"]
        [::std::mem::size_of::<__sifields__bindgen_ty_5__bindgen_ty_1>() - 12usize];
    ["Alignment of __sifields__bindgen_ty_5__bindgen_ty_1"]
        [::std::mem::align_of::<__sifields__bindgen_ty_5__bindgen_ty_1>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1::_trapno"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_5__bindgen_ty_1, _trapno) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1::_addr_lsb"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_5__bindgen_ty_1, _addr_lsb) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1::_addr_bnd"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_5__bindgen_ty_1, _addr_bnd) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1::_addr_pkey"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_5__bindgen_ty_1, _addr_pkey) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_5__bindgen_ty_1::_perf"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_5__bindgen_ty_1, _perf) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_5"]
        [::std::mem::size_of::<__sifields__bindgen_ty_5>() - 16usize];
    ["Alignment of __sifields__bindgen_ty_5"]
        [::std::mem::align_of::<__sifields__bindgen_ty_5>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_5::_addr"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_5, _addr) - 0usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_6 {
    pub _band: ::std::os::raw::c_long,
    pub _fd: ::std::os::raw::c_int,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_6"]
        [::std::mem::size_of::<__sifields__bindgen_ty_6>() - 8usize];
    ["Alignment of __sifields__bindgen_ty_6"]
        [::std::mem::align_of::<__sifields__bindgen_ty_6>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_6::_band"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_6, _band) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_6::_fd"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_6, _fd) - 4usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __sifields__bindgen_ty_7 {
    pub _call_addr: *mut ::std::os::raw::c_void,
    pub _syscall: ::std::os::raw::c_int,
    pub _arch: ::std::os::raw::c_uint,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields__bindgen_ty_7"]
        [::std::mem::size_of::<__sifields__bindgen_ty_7>() - 12usize];
    ["Alignment of __sifields__bindgen_ty_7"]
        [::std::mem::align_of::<__sifields__bindgen_ty_7>() - 4usize];
    ["Offset of field: __sifields__bindgen_ty_7::_call_addr"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_7, _call_addr) - 0usize];
    ["Offset of field: __sifields__bindgen_ty_7::_syscall"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_7, _syscall) - 4usize];
    ["Offset of field: __sifields__bindgen_ty_7::_arch"]
        [::std::mem::offset_of!(__sifields__bindgen_ty_7, _arch) - 8usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __sifields"][::std::mem::size_of::<__sifields>() - 20usize];
    ["Alignment of __sifields"][::std::mem::align_of::<__sifields>() - 4usize];
    ["Offset of field: __sifields::_kill"][::std::mem::offset_of!(__sifields, _kill) - 0usize];
    ["Offset of field: __sifields::_timer"][::std::mem::offset_of!(__sifields, _timer) - 0usize];
    ["Offset of field: __sifields::_rt"][::std::mem::offset_of!(__sifields, _rt) - 0usize];
    ["Offset of field: __sifields::_sigchld"]
        [::std::mem::offset_of!(__sifields, _sigchld) - 0usize];
    ["Offset of field: __sifields::_sigfault"]
        [::std::mem::offset_of!(__sifields, _sigfault) - 0usize];
    ["Offset of field: __sifields::_sigpoll"]
        [::std::mem::offset_of!(__sifields, _sigpoll) - 0usize];
    ["Offset of field: __sifields::_sigsys"][::std::mem::offset_of!(__sifields, _sigsys) - 0usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct siginfo {
    pub __bindgen_anon_1: siginfo__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union siginfo__bindgen_ty_1 {
    pub __bindgen_anon_1: siginfo__bindgen_ty_1__bindgen_ty_1,
    pub _si_pad: [::std::os::raw::c_int; 32usize],
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct siginfo__bindgen_ty_1__bindgen_ty_1 {
    pub si_signo: ::std::os::raw::c_int,
    pub si_errno: ::std::os::raw::c_int,
    pub si_code: ::std::os::raw::c_int,
    pub _sifields: __sifields,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of siginfo__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::size_of::<siginfo__bindgen_ty_1__bindgen_ty_1>() - 32usize];
    ["Alignment of siginfo__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::align_of::<siginfo__bindgen_ty_1__bindgen_ty_1>() - 4usize];
    ["Offset of field: siginfo__bindgen_ty_1__bindgen_ty_1::si_signo"]
        [::std::mem::offset_of!(siginfo__bindgen_ty_1__bindgen_ty_1, si_signo) - 0usize];
    ["Offset of field: siginfo__bindgen_ty_1__bindgen_ty_1::si_errno"]
        [::std::mem::offset_of!(siginfo__bindgen_ty_1__bindgen_ty_1, si_errno) - 4usize];
    ["Offset of field: siginfo__bindgen_ty_1__bindgen_ty_1::si_code"]
        [::std::mem::offset_of!(siginfo__bindgen_ty_1__bindgen_ty_1, si_code) - 8usize];
    ["Offset of field: siginfo__bindgen_ty_1__bindgen_ty_1::_sifields"]
        [::std::mem::offset_of!(siginfo__bindgen_ty_1__bindgen_ty_1, _sifields) - 12usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of siginfo__bindgen_ty_1"][::std::mem::size_of::<siginfo__bindgen_ty_1>() - 128usize];
    ["Alignment of siginfo__bindgen_ty_1"]
        [::std::mem::align_of::<siginfo__bindgen_ty_1>() - 4usize];
    ["Offset of field: siginfo__bindgen_ty_1::_si_pad"]
        [::std::mem::offset_of!(siginfo__bindgen_ty_1, _si_pad) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of siginfo"][::std::mem::size_of::<siginfo>() - 128usize];
    ["Alignment of siginfo"][::std::mem::align_of::<siginfo>() - 4usize];
};
pub type siginfo_t = siginfo;
#[repr(C)]
#[derive(Copy, Clone)]
pub struct sigevent {
    pub sigev_value: sigval_t,
    pub sigev_signo: ::std::os::raw::c_int,
    pub sigev_notify: ::std::os::raw::c_int,
    pub _sigev_un: sigevent__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union sigevent__bindgen_ty_1 {
    pub _pad: [::std::os::raw::c_int; 13usize],
    pub _tid: ::std::os::raw::c_int,
    pub _sigev_thread: sigevent__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sigevent__bindgen_ty_1__bindgen_ty_1 {
    pub _function: ::std::option::Option<unsafe extern "C" fn(arg1: sigval_t)>,
    pub _attribute: *mut ::std::os::raw::c_void,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigevent__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::size_of::<sigevent__bindgen_ty_1__bindgen_ty_1>() - 8usize];
    ["Alignment of sigevent__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::align_of::<sigevent__bindgen_ty_1__bindgen_ty_1>() - 4usize];
    ["Offset of field: sigevent__bindgen_ty_1__bindgen_ty_1::_function"]
        [::std::mem::offset_of!(sigevent__bindgen_ty_1__bindgen_ty_1, _function) - 0usize];
    ["Offset of field: sigevent__bindgen_ty_1__bindgen_ty_1::_attribute"]
        [::std::mem::offset_of!(sigevent__bindgen_ty_1__bindgen_ty_1, _attribute) - 4usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigevent__bindgen_ty_1"][::std::mem::size_of::<sigevent__bindgen_ty_1>() - 52usize];
    ["Alignment of sigevent__bindgen_ty_1"]
        [::std::mem::align_of::<sigevent__bindgen_ty_1>() - 4usize];
    ["Offset of field: sigevent__bindgen_ty_1::_pad"]
        [::std::mem::offset_of!(sigevent__bindgen_ty_1, _pad) - 0usize];
    ["Offset of field: sigevent__bindgen_ty_1::_tid"]
        [::std::mem::offset_of!(sigevent__bindgen_ty_1, _tid) - 0usize];
    ["Offset of field: sigevent__bindgen_ty_1::_sigev_thread"]
        [::std::mem::offset_of!(sigevent__bindgen_ty_1, _sigev_thread) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigevent"][::std::mem::size_of::<sigevent>() - 64usize];
    ["Alignment of sigevent"][::std::mem::align_of::<sigevent>() - 4usize];
    ["Offset of field: sigevent::sigev_value"]
        [::std::mem::offset_of!(sigevent, sigev_value) - 0usize];
    ["Offset of field: sigevent::sigev_signo"]
        [::std::mem::offset_of!(sigevent, sigev_signo) - 4usize];
    ["Offset of field: sigevent::sigev_notify"]
        [::std::mem::offset_of!(sigevent, sigev_notify) - 8usize];
    ["Offset of field: sigevent::_sigev_un"][::std::mem::offset_of!(sigevent, _sigev_un) - 12usize];
};
pub type sigevent_t = sigevent;
pub type sig_atomic_t = ::std::os::raw::c_int;
pub type sig_t = __sighandler_t;
pub type sighandler_t = __sighandler_t;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sigset64_t {
    pub __bits: [::std::os::raw::c_ulong; 2usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigset64_t"][::std::mem::size_of::<sigset64_t>() - 8usize];
    ["Alignment of sigset64_t"][::std::mem::align_of::<sigset64_t>() - 4usize];
    ["Offset of field: sigset64_t::__bits"][::std::mem::offset_of!(sigset64_t, __bits) - 0usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct sigaction {
    pub __bindgen_anon_1: sigaction__bindgen_ty_1,
    pub sa_mask: sigset_t,
    pub sa_flags: ::std::os::raw::c_int,
    pub sa_restorer: ::std::option::Option<unsafe extern "C" fn()>,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union sigaction__bindgen_ty_1 {
    pub sa_handler: sighandler_t,
    pub sa_sigaction: ::std::option::Option<
        unsafe extern "C" fn(
            arg1: ::std::os::raw::c_int,
            arg2: *mut siginfo,
            arg3: *mut ::std::os::raw::c_void,
        ),
    >,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigaction__bindgen_ty_1"][::std::mem::size_of::<sigaction__bindgen_ty_1>() - 4usize];
    ["Alignment of sigaction__bindgen_ty_1"]
        [::std::mem::align_of::<sigaction__bindgen_ty_1>() - 4usize];
    ["Offset of field: sigaction__bindgen_ty_1::sa_handler"]
        [::std::mem::offset_of!(sigaction__bindgen_ty_1, sa_handler) - 0usize];
    ["Offset of field: sigaction__bindgen_ty_1::sa_sigaction"]
        [::std::mem::offset_of!(sigaction__bindgen_ty_1, sa_sigaction) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigaction"][::std::mem::size_of::<sigaction>() - 16usize];
    ["Alignment of sigaction"][::std::mem::align_of::<sigaction>() - 4usize];
    ["Offset of field: sigaction::sa_mask"][::std::mem::offset_of!(sigaction, sa_mask) - 4usize];
    ["Offset of field: sigaction::sa_flags"][::std::mem::offset_of!(sigaction, sa_flags) - 8usize];
    ["Offset of field: sigaction::sa_restorer"]
        [::std::mem::offset_of!(sigaction, sa_restorer) - 12usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct sigaction64 {
    pub __bindgen_anon_1: sigaction64__bindgen_ty_1,
    pub sa_flags: ::std::os::raw::c_int,
    pub sa_restorer: ::std::option::Option<unsafe extern "C" fn()>,
    pub sa_mask: sigset64_t,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union sigaction64__bindgen_ty_1 {
    pub sa_handler: sighandler_t,
    pub sa_sigaction: ::std::option::Option<
        unsafe extern "C" fn(
            arg1: ::std::os::raw::c_int,
            arg2: *mut siginfo,
            arg3: *mut ::std::os::raw::c_void,
        ),
    >,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigaction64__bindgen_ty_1"]
        [::std::mem::size_of::<sigaction64__bindgen_ty_1>() - 4usize];
    ["Alignment of sigaction64__bindgen_ty_1"]
        [::std::mem::align_of::<sigaction64__bindgen_ty_1>() - 4usize];
    ["Offset of field: sigaction64__bindgen_ty_1::sa_handler"]
        [::std::mem::offset_of!(sigaction64__bindgen_ty_1, sa_handler) - 0usize];
    ["Offset of field: sigaction64__bindgen_ty_1::sa_sigaction"]
        [::std::mem::offset_of!(sigaction64__bindgen_ty_1, sa_sigaction) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sigaction64"][::std::mem::size_of::<sigaction64>() - 20usize];
    ["Alignment of sigaction64"][::std::mem::align_of::<sigaction64>() - 4usize];
    ["Offset of field: sigaction64::sa_flags"]
        [::std::mem::offset_of!(sigaction64, sa_flags) - 4usize];
    ["Offset of field: sigaction64::sa_restorer"]
        [::std::mem::offset_of!(sigaction64, sa_restorer) - 8usize];
    ["Offset of field: sigaction64::sa_mask"]
        [::std::mem::offset_of!(sigaction64, sa_mask) - 12usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct timespec {
    pub tv_sec: time_t,
    pub tv_nsec: ::std::os::raw::c_long,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of timespec"][::std::mem::size_of::<timespec>() - 8usize];
    ["Alignment of timespec"][::std::mem::align_of::<timespec>() - 4usize];
    ["Offset of field: timespec::tv_sec"][::std::mem::offset_of!(timespec, tv_sec) - 0usize];
    ["Offset of field: timespec::tv_nsec"][::std::mem::offset_of!(timespec, tv_nsec) - 4usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct user_fpregs_struct {
    pub cwd: ::std::os::raw::c_long,
    pub swd: ::std::os::raw::c_long,
    pub twd: ::std::os::raw::c_long,
    pub fip: ::std::os::raw::c_long,
    pub fcs: ::std::os::raw::c_long,
    pub foo: ::std::os::raw::c_long,
    pub fos: ::std::os::raw::c_long,
    pub st_space: [::std::os::raw::c_long; 20usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of user_fpregs_struct"][::std::mem::size_of::<user_fpregs_struct>() - 108usize];
    ["Alignment of user_fpregs_struct"][::std::mem::align_of::<user_fpregs_struct>() - 4usize];
    ["Offset of field: user_fpregs_struct::cwd"]
        [::std::mem::offset_of!(user_fpregs_struct, cwd) - 0usize];
    ["Offset of field: user_fpregs_struct::swd"]
        [::std::mem::offset_of!(user_fpregs_struct, swd) - 4usize];
    ["Offset of field: user_fpregs_struct::twd"]
        [::std::mem::offset_of!(user_fpregs_struct, twd) - 8usize];
    ["Offset of field: user_fpregs_struct::fip"]
        [::std::mem::offset_of!(user_fpregs_struct, fip) - 12usize];
    ["Offset of field: user_fpregs_struct::fcs"]
        [::std::mem::offset_of!(user_fpregs_struct, fcs) - 16usize];
    ["Offset of field: user_fpregs_struct::foo"]
        [::std::mem::offset_of!(user_fpregs_struct, foo) - 20usize];
    ["Offset of field: user_fpregs_struct::fos"]
        [::std::mem::offset_of!(user_fpregs_struct, fos) - 24usize];
    ["Offset of field: user_fpregs_struct::st_space"]
        [::std::mem::offset_of!(user_fpregs_struct, st_space) - 28usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct user_fpxregs_struct {
    pub cwd: ::std::os::raw::c_ushort,
    pub swd: ::std::os::raw::c_ushort,
    pub twd: ::std::os::raw::c_ushort,
    pub fop: ::std::os::raw::c_ushort,
    pub fip: ::std::os::raw::c_long,
    pub fcs: ::std::os::raw::c_long,
    pub foo: ::std::os::raw::c_long,
    pub fos: ::std::os::raw::c_long,
    pub mxcsr: ::std::os::raw::c_long,
    pub reserved: ::std::os::raw::c_long,
    pub st_space: [::std::os::raw::c_long; 32usize],
    pub xmm_space: [::std::os::raw::c_long; 32usize],
    pub padding: [::std::os::raw::c_long; 56usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of user_fpxregs_struct"][::std::mem::size_of::<user_fpxregs_struct>() - 512usize];
    ["Alignment of user_fpxregs_struct"][::std::mem::align_of::<user_fpxregs_struct>() - 4usize];
    ["Offset of field: user_fpxregs_struct::cwd"]
        [::std::mem::offset_of!(user_fpxregs_struct, cwd) - 0usize];
    ["Offset of field: user_fpxregs_struct::swd"]
        [::std::mem::offset_of!(user_fpxregs_struct, swd) - 2usize];
    ["Offset of field: user_fpxregs_struct::twd"]
        [::std::mem::offset_of!(user_fpxregs_struct, twd) - 4usize];
    ["Offset of field: user_fpxregs_struct::fop"]
        [::std::mem::offset_of!(user_fpxregs_struct, fop) - 6usize];
    ["Offset of field: user_fpxregs_struct::fip"]
        [::std::mem::offset_of!(user_fpxregs_struct, fip) - 8usize];
    ["Offset of field: user_fpxregs_struct::fcs"]
        [::std::mem::offset_of!(user_fpxregs_struct, fcs) - 12usize];
    ["Offset of field: user_fpxregs_struct::foo"]
        [::std::mem::offset_of!(user_fpxregs_struct, foo) - 16usize];
    ["Offset of field: user_fpxregs_struct::fos"]
        [::std::mem::offset_of!(user_fpxregs_struct, fos) - 20usize];
    ["Offset of field: user_fpxregs_struct::mxcsr"]
        [::std::mem::offset_of!(user_fpxregs_struct, mxcsr) - 24usize];
    ["Offset of field: user_fpxregs_struct::reserved"]
        [::std::mem::offset_of!(user_fpxregs_struct, reserved) - 28usize];
    ["Offset of field: user_fpxregs_struct::st_space"]
        [::std::mem::offset_of!(user_fpxregs_struct, st_space) - 32usize];
    ["Offset of field: user_fpxregs_struct::xmm_space"]
        [::std::mem::offset_of!(user_fpxregs_struct, xmm_space) - 160usize];
    ["Offset of field: user_fpxregs_struct::padding"]
        [::std::mem::offset_of!(user_fpxregs_struct, padding) - 288usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct user_regs_struct {
    pub ebx: ::std::os::raw::c_long,
    pub ecx: ::std::os::raw::c_long,
    pub edx: ::std::os::raw::c_long,
    pub esi: ::std::os::raw::c_long,
    pub edi: ::std::os::raw::c_long,
    pub ebp: ::std::os::raw::c_long,
    pub eax: ::std::os::raw::c_long,
    pub xds: ::std::os::raw::c_long,
    pub xes: ::std::os::raw::c_long,
    pub xfs: ::std::os::raw::c_long,
    pub xgs: ::std::os::raw::c_long,
    pub orig_eax: ::std::os::raw::c_long,
    pub eip: ::std::os::raw::c_long,
    pub xcs: ::std::os::raw::c_long,
    pub eflags: ::std::os::raw::c_long,
    pub esp: ::std::os::raw::c_long,
    pub xss: ::std::os::raw::c_long,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of user_regs_struct"][::std::mem::size_of::<user_regs_struct>() - 68usize];
    ["Alignment of user_regs_struct"][::std::mem::align_of::<user_regs_struct>() - 4usize];
    ["Offset of field: user_regs_struct::ebx"]
        [::std::mem::offset_of!(user_regs_struct, ebx) - 0usize];
    ["Offset of field: user_regs_struct::ecx"]
        [::std::mem::offset_of!(user_regs_struct, ecx) - 4usize];
    ["Offset of field: user_regs_struct::edx"]
        [::std::mem::offset_of!(user_regs_struct, edx) - 8usize];
    ["Offset of field: user_regs_struct::esi"]
        [::std::mem::offset_of!(user_regs_struct, esi) - 12usize];
    ["Offset of field: user_regs_struct::edi"]
        [::std::mem::offset_of!(user_regs_struct, edi) - 16usize];
    ["Offset of field: user_regs_struct::ebp"]
        [::std::mem::offset_of!(user_regs_struct, ebp) - 20usize];
    ["Offset of field: user_regs_struct::eax"]
        [::std::mem::offset_of!(user_regs_struct, eax) - 24usize];
    ["Offset of field: user_regs_struct::xds"]
        [::std::mem::offset_of!(user_regs_struct, xds) - 28usize];
    ["Offset of field: user_regs_struct::xes"]
        [::std::mem::offset_of!(user_regs_struct, xes) - 32usize];
    ["Offset of field: user_regs_struct::xfs"]
        [::std::mem::offset_of!(user_regs_struct, xfs) - 36usize];
    ["Offset of field: user_regs_struct::xgs"]
        [::std::mem::offset_of!(user_regs_struct, xgs) - 40usize];
    ["Offset of field: user_regs_struct::orig_eax"]
        [::std::mem::offset_of!(user_regs_struct, orig_eax) - 44usize];
    ["Offset of field: user_regs_struct::eip"]
        [::std::mem::offset_of!(user_regs_struct, eip) - 48usize];
    ["Offset of field: user_regs_struct::xcs"]
        [::std::mem::offset_of!(user_regs_struct, xcs) - 52usize];
    ["Offset of field: user_regs_struct::eflags"]
        [::std::mem::offset_of!(user_regs_struct, eflags) - 56usize];
    ["Offset of field: user_regs_struct::esp"]
        [::std::mem::offset_of!(user_regs_struct, esp) - 60usize];
    ["Offset of field: user_regs_struct::xss"]
        [::std::mem::offset_of!(user_regs_struct, xss) - 64usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct user {
    pub regs: user_regs_struct,
    pub u_fpvalid: ::std::os::raw::c_int,
    pub i387: user_fpregs_struct,
    pub u_tsize: ::std::os::raw::c_ulong,
    pub u_dsize: ::std::os::raw::c_ulong,
    pub u_ssize: ::std::os::raw::c_ulong,
    pub start_code: ::std::os::raw::c_ulong,
    pub start_stack: ::std::os::raw::c_ulong,
    pub signal: ::std::os::raw::c_long,
    pub reserved: ::std::os::raw::c_int,
    pub u_ar0: *mut user_regs_struct,
    pub u_fpstate: *mut user_fpregs_struct,
    pub magic: ::std::os::raw::c_ulong,
    pub u_comm: [::std::os::raw::c_char; 32usize],
    pub u_debugreg: [::std::os::raw::c_int; 8usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of user"][::std::mem::size_of::<user>() - 284usize];
    ["Alignment of user"][::std::mem::align_of::<user>() - 4usize];
    ["Offset of field: user::regs"][::std::mem::offset_of!(user, regs) - 0usize];
    ["Offset of field: user::u_fpvalid"][::std::mem::offset_of!(user, u_fpvalid) - 68usize];
    ["Offset of field: user::i387"][::std::mem::offset_of!(user, i387) - 72usize];
    ["Offset of field: user::u_tsize"][::std::mem::offset_of!(user, u_tsize) - 180usize];
    ["Offset of field: user::u_dsize"][::std::mem::offset_of!(user, u_dsize) - 184usize];
    ["Offset of field: user::u_ssize"][::std::mem::offset_of!(user, u_ssize) - 188usize];
    ["Offset of field: user::start_code"][::std::mem::offset_of!(user, start_code) - 192usize];
    ["Offset of field: user::start_stack"][::std::mem::offset_of!(user, start_stack) - 196usize];
    ["Offset of field: user::signal"][::std::mem::offset_of!(user, signal) - 200usize];
    ["Offset of field: user::reserved"][::std::mem::offset_of!(user, reserved) - 204usize];
    ["Offset of field: user::u_ar0"][::std::mem::offset_of!(user, u_ar0) - 208usize];
    ["Offset of field: user::u_fpstate"][::std::mem::offset_of!(user, u_fpstate) - 212usize];
    ["Offset of field: user::magic"][::std::mem::offset_of!(user, magic) - 216usize];
    ["Offset of field: user::u_comm"][::std::mem::offset_of!(user, u_comm) - 220usize];
    ["Offset of field: user::u_debugreg"][::std::mem::offset_of!(user, u_debugreg) - 252usize];
};
pub const REG_GS: _bindgen_ty_22 = 0;
pub const REG_FS: _bindgen_ty_22 = 1;
pub const REG_ES: _bindgen_ty_22 = 2;
pub const REG_DS: _bindgen_ty_22 = 3;
pub const REG_EDI: _bindgen_ty_22 = 4;
pub const REG_ESI: _bindgen_ty_22 = 5;
pub const REG_EBP: _bindgen_ty_22 = 6;
pub const REG_ESP: _bindgen_ty_22 = 7;
pub const REG_EBX: _bindgen_ty_22 = 8;
pub const REG_EDX: _bindgen_ty_22 = 9;
pub const REG_ECX: _bindgen_ty_22 = 10;
pub const REG_EAX: _bindgen_ty_22 = 11;
pub const REG_TRAPNO: _bindgen_ty_22 = 12;
pub const REG_ERR: _bindgen_ty_22 = 13;
pub const REG_EIP: _bindgen_ty_22 = 14;
pub const REG_CS: _bindgen_ty_22 = 15;
pub const REG_EFL: _bindgen_ty_22 = 16;
pub const REG_UESP: _bindgen_ty_22 = 17;
pub const REG_SS: _bindgen_ty_22 = 18;
pub const NGREG: _bindgen_ty_22 = 19;
pub type _bindgen_ty_22 = ::std::os::raw::c_uint;
pub type greg_t = ::std::os::raw::c_int;
pub type gregset_t = [greg_t; 19usize];
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _libc_fpreg {
    pub significand: [::std::os::raw::c_ushort; 4usize],
    pub exponent: ::std::os::raw::c_ushort,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _libc_fpreg"][::std::mem::size_of::<_libc_fpreg>() - 10usize];
    ["Alignment of _libc_fpreg"][::std::mem::align_of::<_libc_fpreg>() - 2usize];
    ["Offset of field: _libc_fpreg::significand"]
        [::std::mem::offset_of!(_libc_fpreg, significand) - 0usize];
    ["Offset of field: _libc_fpreg::exponent"]
        [::std::mem::offset_of!(_libc_fpreg, exponent) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _libc_fpstate {
    pub cw: ::std::os::raw::c_ulong,
    pub sw: ::std::os::raw::c_ulong,
    pub tag: ::std::os::raw::c_ulong,
    pub ipoff: ::std::os::raw::c_ulong,
    pub cssel: ::std::os::raw::c_ulong,
    pub dataoff: ::std::os::raw::c_ulong,
    pub datasel: ::std::os::raw::c_ulong,
    pub _st: [_libc_fpreg; 8usize],
    pub status: ::std::os::raw::c_ulong,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of _libc_fpstate"][::std::mem::size_of::<_libc_fpstate>() - 112usize];
    ["Alignment of _libc_fpstate"][::std::mem::align_of::<_libc_fpstate>() - 4usize];
    ["Offset of field: _libc_fpstate::cw"][::std::mem::offset_of!(_libc_fpstate, cw) - 0usize];
    ["Offset of field: _libc_fpstate::sw"][::std::mem::offset_of!(_libc_fpstate, sw) - 4usize];
    ["Offset of field: _libc_fpstate::tag"][::std::mem::offset_of!(_libc_fpstate, tag) - 8usize];
    ["Offset of field: _libc_fpstate::ipoff"]
        [::std::mem::offset_of!(_libc_fpstate, ipoff) - 12usize];
    ["Offset of field: _libc_fpstate::cssel"]
        [::std::mem::offset_of!(_libc_fpstate, cssel) - 16usize];
    ["Offset of field: _libc_fpstate::dataoff"]
        [::std::mem::offset_of!(_libc_fpstate, dataoff) - 20usize];
    ["Offset of field: _libc_fpstate::datasel"]
        [::std::mem::offset_of!(_libc_fpstate, datasel) - 24usize];
    ["Offset of field: _libc_fpstate::_st"][::std::mem::offset_of!(_libc_fpstate, _st) - 28usize];
    ["Offset of field: _libc_fpstate::status"]
        [::std::mem::offset_of!(_libc_fpstate, status) - 108usize];
};
pub type fpregset_t = *mut _libc_fpstate;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct mcontext_t {
    pub gregs: gregset_t,
    pub fpregs: fpregset_t,
    pub oldmask: ::std::os::raw::c_ulong,
    pub cr2: ::std::os::raw::c_ulong,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of mcontext_t"][::std::mem::size_of::<mcontext_t>() - 88usize];
    ["Alignment of mcontext_t"][::std::mem::align_of::<mcontext_t>() - 4usize];
    ["Offset of field: mcontext_t::gregs"][::std::mem::offset_of!(mcontext_t, gregs) - 0usize];
    ["Offset of field: mcontext_t::fpregs"][::std::mem::offset_of!(mcontext_t, fpregs) - 76usize];
    ["Offset of field: mcontext_t::oldmask"][::std::mem::offset_of!(mcontext_t, oldmask) - 80usize];
    ["Offset of field: mcontext_t::cr2"][::std::mem::offset_of!(mcontext_t, cr2) - 84usize];
};
#[repr(C)]
#[derive(Copy, Clone)]
pub struct ucontext {
    pub uc_flags: ::std::os::raw::c_ulong,
    pub uc_link: *mut ucontext,
    pub uc_stack: stack_t,
    pub uc_mcontext: mcontext_t,
    pub __bindgen_anon_1: ucontext__bindgen_ty_1,
    pub __fpregs_mem: _libc_fpstate,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union ucontext__bindgen_ty_1 {
    pub __bindgen_anon_1: ucontext__bindgen_ty_1__bindgen_ty_1,
    pub uc_sigmask64: sigset64_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct ucontext__bindgen_ty_1__bindgen_ty_1 {
    pub uc_sigmask: sigset_t,
    pub __padding_rt_sigset: u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of ucontext__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::size_of::<ucontext__bindgen_ty_1__bindgen_ty_1>() - 8usize];
    ["Alignment of ucontext__bindgen_ty_1__bindgen_ty_1"]
        [::std::mem::align_of::<ucontext__bindgen_ty_1__bindgen_ty_1>() - 4usize];
    ["Offset of field: ucontext__bindgen_ty_1__bindgen_ty_1::uc_sigmask"]
        [::std::mem::offset_of!(ucontext__bindgen_ty_1__bindgen_ty_1, uc_sigmask) - 0usize];
    ["Offset of field: ucontext__bindgen_ty_1__bindgen_ty_1::__padding_rt_sigset"][::std::mem::offset_of!(
        ucontext__bindgen_ty_1__bindgen_ty_1,
        __padding_rt_sigset
    ) - 4usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of ucontext__bindgen_ty_1"][::std::mem::size_of::<ucontext__bindgen_ty_1>() - 8usize];
    ["Alignment of ucontext__bindgen_ty_1"]
        [::std::mem::align_of::<ucontext__bindgen_ty_1>() - 4usize];
    ["Offset of field: ucontext__bindgen_ty_1::uc_sigmask64"]
        [::std::mem::offset_of!(ucontext__bindgen_ty_1, uc_sigmask64) - 0usize];
};
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of ucontext"][::std::mem::size_of::<ucontext>() - 228usize];
    ["Alignment of ucontext"][::std::mem::align_of::<ucontext>() - 4usize];
    ["Offset of field: ucontext::uc_flags"][::std::mem::offset_of!(ucontext, uc_flags) - 0usize];
    ["Offset of field: ucontext::uc_link"][::std::mem::offset_of!(ucontext, uc_link) - 4usize];
    ["Offset of field: ucontext::uc_stack"][::std::mem::offset_of!(ucontext, uc_stack) - 8usize];
    ["Offset of field: ucontext::uc_mcontext"]
        [::std::mem::offset_of!(ucontext, uc_mcontext) - 20usize];
    ["Offset of field: ucontext::__fpregs_mem"]
        [::std::mem::offset_of!(ucontext, __fpregs_mem) - 116usize];
};
pub type ucontext_t = ucontext;
unsafe extern "C" {
    pub fn __libc_current_sigrtmin() -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn __libc_current_sigrtmax() -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub static sys_siglist: [*const ::std::os::raw::c_char; 65usize];
}
unsafe extern "C" {
    pub static sys_signame: [*const ::std::os::raw::c_char; 65usize];
}
unsafe extern "C" {
    pub fn sigaction(
        __signal: ::std::os::raw::c_int,
        __new_action: *const sigaction,
        __old_action: *mut sigaction,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn siginterrupt(
        __signal: ::std::os::raw::c_int,
        __flag: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn signal(__signal: ::std::os::raw::c_int, __handler: sighandler_t) -> sighandler_t;
}
unsafe extern "C" {
    pub fn sigaddset(
        __set: *mut sigset_t,
        __signal: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigdelset(
        __set: *mut sigset_t,
        __signal: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigemptyset(__set: *mut sigset_t) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigfillset(__set: *mut sigset_t) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigismember(
        __set: *const sigset_t,
        __signal: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigpending(__set: *mut sigset_t) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigprocmask(
        __how: ::std::os::raw::c_int,
        __new_set: *const sigset_t,
        __old_set: *mut sigset_t,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigsuspend(__mask: *const sigset_t) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigwait(
        __set: *const sigset_t,
        __signal: *mut ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn raise(__signal: ::std::os::raw::c_int) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn kill(__pid: pid_t, __signal: ::std::os::raw::c_int) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn killpg(
        __pgrp: ::std::os::raw::c_int,
        __signal: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn tgkill(
        __tgid: ::std::os::raw::c_int,
        __tid: ::std::os::raw::c_int,
        __signal: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sigaltstack(
        __new_signal_stack: *const stack_t,
        __old_signal_stack: *mut stack_t,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn psiginfo(__info: *const siginfo_t, __msg: *const ::std::os::raw::c_char);
}
unsafe extern "C" {
    pub fn psignal(__signal: ::std::os::raw::c_int, __msg: *const ::std::os::raw::c_char);
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __kernel_timespec {
    pub tv_sec: __kernel_time64_t,
    pub tv_nsec: ::std::os::raw::c_longlong,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_timespec"][::std::mem::size_of::<__kernel_timespec>() - 16usize];
    ["Alignment of __kernel_timespec"][::std::mem::align_of::<__kernel_timespec>() - 4usize];
    ["Offset of field: __kernel_timespec::tv_sec"]
        [::std::mem::offset_of!(__kernel_timespec, tv_sec) - 0usize];
    ["Offset of field: __kernel_timespec::tv_nsec"]
        [::std::mem::offset_of!(__kernel_timespec, tv_nsec) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __kernel_itimerspec {
    pub it_interval: __kernel_timespec,
    pub it_value: __kernel_timespec,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_itimerspec"][::std::mem::size_of::<__kernel_itimerspec>() - 32usize];
    ["Alignment of __kernel_itimerspec"][::std::mem::align_of::<__kernel_itimerspec>() - 4usize];
    ["Offset of field: __kernel_itimerspec::it_interval"]
        [::std::mem::offset_of!(__kernel_itimerspec, it_interval) - 0usize];
    ["Offset of field: __kernel_itimerspec::it_value"]
        [::std::mem::offset_of!(__kernel_itimerspec, it_value) - 16usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __kernel_old_timespec {
    pub tv_sec: __kernel_old_time_t,
    pub tv_nsec: ::std::os::raw::c_long,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_old_timespec"][::std::mem::size_of::<__kernel_old_timespec>() - 8usize];
    ["Alignment of __kernel_old_timespec"]
        [::std::mem::align_of::<__kernel_old_timespec>() - 4usize];
    ["Offset of field: __kernel_old_timespec::tv_sec"]
        [::std::mem::offset_of!(__kernel_old_timespec, tv_sec) - 0usize];
    ["Offset of field: __kernel_old_timespec::tv_nsec"]
        [::std::mem::offset_of!(__kernel_old_timespec, tv_nsec) - 4usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __kernel_sock_timeval {
    pub tv_sec: __s64,
    pub tv_usec: __s64,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __kernel_sock_timeval"][::std::mem::size_of::<__kernel_sock_timeval>() - 16usize];
    ["Alignment of __kernel_sock_timeval"]
        [::std::mem::align_of::<__kernel_sock_timeval>() - 4usize];
    ["Offset of field: __kernel_sock_timeval::tv_sec"]
        [::std::mem::offset_of!(__kernel_sock_timeval, tv_sec) - 0usize];
    ["Offset of field: __kernel_sock_timeval::tv_usec"]
        [::std::mem::offset_of!(__kernel_sock_timeval, tv_usec) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct timeval {
    pub tv_sec: __kernel_old_time_t,
    pub tv_usec: __kernel_suseconds_t,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of timeval"][::std::mem::size_of::<timeval>() - 8usize];
    ["Alignment of timeval"][::std::mem::align_of::<timeval>() - 4usize];
    ["Offset of field: timeval::tv_sec"][::std::mem::offset_of!(timeval, tv_sec) - 0usize];
    ["Offset of field: timeval::tv_usec"][::std::mem::offset_of!(timeval, tv_usec) - 4usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct itimerspec {
    pub it_interval: timespec,
    pub it_value: timespec,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of itimerspec"][::std::mem::size_of::<itimerspec>() - 16usize];
    ["Alignment of itimerspec"][::std::mem::align_of::<itimerspec>() - 4usize];
    ["Offset of field: itimerspec::it_interval"]
        [::std::mem::offset_of!(itimerspec, it_interval) - 0usize];
    ["Offset of field: itimerspec::it_value"]
        [::std::mem::offset_of!(itimerspec, it_value) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct itimerval {
    pub it_interval: timeval,
    pub it_value: timeval,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of itimerval"][::std::mem::size_of::<itimerval>() - 16usize];
    ["Alignment of itimerval"][::std::mem::align_of::<itimerval>() - 4usize];
    ["Offset of field: itimerval::it_interval"]
        [::std::mem::offset_of!(itimerval, it_interval) - 0usize];
    ["Offset of field: itimerval::it_value"][::std::mem::offset_of!(itimerval, it_value) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct timezone {
    pub tz_minuteswest: ::std::os::raw::c_int,
    pub tz_dsttime: ::std::os::raw::c_int,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of timezone"][::std::mem::size_of::<timezone>() - 8usize];
    ["Alignment of timezone"][::std::mem::align_of::<timezone>() - 4usize];
    ["Offset of field: timezone::tz_minuteswest"]
        [::std::mem::offset_of!(timezone, tz_minuteswest) - 0usize];
    ["Offset of field: timezone::tz_dsttime"]
        [::std::mem::offset_of!(timezone, tz_dsttime) - 4usize];
};
pub type fd_mask = ::std::os::raw::c_ulong;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct fd_set {
    pub fds_bits: [fd_mask; 32usize],
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of fd_set"][::std::mem::size_of::<fd_set>() - 128usize];
    ["Alignment of fd_set"][::std::mem::align_of::<fd_set>() - 4usize];
    ["Offset of field: fd_set::fds_bits"][::std::mem::offset_of!(fd_set, fds_bits) - 0usize];
};
unsafe extern "C" {
    pub fn __FD_CLR_chk(arg1: ::std::os::raw::c_int, arg2: *mut fd_set, arg3: usize);
}
unsafe extern "C" {
    pub fn __FD_SET_chk(arg1: ::std::os::raw::c_int, arg2: *mut fd_set, arg3: usize);
}
unsafe extern "C" {
    pub fn __FD_ISSET_chk(
        arg1: ::std::os::raw::c_int,
        arg2: *const fd_set,
        arg3: usize,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn select(
        __max_fd_plus_one: ::std::os::raw::c_int,
        __read_fds: *mut fd_set,
        __write_fds: *mut fd_set,
        __exception_fds: *mut fd_set,
        __timeout: *mut timeval,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn pselect(
        __max_fd_plus_one: ::std::os::raw::c_int,
        __read_fds: *mut fd_set,
        __write_fds: *mut fd_set,
        __exception_fds: *mut fd_set,
        __timeout: *const timespec,
        __mask: *const sigset_t,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn gettimeofday(__tv: *mut timeval, __tz: *mut timezone) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn settimeofday(__tv: *const timeval, __tz: *const timezone) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn getitimer(
        __which: ::std::os::raw::c_int,
        __current_value: *mut itimerval,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn setitimer(
        __which: ::std::os::raw::c_int,
        __new_value: *const itimerval,
        __old_value: *mut itimerval,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn utimes(
        __path: *const ::std::os::raw::c_char,
        __times: *const timeval,
    ) -> ::std::os::raw::c_int;
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __locale_t {
    _unused: [u8; 0],
}
pub type locale_t = *mut __locale_t;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __timezone_t {
    _unused: [u8; 0],
}
pub type timezone_t = *mut __timezone_t;
unsafe extern "C" {
    pub static mut tzname: [*mut ::std::os::raw::c_char; 0usize];
}
unsafe extern "C" {
    pub static mut daylight: ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub static mut timezone: ::std::os::raw::c_long;
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct tm {
    pub tm_sec: ::std::os::raw::c_int,
    pub tm_min: ::std::os::raw::c_int,
    pub tm_hour: ::std::os::raw::c_int,
    pub tm_mday: ::std::os::raw::c_int,
    pub tm_mon: ::std::os::raw::c_int,
    pub tm_year: ::std::os::raw::c_int,
    pub tm_wday: ::std::os::raw::c_int,
    pub tm_yday: ::std::os::raw::c_int,
    pub tm_isdst: ::std::os::raw::c_int,
    pub tm_gmtoff: ::std::os::raw::c_long,
    pub tm_zone: *const ::std::os::raw::c_char,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of tm"][::std::mem::size_of::<tm>() - 44usize];
    ["Alignment of tm"][::std::mem::align_of::<tm>() - 4usize];
    ["Offset of field: tm::tm_sec"][::std::mem::offset_of!(tm, tm_sec) - 0usize];
    ["Offset of field: tm::tm_min"][::std::mem::offset_of!(tm, tm_min) - 4usize];
    ["Offset of field: tm::tm_hour"][::std::mem::offset_of!(tm, tm_hour) - 8usize];
    ["Offset of field: tm::tm_mday"][::std::mem::offset_of!(tm, tm_mday) - 12usize];
    ["Offset of field: tm::tm_mon"][::std::mem::offset_of!(tm, tm_mon) - 16usize];
    ["Offset of field: tm::tm_year"][::std::mem::offset_of!(tm, tm_year) - 20usize];
    ["Offset of field: tm::tm_wday"][::std::mem::offset_of!(tm, tm_wday) - 24usize];
    ["Offset of field: tm::tm_yday"][::std::mem::offset_of!(tm, tm_yday) - 28usize];
    ["Offset of field: tm::tm_isdst"][::std::mem::offset_of!(tm, tm_isdst) - 32usize];
    ["Offset of field: tm::tm_gmtoff"][::std::mem::offset_of!(tm, tm_gmtoff) - 36usize];
    ["Offset of field: tm::tm_zone"][::std::mem::offset_of!(tm, tm_zone) - 40usize];
};
unsafe extern "C" {
    pub fn time(__t: *mut time_t) -> time_t;
}
unsafe extern "C" {
    pub fn nanosleep(
        __duration: *const timespec,
        __remainder: *mut timespec,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn asctime(__tm: *const tm) -> *mut ::std::os::raw::c_char;
}
unsafe extern "C" {
    pub fn asctime_r(
        __tm: *const tm,
        __buf: *mut ::std::os::raw::c_char,
    ) -> *mut ::std::os::raw::c_char;
}
unsafe extern "C" {
    pub fn difftime(__lhs: time_t, __rhs: time_t) -> f64;
}
unsafe extern "C" {
    pub fn mktime(__tm: *mut tm) -> time_t;
}
unsafe extern "C" {
    pub fn localtime(__t: *const time_t) -> *mut tm;
}
unsafe extern "C" {
    pub fn localtime_r(__t: *const time_t, __tm: *mut tm) -> *mut tm;
}
unsafe extern "C" {
    pub fn timelocal(__tm: *mut tm) -> time_t;
}
unsafe extern "C" {
    pub fn gmtime(__t: *const time_t) -> *mut tm;
}
unsafe extern "C" {
    pub fn gmtime_r(__t: *const time_t, __tm: *mut tm) -> *mut tm;
}
unsafe extern "C" {
    pub fn timegm(__tm: *mut tm) -> time_t;
}
unsafe extern "C" {
    pub fn strptime(
        __s: *const ::std::os::raw::c_char,
        __fmt: *const ::std::os::raw::c_char,
        __tm: *mut tm,
    ) -> *mut ::std::os::raw::c_char;
}
unsafe extern "C" {
    pub fn strftime(
        __buf: *mut ::std::os::raw::c_char,
        __n: usize,
        __fmt: *const ::std::os::raw::c_char,
        __tm: *const tm,
    ) -> usize;
}
unsafe extern "C" {
    pub fn strftime_l(
        __buf: *mut ::std::os::raw::c_char,
        __n: usize,
        __fmt: *const ::std::os::raw::c_char,
        __tm: *const tm,
        __l: locale_t,
    ) -> usize;
}
unsafe extern "C" {
    pub fn ctime(__t: *const time_t) -> *mut ::std::os::raw::c_char;
}
unsafe extern "C" {
    pub fn ctime_r(
        __t: *const time_t,
        __buf: *mut ::std::os::raw::c_char,
    ) -> *mut ::std::os::raw::c_char;
}
unsafe extern "C" {
    pub fn tzset();
}
unsafe extern "C" {
    pub fn clock() -> clock_t;
}
unsafe extern "C" {
    pub fn clock_getres(__clock: clockid_t, __resolution: *mut timespec) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn clock_gettime(__clock: clockid_t, __ts: *mut timespec) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn clock_nanosleep(
        __clock: clockid_t,
        __flags: ::std::os::raw::c_int,
        __time: *const timespec,
        __remainder: *mut timespec,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn clock_settime(__clock: clockid_t, __ts: *const timespec) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn timer_create(
        __clock: clockid_t,
        __event: *mut sigevent,
        __timer_ptr: *mut timer_t,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn timer_delete(__timer: timer_t) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn timer_settime(
        __timer: timer_t,
        __flags: ::std::os::raw::c_int,
        __new_value: *const itimerspec,
        __old_value: *mut itimerspec,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn timer_gettime(_timer: timer_t, __ts: *mut itimerspec) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn timer_getoverrun(__timer: timer_t) -> ::std::os::raw::c_int;
}
pub type nfds_t = ::std::os::raw::c_uint;
unsafe extern "C" {
    pub fn poll(
        __fds: *mut pollfd,
        __count: nfds_t,
        __timeout_ms: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn ppoll(
        __fds: *mut pollfd,
        __count: nfds_t,
        __timeout: *const timespec,
        __mask: *const sigset_t,
    ) -> ::std::os::raw::c_int;
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone)]
pub struct clone_args {
    pub flags: __u64,
    pub pidfd: __u64,
    pub child_tid: __u64,
    pub parent_tid: __u64,
    pub exit_signal: __u64,
    pub stack: __u64,
    pub stack_size: __u64,
    pub tls: __u64,
    pub set_tid: __u64,
    pub set_tid_size: __u64,
    pub cgroup: __u64,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of clone_args"][::std::mem::size_of::<clone_args>() - 88usize];
    ["Alignment of clone_args"][::std::mem::align_of::<clone_args>() - 8usize];
    ["Offset of field: clone_args::flags"][::std::mem::offset_of!(clone_args, flags) - 0usize];
    ["Offset of field: clone_args::pidfd"][::std::mem::offset_of!(clone_args, pidfd) - 8usize];
    ["Offset of field: clone_args::child_tid"]
        [::std::mem::offset_of!(clone_args, child_tid) - 16usize];
    ["Offset of field: clone_args::parent_tid"]
        [::std::mem::offset_of!(clone_args, parent_tid) - 24usize];
    ["Offset of field: clone_args::exit_signal"]
        [::std::mem::offset_of!(clone_args, exit_signal) - 32usize];
    ["Offset of field: clone_args::stack"][::std::mem::offset_of!(clone_args, stack) - 40usize];
    ["Offset of field: clone_args::stack_size"]
        [::std::mem::offset_of!(clone_args, stack_size) - 48usize];
    ["Offset of field: clone_args::tls"][::std::mem::offset_of!(clone_args, tls) - 56usize];
    ["Offset of field: clone_args::set_tid"][::std::mem::offset_of!(clone_args, set_tid) - 64usize];
    ["Offset of field: clone_args::set_tid_size"]
        [::std::mem::offset_of!(clone_args, set_tid_size) - 72usize];
    ["Offset of field: clone_args::cgroup"][::std::mem::offset_of!(clone_args, cgroup) - 80usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sched_attr {
    pub size: __u32,
    pub sched_policy: __u32,
    pub sched_flags: __u64,
    pub sched_nice: __s32,
    pub sched_priority: __u32,
    pub sched_runtime: __u64,
    pub sched_deadline: __u64,
    pub sched_period: __u64,
    pub sched_util_min: __u32,
    pub sched_util_max: __u32,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sched_attr"][::std::mem::size_of::<sched_attr>() - 56usize];
    ["Alignment of sched_attr"][::std::mem::align_of::<sched_attr>() - 4usize];
    ["Offset of field: sched_attr::size"][::std::mem::offset_of!(sched_attr, size) - 0usize];
    ["Offset of field: sched_attr::sched_policy"]
        [::std::mem::offset_of!(sched_attr, sched_policy) - 4usize];
    ["Offset of field: sched_attr::sched_flags"]
        [::std::mem::offset_of!(sched_attr, sched_flags) - 8usize];
    ["Offset of field: sched_attr::sched_nice"]
        [::std::mem::offset_of!(sched_attr, sched_nice) - 16usize];
    ["Offset of field: sched_attr::sched_priority"]
        [::std::mem::offset_of!(sched_attr, sched_priority) - 20usize];
    ["Offset of field: sched_attr::sched_runtime"]
        [::std::mem::offset_of!(sched_attr, sched_runtime) - 24usize];
    ["Offset of field: sched_attr::sched_deadline"]
        [::std::mem::offset_of!(sched_attr, sched_deadline) - 32usize];
    ["Offset of field: sched_attr::sched_period"]
        [::std::mem::offset_of!(sched_attr, sched_period) - 40usize];
    ["Offset of field: sched_attr::sched_util_min"]
        [::std::mem::offset_of!(sched_attr, sched_util_min) - 48usize];
    ["Offset of field: sched_attr::sched_util_max"]
        [::std::mem::offset_of!(sched_attr, sched_util_max) - 52usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct sched_param {
    pub sched_priority: ::std::os::raw::c_int,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of sched_param"][::std::mem::size_of::<sched_param>() - 4usize];
    ["Alignment of sched_param"][::std::mem::align_of::<sched_param>() - 4usize];
    ["Offset of field: sched_param::sched_priority"]
        [::std::mem::offset_of!(sched_param, sched_priority) - 0usize];
};
unsafe extern "C" {
    pub fn sched_setscheduler(
        __pid: pid_t,
        __policy: ::std::os::raw::c_int,
        __param: *const sched_param,
    ) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_getscheduler(__pid: pid_t) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_yield() -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_get_priority_max(__policy: ::std::os::raw::c_int) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_get_priority_min(__policy: ::std::os::raw::c_int) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_setparam(__pid: pid_t, __param: *const sched_param) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_getparam(__pid: pid_t, __param: *mut sched_param) -> ::std::os::raw::c_int;
}
unsafe extern "C" {
    pub fn sched_rr_get_interval(__pid: pid_t, __quantum: *mut timespec) -> ::std::os::raw::c_int;
}
pub const PTHREAD_MUTEX_NORMAL: _bindgen_ty_23 = 0;
pub const PTHREAD_MUTEX_RECURSIVE: _bindgen_ty_23 = 1;
pub const PTHREAD_MUTEX_ERRORCHECK: _bindgen_ty_23 = 2;
pub const PTHREAD_MUTEX_ERRORCHECK_NP: _bindgen_ty_23 = 2;
pub const PTHREAD_MUTEX_RECURSIVE_NP: _bindgen_ty_23 = 1;
pub const PTHREAD_MUTEX_DEFAULT: _bindgen_ty_23 = 0;
pub type _bindgen_ty_23 = ::std::os::raw::c_uint;
pub const PTHREAD_RWLOCK_PREFER_READER_NP: _bindgen_ty_24 = 0;
pub const PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP: _bindgen_ty_24 = 1;
pub type _bindgen_ty_24 = ::std::os::raw::c_uint;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct __pthread_cleanup_t {
    pub __cleanup_prev: *mut __pthread_cleanup_t,
    pub __cleanup_routine:
        ::std::option::Option<unsafe extern "C" fn(arg1: *mut ::std::os::raw::c_void)>,
    pub __cleanup_arg: *mut ::std::os::raw::c_void,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of __pthread_cleanup_t"][::std::mem::size_of::<__pthread_cleanup_t>() - 12usize];
    ["Alignment of __pthread_cleanup_t"][::std::mem::align_of::<__pthread_cleanup_t>() - 4usize];
    ["Offset of field: __pthread_cleanup_t::__cleanup_prev"]
        [::std::mem::offset_of!(__pthread_cleanup_t, __cleanup_prev) - 0usize];
    ["Offset of field: __pthread_cleanup_t::__cleanup_routine"]
        [::std::mem::offset_of!(__pthread_cleanup_t, __cleanup_routine) - 4usize];
    ["Offset of field: __pthread_cleanup_t::__cleanup_arg"]
        [::std::mem::offset_of!(__pthread_cleanup_t, __cleanup_arg) - 8usize];
};
unsafe extern "C" {
    pub fn __pthread_cleanup_push(
        c: *mut __pthread_cleanup_t,
        arg1: ::std::option::Option<unsafe extern "C" fn(arg1: *mut ::std::os::raw::c_void)>,
        arg2: *mut ::std::os::raw::c_void,
    );
}
unsafe extern "C" {
    pub fn __pthread_cleanup_pop(arg1: *mut __pthread_cleanup_t, arg2: ::std::os::raw::c_int);
}
#[doc = " Data associated with an ALooper fd that will be returned as the \"outData\"\n when that source has data ready."]
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct android_poll_source {
    #[doc = " The identifier of this source.  May be LOOPER_ID_MAIN or\n LOOPER_ID_INPUT."]
    pub id: i32,
    #[doc = " The android_app this ident is associated with."]
    pub app: *mut android_app,
    #[doc = " Function to call to perform the standard processing of data from\n this source."]
    pub process: ::std::option::Option<
        unsafe extern "C" fn(app: *mut android_app, source: *mut android_poll_source),
    >,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of android_poll_source"][::std::mem::size_of::<android_poll_source>() - 12usize];
    ["Alignment of android_poll_source"][::std::mem::align_of::<android_poll_source>() - 4usize];
    ["Offset of field: android_poll_source::id"]
        [::std::mem::offset_of!(android_poll_source, id) - 0usize];
    ["Offset of field: android_poll_source::app"]
        [::std::mem::offset_of!(android_poll_source, app) - 4usize];
    ["Offset of field: android_poll_source::process"]
        [::std::mem::offset_of!(android_poll_source, process) - 8usize];
};
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct android_input_buffer {
    #[doc = " Pointer to a read-only array of GameActivityMotionEvent.\n Only the first motionEventsCount events are valid."]
    pub motionEvents: *mut GameActivityMotionEvent,
    #[doc = " The number of valid motion events in `motionEvents`."]
    pub motionEventsCount: u64,
    #[doc = " The size of the `motionEvents` buffer."]
    pub motionEventsBufferSize: u64,
    #[doc = " Pointer to a read-only array of GameActivityKeyEvent.\n Only the first keyEventsCount events are valid."]
    pub keyEvents: *mut GameActivityKeyEvent,
    #[doc = " The number of valid \"Key\" events in `keyEvents`."]
    pub keyEventsCount: u64,
    #[doc = " The size of the `keyEvents` buffer."]
    pub keyEventsBufferSize: u64,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of android_input_buffer"][::std::mem::size_of::<android_input_buffer>() - 40usize];
    ["Alignment of android_input_buffer"][::std::mem::align_of::<android_input_buffer>() - 4usize];
    ["Offset of field: android_input_buffer::motionEvents"]
        [::std::mem::offset_of!(android_input_buffer, motionEvents) - 0usize];
    ["Offset of field: android_input_buffer::motionEventsCount"]
        [::std::mem::offset_of!(android_input_buffer, motionEventsCount) - 4usize];
    ["Offset of field: android_input_buffer::motionEventsBufferSize"]
        [::std::mem::offset_of!(android_input_buffer, motionEventsBufferSize) - 12usize];
    ["Offset of field: android_input_buffer::keyEvents"]
        [::std::mem::offset_of!(android_input_buffer, keyEvents) - 20usize];
    ["Offset of field: android_input_buffer::keyEventsCount"]
        [::std::mem::offset_of!(android_input_buffer, keyEventsCount) - 24usize];
    ["Offset of field: android_input_buffer::keyEventsBufferSize"]
        [::std::mem::offset_of!(android_input_buffer, keyEventsBufferSize) - 32usize];
};
#[doc = " Function pointer declaration for the filtering of key events.\n A function with this signature should be passed to\n android_app_set_key_event_filter and return false for any events that should\n not be handled by android_native_app_glue. These events will be handled by\n the system instead."]
pub type android_key_event_filter =
    ::std::option::Option<unsafe extern "C" fn(arg1: *const GameActivityKeyEvent) -> bool>;
#[doc = " Function pointer definition for the filtering of motion events.\n A function with this signature should be passed to\n android_app_set_motion_event_filter and return false for any events that\n should not be handled by android_native_app_glue. These events will be\n handled by the system instead."]
pub type android_motion_event_filter =
    ::std::option::Option<unsafe extern "C" fn(arg1: *const GameActivityMotionEvent) -> bool>;
#[doc = " The GameActivity interface provided by <game-activity/GameActivity.h>\n is based on a set of application-provided callbacks that will be called\n by the Activity's main thread when certain events occur.\n\n This means that each one of this callbacks _should_ _not_ block, or they\n risk having the system force-close the application. This programming\n model is direct, lightweight, but constraining.\n\n The 'android_native_app_glue' static library is used to provide a different\n execution model where the application can implement its own main event\n loop in a different thread instead. Here's how it works:\n\n 1/ The application must provide a function named \"android_main()\" that\n    will be called when the activity is created, in a new thread that is\n    distinct from the activity's main thread.\n\n 2/ android_main() receives a pointer to a valid \"android_app\" structure\n    that contains references to other important objects, e.g. the\n    GameActivity obejct instance the application is running in.\n\n 3/ the \"android_app\" object holds an ALooper instance that already\n    listens to activity lifecycle events (e.g. \"pause\", \"resume\").\n    See APP_CMD_XXX declarations below.\n\n    This corresponds to an ALooper identifier returned by\n    ALooper_pollOnce with value LOOPER_ID_MAIN.\n\n    Your application can use the same ALooper to listen to additional\n    file-descriptors.  They can either be callback based, or with return\n    identifiers starting with LOOPER_ID_USER.\n\n 4/ Whenever you receive a LOOPER_ID_MAIN event,\n    the returned data will point to an android_poll_source structure.  You\n    can call the process() function on it, and fill in android_app->onAppCmd\n    to be called for your own processing of the event.\n\n    Alternatively, you can call the low-level functions to read and process\n    the data directly...  look at the process_cmd() and process_input()\n    implementations in the glue to see how to do this.\n\n See the sample named \"native-activity\" that comes with the NDK with a\n full usage example.  Also look at the documentation of GameActivity."]
#[repr(C)]
pub struct android_app {
    #[doc = " An optional pointer to application-defined state."]
    pub userData: *mut ::std::os::raw::c_void,
    #[doc = " A required callback for processing main app commands (`APP_CMD_*`).\n This is called each frame if there are app commands that need processing."]
    pub onAppCmd: ::std::option::Option<unsafe extern "C" fn(app: *mut android_app, cmd: i32)>,
    #[doc = " The GameActivity object instance that this app is running in."]
    pub activity: *mut GameActivity,
    #[doc = " The current configuration the app is running in."]
    pub config: *mut AConfiguration,
    #[doc = " The last activity saved state, as provided at creation time.\n It is NULL if there was no state.  You can use this as you need; the\n memory will remain around until you call android_app_exec_cmd() for\n APP_CMD_RESUME, at which point it will be freed and savedState set to\n NULL. These variables should only be changed when processing a\n APP_CMD_SAVE_STATE, at which point they will be initialized to NULL and\n you can malloc your state and place the information here.  In that case\n the memory will be freed for you later."]
    pub savedState: *mut ::std::os::raw::c_void,
    #[doc = " The size of the activity saved state. It is 0 if `savedState` is NULL."]
    pub savedStateSize: usize,
    #[doc = " The ALooper associated with the app's thread."]
    pub looper: *mut ALooper,
    #[doc = " The ALooper associated with the app's Java main/UI thread."]
    pub mainLooper: *mut ALooper,
    #[doc = " When non-NULL, this is the window surface that the app can draw in."]
    pub window: *mut ANativeWindow,
    #[doc = " Current content rectangle of the window; this is the area where the\n window's content should be placed to be seen by the user."]
    pub contentRect: ARect,
    #[doc = " Whether the software keyboard is visible or not."]
    pub softwareKeyboardVisible: bool,
    #[doc = " Last editor action. Valid within APP_CMD_SOFTWARE_KB_VIS_CHANGED handler.\n\n Note: the upstream comment above isn't accurate.\n - `APP_CMD_SOFTWARE_KB_VIS_CHANGED` is associated with `softwareKeyboardVisible`\n   changes, not `editorAction`.\n - `APP_CMD_EDITOR_ACTION` is associated with this state but unlike for\n   `window` state there's no synchonization that blocks the Java main\n   thread, so we can't say that this is only valid within the `APP_CMD_` handler."]
    pub editorAction: ::std::os::raw::c_int,
    #[doc = " true when editorAction has been set"]
    pub pendingEditorAction: bool,
    #[doc = " Current state of the app's activity.  May be either APP_CMD_START,\n APP_CMD_RESUME, APP_CMD_PAUSE, or APP_CMD_STOP."]
    pub activityState: ::std::os::raw::c_int,
    #[doc = " This is non-zero when the application's GameActivity is being\n destroyed and waiting for the app thread to complete."]
    pub destroyRequested: ::std::os::raw::c_int,
    #[doc = " This is used for buffering input from GameActivity. Once ready, the\n application thread switches the buffers and processes what was\n accumulated."]
    pub inputBuffers: [android_input_buffer; 2usize],
    pub currentInputBuffer: ::std::os::raw::c_int,
    #[doc = " 0 if no text input event is outstanding, 1 if it is.\n Use `GameActivity_getTextInputState` to get information\n about the text entered by the user."]
    pub textInputState: ::std::os::raw::c_int,
    #[doc = " @cond INTERNAL"]
    pub mutex: pthread_mutex_t,
    pub cond: pthread_cond_t,
    pub msgread: ::std::os::raw::c_int,
    pub msgwrite: ::std::os::raw::c_int,
    pub thread: pthread_t,
    pub cmdPollSource: android_poll_source,
    pub running: ::std::os::raw::c_int,
    pub stateSaved: ::std::os::raw::c_int,
    pub destroyed: ::std::os::raw::c_int,
    pub redrawNeeded: ::std::os::raw::c_int,
    pub pendingWindow: *mut ANativeWindow,
    pub pendingContentRect: ARect,
    pub keyEventFilter: android_key_event_filter,
    pub motionEventFilter: android_motion_event_filter,
    pub inputAvailableWakeUp: bool,
    pub inputSwapPending: bool,
}
#[allow(clippy::unnecessary_operation, clippy::identity_op)]
const _: () = {
    ["Size of android_app"][::std::mem::size_of::<android_app>() - 240usize];
    ["Alignment of android_app"][::std::mem::align_of::<android_app>() - 4usize];
    ["Offset of field: android_app::userData"]
        [::std::mem::offset_of!(android_app, userData) - 0usize];
    ["Offset of field: android_app::onAppCmd"]
        [::std::mem::offset_of!(android_app, onAppCmd) - 4usize];
    ["Offset of field: android_app::activity"]
        [::std::mem::offset_of!(android_app, activity) - 8usize];
    ["Offset of field: android_app::config"][::std::mem::offset_of!(android_app, config) - 12usize];
    ["Offset of field: android_app::savedState"]
        [::std::mem::offset_of!(android_app, savedState) - 16usize];
    ["Offset of field: android_app::savedStateSize"]
        [::std::mem::offset_of!(android_app, savedStateSize) - 20usize];
    ["Offset of field: android_app::looper"][::std::mem::offset_of!(android_app, looper) - 24usize];
    ["Offset of field: android_app::mainLooper"]
        [::std::mem::offset_of!(android_app, mainLooper) - 28usize];
    ["Offset of field: android_app::window"][::std::mem::offset_of!(android_app, window) - 32usize];
    ["Offset of field: android_app::contentRect"]
        [::std::mem::offset_of!(android_app, contentRect) - 36usize];
    ["Offset of field: android_app::softwareKeyboardVisible"]
        [::std::mem::offset_of!(android_app, softwareKeyboardVisible) - 52usize];
    ["Offset of field: android_app::editorAction"]
        [::std::mem::offset_of!(android_app, editorAction) - 56usize];
    ["Offset of field: android_app::pendingEditorAction"]
        [::std::mem::offset_of!(android_app, pendingEditorAction) - 60usize];
    ["Offset of field: android_app::activityState"]
        [::std::mem::offset_of!(android_app, activityState) - 64usize];
    ["Offset of field: android_app::destroyRequested"]
        [::std::mem::offset_of!(android_app, destroyRequested) - 68usize];
    ["Offset of field: android_app::inputBuffers"]
        [::std::mem::offset_of!(android_app, inputBuffers) - 72usize];
    ["Offset of field: android_app::currentInputBuffer"]
        [::std::mem::offset_of!(android_app, currentInputBuffer) - 152usize];
    ["Offset of field: android_app::textInputState"]
        [::std::mem::offset_of!(android_app, textInputState) - 156usize];
    ["Offset of field: android_app::mutex"][::std::mem::offset_of!(android_app, mutex) - 160usize];
    ["Offset of field: android_app::cond"][::std::mem::offset_of!(android_app, cond) - 164usize];
    ["Offset of field: android_app::msgread"]
        [::std::mem::offset_of!(android_app, msgread) - 168usize];
    ["Offset of field: android_app::msgwrite"]
        [::std::mem::offset_of!(android_app, msgwrite) - 172usize];
    ["Offset of field: android_app::thread"]
        [::std::mem::offset_of!(android_app, thread) - 176usize];
    ["Offset of field: android_app::cmdPollSource"]
        [::std::mem::offset_of!(android_app, cmdPollSource) - 180usize];
    ["Offset of field: android_app::running"]
        [::std::mem::offset_of!(android_app, running) - 192usize];
    ["Offset of field: android_app::stateSaved"]
        [::std::mem::offset_of!(android_app, stateSaved) - 196usize];
    ["Offset of field: android_app::destroyed"]
        [::std::mem::offset_of!(android_app, destroyed) - 200usize];
    ["Offset of field: android_app::redrawNeeded"]
        [::std::mem::offset_of!(android_app, redrawNeeded) - 204usize];
    ["Offset of field: android_app::pendingWindow"]
        [::std::mem::offset_of!(android_app, pendingWindow) - 208usize];
    ["Offset of field: android_app::pendingContentRect"]
        [::std::mem::offset_of!(android_app, pendingContentRect) - 212usize];
    ["Offset of field: android_app::keyEventFilter"]
        [::std::mem::offset_of!(android_app, keyEventFilter) - 228usize];
    ["Offset of field: android_app::motionEventFilter"]
        [::std::mem::offset_of!(android_app, motionEventFilter) - 232usize];
    ["Offset of field: android_app::inputAvailableWakeUp"]
        [::std::mem::offset_of!(android_app, inputAvailableWakeUp) - 236usize];
    ["Offset of field: android_app::inputSwapPending"]
        [::std::mem::offset_of!(android_app, inputSwapPending) - 237usize];
};
#[doc = " Looper data ID of commands coming from the app's main thread, which\n is returned as an identifier from ALooper_pollOnce().  The data for this\n identifier is a pointer to an android_poll_source structure.\n These can be retrieved and processed with android_app_read_cmd()\n and android_app_exec_cmd()."]
pub const NativeAppGlueLooperId_LOOPER_ID_MAIN: NativeAppGlueLooperId = 1;
#[doc = " Unused. Reserved for future use when usage of AInputQueue will be\n supported."]
pub const NativeAppGlueLooperId_LOOPER_ID_INPUT: NativeAppGlueLooperId = 2;
#[doc = " Start of user-defined ALooper identifiers."]
pub const NativeAppGlueLooperId_LOOPER_ID_USER: NativeAppGlueLooperId = 3;
#[doc = " Looper ID of commands coming from the app's main thread, an AInputQueue or\n user-defined sources."]
pub type NativeAppGlueLooperId = i8;
#[doc = " Unused. Reserved for future use when usage of AInputQueue will be\n supported."]
pub const NativeAppGlueAppCmd_UNUSED_APP_CMD_INPUT_CHANGED: NativeAppGlueAppCmd = 0;
#[doc = " Command from main thread: a new ANativeWindow is ready for use.  Upon\n receiving this command, android_app->window will contain the new window\n surface."]
pub const NativeAppGlueAppCmd_APP_CMD_INIT_WINDOW: NativeAppGlueAppCmd = 1;
#[doc = " Command from main thread: the existing ANativeWindow needs to be\n terminated.  Upon receiving this command, android_app->window still\n contains the existing window; after calling android_app_exec_cmd\n it will be set to NULL."]
pub const NativeAppGlueAppCmd_APP_CMD_TERM_WINDOW: NativeAppGlueAppCmd = 2;
#[doc = " Command from main thread: the current ANativeWindow has been resized.\n Please redraw with its new size."]
pub const NativeAppGlueAppCmd_APP_CMD_WINDOW_RESIZED: NativeAppGlueAppCmd = 3;
#[doc = " Command from main thread: the system needs that the current ANativeWindow\n be redrawn.  You should redraw the window before handing this to\n android_app_exec_cmd() in order to avoid transient drawing glitches."]
pub const NativeAppGlueAppCmd_APP_CMD_WINDOW_REDRAW_NEEDED: NativeAppGlueAppCmd = 4;
#[doc = " Command from main thread: the content area of the window has changed,\n such as from the soft input window being shown or hidden.  You can\n find the new content rect in android_app::contentRect."]
pub const NativeAppGlueAppCmd_APP_CMD_CONTENT_RECT_CHANGED: NativeAppGlueAppCmd = 5;
#[doc = " Command from main thread: the software keyboard was shown or hidden."]
pub const NativeAppGlueAppCmd_APP_CMD_SOFTWARE_KB_VIS_CHANGED: NativeAppGlueAppCmd = 6;
#[doc = " Command from main thread: the app's activity window has gained\n input focus."]
pub const NativeAppGlueAppCmd_APP_CMD_GAINED_FOCUS: NativeAppGlueAppCmd = 7;
#[doc = " Command from main thread: the app's activity window has lost\n input focus."]
pub const NativeAppGlueAppCmd_APP_CMD_LOST_FOCUS: NativeAppGlueAppCmd = 8;
#[doc = " Command from main thread: the current device configuration has changed."]
pub const NativeAppGlueAppCmd_APP_CMD_CONFIG_CHANGED: NativeAppGlueAppCmd = 9;
#[doc = " Command from main thread: the system is running low on memory.\n Try to reduce your memory use."]
pub const NativeAppGlueAppCmd_APP_CMD_LOW_MEMORY: NativeAppGlueAppCmd = 10;
#[doc = " Command from main thread: the app's activity has been started."]
pub const NativeAppGlueAppCmd_APP_CMD_START: NativeAppGlueAppCmd = 11;
#[doc = " Command from main thread: the app's activity has been resumed."]
pub const NativeAppGlueAppCmd_APP_CMD_RESUME: NativeAppGlueAppCmd = 12;
#[doc = " Command from main thread: the app should generate a new saved state\n for itself, to restore from later if needed.  If you have saved state,\n allocate it with malloc and place it in android_app.savedState with\n the size in android_app.savedStateSize.  The will be freed for you\n later."]
pub const NativeAppGlueAppCmd_APP_CMD_SAVE_STATE: NativeAppGlueAppCmd = 13;
#[doc = " Command from main thread: the app's activity has been paused."]
pub const NativeAppGlueAppCmd_APP_CMD_PAUSE: NativeAppGlueAppCmd = 14;
#[doc = " Command from main thread: the app's activity has been stopped."]
pub const NativeAppGlueAppCmd_APP_CMD_STOP: NativeAppGlueAppCmd = 15;
#[doc = " Command from main thread: the app's activity is being destroyed,\n and waiting for the app thread to clean up and exit before proceeding."]
pub const NativeAppGlueAppCmd_APP_CMD_DESTROY: NativeAppGlueAppCmd = 16;
#[doc = " Command from main thread: the app's insets have changed."]
pub const NativeAppGlueAppCmd_APP_CMD_WINDOW_INSETS_CHANGED: NativeAppGlueAppCmd = 17;
#[doc = " Commands passed from the application's main Java thread to the game's thread.\n\n Values from 0 to 127 are reserved for this library; values from -128 to -1\n can be used for custom user's events."]
pub type NativeAppGlueAppCmd = i8;
unsafe extern "C" {
    #[doc = " Call when ALooper_pollOnce() returns LOOPER_ID_MAIN, reading the next\n app command message."]
    pub fn android_app_read_cmd(android_app: *mut android_app) -> i8;
}
unsafe extern "C" {
    #[doc = " Call with the command returned by android_app_read_cmd() to do the\n initial pre-processing of the given command.  You can perform your own\n actions for the command after calling this function."]
    pub fn android_app_pre_exec_cmd(android_app: *mut android_app, cmd: i8);
}
unsafe extern "C" {
    #[doc = " Call with the command returned by android_app_read_cmd() to do the\n final post-processing of the given command.  You must have done your own\n actions for the command before calling this function."]
    pub fn android_app_post_exec_cmd(android_app: *mut android_app, cmd: i8);
}
unsafe extern "C" {
    #[doc = " Call this before processing input events to get the events buffer.\n The function returns NULL if there are no events to process."]
    pub fn android_app_swap_input_buffers(
        android_app: *mut android_app,
    ) -> *mut android_input_buffer;
}
unsafe extern "C" {
    #[doc = " Clear the array of motion events that were waiting to be handled, and release\n each of them.\n\n This method should be called after you have processed the motion events in\n your game loop. You should handle events at each iteration of your game loop."]
    pub fn android_app_clear_motion_events(inputBuffer: *mut android_input_buffer);
}
unsafe extern "C" {
    #[doc = " Clear the array of key events that were waiting to be handled, and release\n each of them.\n\n This method should be called after you have processed the key up events in\n your game loop. You should handle events at each iteration of your game loop."]
    pub fn android_app_clear_key_events(inputBuffer: *mut android_input_buffer);
}
unsafe extern "C" {
    #[doc = " This is a springboard into the Rust glue layer that wraps calling the\n main entry for the app itself."]
    pub fn _rust_glue_entry(app: *mut android_app);
}
unsafe extern "C" {
    #[doc = " Set the filter to use when processing key events.\n Any events for which the filter returns false will be ignored by\n android_native_app_glue. If filter is set to NULL, no filtering is done.\n\n The default key filter will filter out volume and camera button presses."]
    pub fn android_app_set_key_event_filter(
        app: *mut android_app,
        filter: android_key_event_filter,
    );
}
unsafe extern "C" {
    #[doc = " Set the filter to use when processing touch and motion events.\n Any events for which the filter returns false will be ignored by\n android_native_app_glue. If filter is set to NULL, no filtering is done.\n\n Note that the default motion event filter will only allow touchscreen events\n through, in order to mimic NativeActivity's behaviour, so for controller\n events to be passed to the app, set the filter to NULL."]
    pub fn android_app_set_motion_event_filter(
        app: *mut android_app,
        filter: android_motion_event_filter,
    );
}
unsafe extern "C" {
    #[doc = " You can send your custom events using the function below.\n\n Make sure your custom codes do not overlap with this library's ones.\n\n Values from 0 to 127 are reserved for this library; values from -128 to -1\n can be used for custom user's events.\n\n The function returns true if the write operation was successful."]
    pub fn android_app_write_cmd(android_app: *mut android_app, cmd: i8) -> bool;
}
unsafe extern "C" {
    #[doc = " Determines if a looper wake up was due to new input becoming available"]
    pub fn android_app_input_available_wake_up(app: *mut android_app) -> bool;
}
pub type __builtin_va_list = *mut ::std::os::raw::c_char;
